@@ -1,5 +1,5 @@
 /* pubkey.c  -	pubkey dispatcher
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,13 +24,14 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
+
+#include "g10lib.h"
 #include "util.h"
 #include "errors.h"
 #include "mpi.h"
 #include "cipher.h"
 #include "elgamal.h"
 #include "dsa.h"
-#include "rsa.h"
 #include "dynload.h"
 
 
@@ -151,58 +152,6 @@ setup_pubkey_table(void)
 	BUG();
     i++;
 
-    pubkey_table[i].algo = PUBKEY_ALGO_RSA;
-    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
-					 &pubkey_table[i].npkey,
-					 &pubkey_table[i].nskey,
-					 &pubkey_table[i].nenc,
-					 &pubkey_table[i].nsig,
-					 &pubkey_table[i].use );
-    pubkey_table[i].generate	     = rsa_generate;
-    pubkey_table[i].check_secret_key = rsa_check_secret_key;
-    pubkey_table[i].encrypt	     = rsa_encrypt;
-    pubkey_table[i].decrypt	     = rsa_decrypt;
-    pubkey_table[i].sign	     = rsa_sign;
-    pubkey_table[i].verify	     = rsa_verify;
-    pubkey_table[i].get_nbits	     = rsa_get_nbits;
-    if( !pubkey_table[i].name )
-	BUG();
-    i++;
-    pubkey_table[i].algo = PUBKEY_ALGO_RSA_E;
-    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
-					 &pubkey_table[i].npkey,
-					 &pubkey_table[i].nskey,
-					 &pubkey_table[i].nenc,
-					 &pubkey_table[i].nsig,
-					 &pubkey_table[i].use );
-    pubkey_table[i].generate	     = rsa_generate;
-    pubkey_table[i].check_secret_key = rsa_check_secret_key;
-    pubkey_table[i].encrypt	     = rsa_encrypt;
-    pubkey_table[i].decrypt	     = rsa_decrypt;
-    pubkey_table[i].sign	     = dummy_sign;
-    pubkey_table[i].verify	     = dummy_verify;
-    pubkey_table[i].get_nbits	     = rsa_get_nbits;
-    if( !pubkey_table[i].name )
-	BUG();
-    i++;
-    pubkey_table[i].algo = PUBKEY_ALGO_RSA_S;
-    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
-					 &pubkey_table[i].npkey,
-					 &pubkey_table[i].nskey,
-					 &pubkey_table[i].nenc,
-					 &pubkey_table[i].nsig,
-					 &pubkey_table[i].use );
-    pubkey_table[i].generate	     = rsa_generate;
-    pubkey_table[i].check_secret_key = rsa_check_secret_key;
-    pubkey_table[i].encrypt	     = dummy_encrypt;
-    pubkey_table[i].decrypt	     = dummy_decrypt;
-    pubkey_table[i].sign	     = rsa_sign;
-    pubkey_table[i].verify	     = rsa_verify;
-    pubkey_table[i].get_nbits	     = rsa_get_nbits;
-    if( !pubkey_table[i].name )
-	BUG();
-    i++;
-
     for( ; i < TABLE_SIZE; i++ )
 	pubkey_table[i].name = NULL;
 }
@@ -289,14 +238,14 @@ load_pubkey_modules(void)
  * Map a string to the pubkey algo
  */
 int
-string_to_pubkey_algo( const char *string )
+gcry_pk_map_name( const char *string )
 {
     int i;
     const char *s;
 
     do {
 	for(i=0; (s=pubkey_table[i].name); i++ )
-	    if( !ascii_strcasecmp( s, string ) )
+	    if( !stricmp( s, string ) )
 		return pubkey_table[i].algo;
     } while( load_pubkey_modules() );
     return 0;
@@ -307,7 +256,7 @@ string_to_pubkey_algo( const char *string )
  * Map a pubkey algo to a string
  */
 const char *
-pubkey_algo_to_string( int algo )
+gcry_pk_algo_name( int algo )
 {
     int i;
 
@@ -384,6 +333,8 @@ pubkey_get_npkey( int algo )
 	    if( pubkey_table[i].algo == algo )
 		return pubkey_table[i].npkey;
     } while( load_pubkey_modules() );
+    if( is_RSA(algo) )	  /* special hack, so that we are able to */
+	return 2;	  /* see the RSA keyids */
     return 0;
 }
 
@@ -616,3 +567,78 @@ pubkey_verify( int algo, MPI hash, MPI *data, MPI *pkey,
     return rc;
 }
 
+
+int
+gcry_pk_encrypt( GCRY_SEXP *result, GCRY_SEXP data, GCRY_SEXP pkey )
+{
+	/* ... */
+    return 0;
+}
+
+int
+gcry_pk_decrypt( GCRY_SEXP *result, GCRY_SEXP data, GCRY_SEXP skey )
+{
+	/* ... */
+    return 0;
+}
+
+int
+gcry_pk_sign( GCRY_SEXP *result, GCRY_SEXP data, GCRY_SEXP skey )
+{
+    GCRY_SEXP s;
+    /* get the secret key */
+    s = NULL; /*gcry_sexp_find_token( skey, "private-key", 0 );*/
+    if( !s )
+	return -1; /* no private key */
+	/* ... */
+    return 0;
+}
+
+int
+gcry_pk_verify( GCRY_SEXP *result, GCRY_SEXP data, GCRY_SEXP pkey )
+{
+	/* ... */
+    return 0;
+}
+
+
+
+
+/****************
+ * Return information about the given algorithm
+ * WHAT select the kind of information returned:
+ *  GCRYCTL_TEST_ALGO:
+ *	Returns 0 when the specified algorithm is available for use.
+ *	buffer and nbytes must be zero.
+ *
+ * On error the value -1 is returned and the error reason may be
+ * retrieved by gcry_errno().
+ * Note:  Because this function is in most caes used to return an
+ * integer value, we can make it easier for the caller to just look at
+ * the return value.  The caller will in all cases consult the value
+ * and thereby detecting whether a error occured or not (i.e. while checking
+ * the block size)
+ */
+int
+gcry_pk_algo_info( int algo, int what, void *buffer, size_t *nbytes)
+{
+    switch( what ) {
+      case GCRYCTL_TEST_ALGO:
+	if( buffer || nbytes ) {
+	    set_lasterr( GCRYERR_INV_ARG );
+	    return -1;
+	}
+	if( check_pubkey_algo( algo ) ) {
+	    set_lasterr( GCRYERR_INV_ALGO );
+	    return -1;
+	}
+	break;
+
+      default:
+	set_lasterr( GCRYERR_INV_OP );
+	return -1;
+    }
+    return 0;
+}
+
+
