@@ -1,5 +1,5 @@
 /* build-packet.c - assemble packets and write them
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -44,7 +44,6 @@ static u32 calc_plaintext( PKT_plaintext *pt );
 static int do_plaintext( IOBUF out, int ctb, PKT_plaintext *pt );
 static int do_encrypted( IOBUF out, int ctb, PKT_encrypted *ed );
 static int do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed );
-static int do_mdc( IOBUF out, PKT_mdc *mdc );
 static int do_compressed( IOBUF out, int ctb, PKT_compressed *cd );
 static int do_signature( IOBUF out, int ctb, PKT_signature *sig );
 static int do_onepass_sig( IOBUF out, int ctb, PKT_onepass_sig *ops );
@@ -68,31 +67,25 @@ int
 build_packet( IOBUF out, PACKET *pkt )
 {
     int new_ctb=0, rc=0, ctb;
-    int pkttype;
 
     if( DBG_PACKET )
 	log_debug("build_packet() type=%d\n", pkt->pkttype );
     assert( pkt->pkt.generic );
 
-    switch( (pkttype = pkt->pkttype) ) {
-      case PKT_OLD_COMMENT: pkttype = pkt->pkttype = PKT_COMMENT; break;
+    switch( pkt->pkttype ) {
+      case PKT_OLD_COMMENT: pkt->pkttype = PKT_COMMENT; break;
       case PKT_PLAINTEXT: new_ctb = pkt->pkt.plaintext->new_ctb; break;
       case PKT_ENCRYPTED:
       case PKT_ENCRYPTED_MDC: new_ctb = pkt->pkt.encrypted->new_ctb; break;
       case PKT_COMPRESSED:new_ctb = pkt->pkt.compressed->new_ctb; break;
-      case PKT_USER_ID:
-	    if( pkt->pkt.user_id->attrib_data )
-		pkttype = PKT_ATTRIBUTE;
-	    break;
       default: break;
     }
 
-    if( new_ctb || pkttype > 15 ) /* new format */
-	ctb = 0xc0 | (pkttype & 0x3f);
+    if( new_ctb || pkt->pkttype > 15 ) /* new format */
+	ctb = 0xc0 | (pkt->pkttype & 0x3f);
     else
-	ctb = 0x80 | ((pkttype & 15)<<2);
-    switch( pkttype ) {
-      case PKT_ATTRIBUTE:
+	ctb = 0x80 | ((pkt->pkttype & 15)<<2);
+    switch( pkt->pkttype ) {
       case PKT_USER_ID:
 	rc = do_user_id( out, ctb, pkt->pkt.user_id );
 	break;
@@ -122,9 +115,6 @@ build_packet( IOBUF out, PACKET *pkt )
       case PKT_ENCRYPTED_MDC:
 	rc = do_encrypted_mdc( out, ctb, pkt->pkt.encrypted );
 	break;
-      case PKT_MDC:
-	rc = do_mdc( out, pkt->pkt.mdc );
-	break;
       case PKT_COMPRESSED:
 	rc = do_compressed( out, ctb, pkt->pkt.compressed );
 	break;
@@ -135,7 +125,7 @@ build_packet( IOBUF out, PACKET *pkt )
 	rc = do_onepass_sig( out, ctb, pkt->pkt.onepass_sig );
 	break;
       case PKT_RING_TRUST:
-	break; /* ignore it (keyring.c does write it directly)*/
+	break; /* ignore it */
       default:
 	log_bug("invalid packet type in build_packet()\n");
 	break;
@@ -159,7 +149,6 @@ calc_packet_length( PACKET *pkt )
 	n = calc_plaintext( pkt->pkt.plaintext );
 	new_ctb = pkt->pkt.plaintext->new_ctb;
 	break;
-      case PKT_ATTRIBUTE:
       case PKT_USER_ID:
       case PKT_COMMENT:
       case PKT_PUBLIC_KEY:
@@ -207,16 +196,9 @@ do_comment( IOBUF out, int ctb, PKT_comment *rem )
 static int
 do_user_id( IOBUF out, int ctb, PKT_user_id *uid )
 {
-    if( uid->attrib_data ) {
-	write_header(out, ctb, uid->attrib_len);
-	if( iobuf_write( out, uid->attrib_data, uid->attrib_len ) )
-	    return G10ERR_WRITE_FILE;
-    }
-    else {
-	write_header(out, ctb, uid->len);
-	if( iobuf_write( out, uid->name, uid->len ) )
-	    return G10ERR_WRITE_FILE;
-    }
+    write_header(out, ctb, uid->len);
+    if( iobuf_write( out, uid->name, uid->len ) )
+	return G10ERR_WRITE_FILE;
     return 0;
 }
 
@@ -260,7 +242,7 @@ do_public_key( IOBUF out, int ctb, PKT_public_key *pk )
  * Make a hash value from the public key certificate
  */
 void
-hash_public_key( MD_HANDLE md, PKT_public_key *pk )
+hash_public_key( GCRY_MD_HD md, PKT_public_key *pk )
 {
     PACKET pkt;
     int rc = 0;
@@ -311,10 +293,10 @@ hash_public_key( MD_HANDLE md, PKT_public_key *pk )
 	    }
 	}
 	/* hash a header */
-	md_putc( md, 0x99 );
+	gcry_md_putc( md, 0x99 );
 	pktlen &= 0xffff; /* can't handle longer packets */
-	md_putc( md, pktlen >> 8 );
-	md_putc( md, pktlen & 0xff );
+	gcry_md_putc( md, pktlen >> 8 );
+	gcry_md_putc( md, pktlen & 0xff );
     }
     /* hash the packet body */
     while( (c=iobuf_get(a)) != -1 ) {
@@ -325,7 +307,7 @@ hash_public_key( MD_HANDLE md, PKT_public_key *pk )
 	    i=0;
 	}
       #endif
-	md_putc( md, c );
+	gcry_md_putc( md, c );
     }
   #if 0
     putc('\n', fp);
@@ -340,111 +322,71 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
 {
     int rc = 0;
     int i, nskey, npkey;
-    IOBUF a = iobuf_temp(); /* build in a self-enlarging buffer */
+    IOBUF a = iobuf_temp();
 
-    /* Write the version number - if none is specified, use 3 */
     if( !sk->version )
 	iobuf_put( a, 3 );
     else
 	iobuf_put( a, sk->version );
     write_32(a, sk->timestamp );
-
-    /* v3  needs the expiration time */
     if( sk->version < 4 ) {
 	u16 ndays;
 	if( sk->expiredate )
 	    ndays = (u16)((sk->expiredate - sk->timestamp) / 86400L);
 	else
 	    ndays = 0;
-	write_16(a, ndays);
+	write_16(a, 0 );
     }
-
     iobuf_put(a, sk->pubkey_algo );
-
-    /* get number of secret and public parameters.  They are held in
-       one array first the public ones, then the secret ones */
     nskey = pubkey_get_nskey( sk->pubkey_algo );
     npkey = pubkey_get_npkey( sk->pubkey_algo );
-
-    /* If we don't have any public parameters - which is the case if
-       we don't know the algorithm used - the parameters are stored as
-       one blob in a faked (opaque) MPI */
     if( !npkey ) {
 	write_fake_data( a, sk->skey[0] );
 	goto leave;
     }
     assert( npkey < nskey );
 
-    /* Writing the public parameters is easy */
     for(i=0; i < npkey; i++ )
 	mpi_write(a, sk->skey[i] );
-
-    /* build the header for protected (encrypted) secret parameters */
     if( sk->is_protected ) {
 	if( is_RSA(sk->pubkey_algo) && sk->version < 4
 				    && !sk->protect.s2k.mode ) {
-            /* the simple rfc1991 (v3) way */
 	    iobuf_put(a, sk->protect.algo );
 	    iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
 	}
 	else {
-          /* OpenPGP protection according to rfc2440 */
-	    iobuf_put(a, sk->protect.sha1chk? 0xfe : 0xff );
+	    iobuf_put(a, 0xff );
 	    iobuf_put(a, sk->protect.algo );
-	    if( sk->protect.s2k.mode >= 1000 ) {
-                /* These modes are not possible in OpenPGP, we use them
-                   to implement our extesnsions, 101 can ve views as a
-                   private/experimental extension (this is not
-                   specified in rfc2440 but the same scheme is used
-                   for all other algorithm identifiers) */
-		iobuf_put(a, 101 ); 
-		iobuf_put(a, sk->protect.s2k.hash_algo );
-		iobuf_write(a, "GNU", 3 );
-		iobuf_put(a, sk->protect.s2k.mode - 1000 );
-	    }
-	    else {
-		iobuf_put(a, sk->protect.s2k.mode );
-		iobuf_put(a, sk->protect.s2k.hash_algo );
-	    }
+	    iobuf_put(a, sk->protect.s2k.mode );
+	    iobuf_put(a, sk->protect.s2k.hash_algo );
 	    if( sk->protect.s2k.mode == 1
 		|| sk->protect.s2k.mode == 3 )
 		iobuf_write(a, sk->protect.s2k.salt, 8 );
 	    if( sk->protect.s2k.mode == 3 )
-		iobuf_put(a, sk->protect.s2k.count ); 
-
-            /* For out special mode 1001 we do not need an IV */
-	    if( sk->protect.s2k.mode != 1001 )
-		iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
+		iobuf_put(a, sk->protect.s2k.count );
+	    iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
 	}
     }
     else
 	iobuf_put(a, 0 );
-
-    if( sk->protect.s2k.mode == 1001 )
-        ; /* GnuPG extension - don't write a secret key at all */ 
-    else if( sk->is_protected && sk->version >= 4 ) {
-        /* The secret key is protected - write it out as it is */
+    if( sk->is_protected && sk->version >= 4 ) {
 	byte *p;
 	assert( mpi_is_opaque( sk->skey[npkey] ) );
 	p = mpi_get_opaque( sk->skey[npkey], &i );
 	iobuf_write(a, p, i );
     }
     else {
-        /* v3 way - same code for protected and non- protected key */
 	for(   ; i < nskey; i++ )
 	    mpi_write(a, sk->skey[i] );
 	write_16(a, sk->csum );
     }
 
   leave:
-    /* Build the header of the packet - which we must do after writing all
-       the other stuff, so that we know the length of the packet */
     write_header2(out, ctb, iobuf_get_temp_length(a), sk->hdrbytes, 1 );
-    /* And finally write it out the real stream */
     if( iobuf_write_temp( out, a ) )
 	rc = G10ERR_WRITE_FILE;
 
-    iobuf_close(a); /* close the remporary buffer */
+    iobuf_close(a);
     return rc;
 }
 
@@ -564,7 +506,7 @@ do_encrypted( IOBUF out, int ctb, PKT_encrypted *ed )
     int rc = 0;
     u32 n;
 
-    n = ed->len ? (ed->len + ed->extralen) : 0;
+    n = ed->len ? (ed->len + 10) : 0;
     write_header(out, ctb, n );
 
     /* This is all. The caller has to write the real data */
@@ -580,27 +522,16 @@ do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed )
 
     assert( ed->mdc_method );
 
-    n = ed->len ? (ed->len + ed->extralen) : 0;
+    n = ed->len ? (ed->len + 10) : 0;
     write_header(out, ctb, n );
     iobuf_put(out, 1 );  /* version */
+    iobuf_put(out, ed->mdc_method );
 
     /* This is all. The caller has to write the real data */
 
     return rc;
 }
 
-
-static int
-do_mdc( IOBUF out, PKT_mdc *mdc )
-{
-    /* This packet requires a fixed header encoding */
-    iobuf_put( out, 0xd3 ); /* packet ID and 1 byte length */
-    iobuf_put( out, 0x14 ); /* length = 20 */
-    if( iobuf_write( out, mdc->hash, sizeof(mdc->hash) ) )
-	return G10ERR_WRITE_FILE;
-    return 0;
-}
-
 static int
 do_compressed( IOBUF out, int ctb, PKT_compressed *cd )
 {
@@ -616,36 +547,36 @@ do_compressed( IOBUF out, int ctb, PKT_compressed *cd )
 }
 
 
+
 /****************
- * Delete all subpackets of type REQTYPE and return a bool whether a packet
- * was deleted.
+ * Find a subpacket of type REQTYPE in BUFFER and a return a pointer
+ * to the first byte of that subpacket data.
+ * And return the length of the packet in RET_N and the number of
+ * header bytes in RET_HLEN (length header and type byte).
  */
-int
-delete_sig_subpkt (subpktarea_t *area, sigsubpkttype_t reqtype )
+byte *
+find_subpkt( byte *buffer, sigsubpkttype_t reqtype,
+	     size_t *ret_hlen, size_t *ret_n )
 {
     int buflen;
     sigsubpkttype_t type;
-    byte *buffer, *bufstart;
+    byte *bufstart;
     size_t n;
-    size_t unused = 0;
-    int okay = 0;
 
-    if( !area )
-	return 0;
-    buflen = area->len;
-    buffer = area->data;
+    if( !buffer )
+	return NULL;
+    buflen = (*buffer << 8) | buffer[1];
+    buffer += 2;
     for(;;) {
-	if( !buflen ) {
-            okay = 1;
-            break;
-        }
+	if( !buflen )
+	    return NULL; /* end of packets; not found */
 	bufstart = buffer;
 	n = *buffer++; buflen--;
 	if( n == 255 ) {
 	    if( buflen < 4 )
 		break;
 	    n = (buffer[0] << 24) | (buffer[1] << 16)
-                | (buffer[2] << 8) | buffer[3];
+				  | (buffer[2] << 8) | buffer[3];
 	    buffer += 4;
 	    buflen -= 4;
 	}
@@ -658,139 +589,135 @@ delete_sig_subpkt (subpktarea_t *area, sigsubpkttype_t reqtype )
 	}
 	if( buflen < n )
 	    break;
-        
 	type = *buffer & 0x7f;
 	if( type == reqtype ) {
 	    buffer++;
-            buflen--;
 	    n--;
 	    if( n > buflen )
 		break;
-            buffer += n; /* point to next subpkt */
-            buflen -= n;
-            memmove (bufstart, buffer, buflen); /* shift */
-            unused +=  buffer - bufstart;
-            buffer = bufstart;
+	    if( ret_hlen )
+		*ret_hlen = buffer - bufstart;
+	    if( ret_n )
+		*ret_n = n;
+	    return buffer;
 	}
-        else {
-            buffer += n; buflen -=n;
-        }
+	buffer += n; buflen -=n;
     }
 
-    if (!okay)
-        log_error ("delete_subpkt: buffer shorter than subpacket\n");
-    assert (unused <= area->len);
-    area->len -= unused;
-    return !!unused;
+    log_error("find_subpkt: buffer shorter than subpacket\n");
+    return NULL;
 }
 
 
 /****************
- * Create or update a signature subpacket for SIG of TYPE.  This
- * functions knows where to put the data (hashed or unhashed).  The
- * function may move data from the unhashed part to the hashed one.
- * Note: All pointers into sig->[un]hashed (e.g. returned by
- * parse_sig_subpkt) are not valid after a call to this function.  The
- * data to put into the subpaket should be in a buffer with a length
- * of buflen. 
+ * Create or update a signature subpacket for SIG of TYPE.
+ * This functions knows where to put the data (hashed or unhashed).
+ * The function may move data from the unhased part to the hashed one.
+ * Note: All pointers into sig->[un]hashed are not valid after a call
+ * to this function.  The data to but into the subpaket should be
+ * in buffer with a length of buflen.
  */
 void
-build_sig_subpkt (PKT_signature *sig, sigsubpkttype_t type,
+build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 		  const byte *buffer, size_t buflen )
 {
-    byte *p;
-    int critical, hashed;
-    subpktarea_t *oldarea, *newarea;
-    size_t nlen, n, n0;
+
+    byte *data;
+    size_t hlen, dlen, nlen;
+    int found=0;
+    int critical, hashed, realloced;
+    size_t n, n0;
 
     critical = (type & SIGSUBPKT_FLAG_CRITICAL);
     type &= ~SIGSUBPKT_FLAG_CRITICAL;
 
-    switch(type)
-      {
-      case SIGSUBPKT_NOTATION:
-	/* we do allow multiple subpackets */
-	break;
-
-      default:
-	/* we don't allow multiple subpackets */
-	delete_sig_subpkt(sig->hashed,type);
-	delete_sig_subpkt(sig->unhashed,type);
-	break;
-      }
+    if( type == SIGSUBPKT_NOTATION )
+	; /* we allow multiple packets */
+    else if( (data = find_subpkt( sig->hashed_data, type, &hlen, &dlen )) )
+	found = 1;
+    else if( (data = find_subpkt( sig->unhashed_data, type, &hlen, &dlen )))
+	found = 2;
 
+    if( found )
+	log_bug("build_sig_packet: update nyi\n");
     if( (buflen+1) >= 8384 )
-	nlen = 5; /* write 5 byte length header */
+	nlen = 5;
     else if( (buflen+1) >= 192 )
-	nlen = 2; /* write 2 byte length header */
+	nlen = 2;
     else
-	nlen = 1; /* just a 1 byte length header */
+	nlen = 1;
 
     switch( type ) {
-      case SIGSUBPKT_ISSUER:
-      case SIGSUBPKT_PRIV_VERIFY_CACHE: /*(obsolete)*/
-        hashed = 0;
-        break;
-      default: 
-        hashed = 1;
-        break;
+      case SIGSUBPKT_SIG_CREATED:
+      case SIGSUBPKT_PRIV_ADD_SIG:
+      case SIGSUBPKT_PREF_SYM:
+      case SIGSUBPKT_PREF_HASH:
+      case SIGSUBPKT_PREF_COMPR:
+      case SIGSUBPKT_KS_FLAGS:
+      case SIGSUBPKT_KEY_EXPIRE:
+      case SIGSUBPKT_NOTATION:
+      case SIGSUBPKT_POLICY:
+	       hashed = 1; break;
+      default: hashed = 0; break;
+    }
+
+    if( hashed ) {
+	n0 = sig->hashed_data ? ((*sig->hashed_data << 8)
+				    | sig->hashed_data[1]) : 0;
+	n = n0 + nlen + 1 + buflen; /* length, type, buffer */
+	realloced = !!sig->hashed_data;
+	data = sig->hashed_data ? m_realloc( sig->hashed_data, n+2 )
+				: m_alloc( n+2 );
+    }
+    else {
+	n0 = sig->unhashed_data ? ((*sig->unhashed_data << 8)
+				      | sig->unhashed_data[1]) : 0;
+	n = n0 + nlen + 1 + buflen; /* length, type, buffer */
+	realloced = !!sig->unhashed_data;
+	data = sig->unhashed_data ? m_realloc( sig->unhashed_data, n+2 )
+				  : m_alloc( n+2 );
     }
 
     if( critical )
 	type |= SIGSUBPKT_FLAG_CRITICAL;
 
-    oldarea = hashed? sig->hashed : sig->unhashed;
-
-    /* Calculate new size of the area and allocate */
-    n0 = oldarea? oldarea->len : 0;
-    n = n0 + nlen + 1 + buflen; /* length, type, buffer */
-    if (oldarea && n <= oldarea->size) { /* fits into the unused space */
-        newarea = oldarea;
-        /*log_debug ("updating area for type %d\n", type );*/
+    data[0] = (n >> 8) & 0xff;
+    data[1] = n & 0xff;
+    if( nlen == 5 ) {
+	data[n0+2] = 255;
+	data[n0+3] = (buflen+1) >> 24;
+	data[n0+4] = (buflen+1) >> 16;
+	data[n0+5] = (buflen+1) >>  8;
+	data[n0+6] = (buflen+1);
+	data[n0+7] = type;
+	memcpy(data+n0+8, buffer, buflen );
     }
-    else if (oldarea) {
-        newarea = m_realloc (oldarea, sizeof (*newarea) + n - 1);
-        newarea->size = n;
-        /*log_debug ("reallocating area for type %d\n", type );*/
+    else if( nlen == 2 ) {
+	data[n0+2] = (buflen+1-192) / 256 + 192;
+	data[n0+3] = (buflen+1-192) & 256;
+	data[n0+4] = type;
+	memcpy(data+n0+5, buffer, buflen );
     }
     else {
-        newarea = m_alloc (sizeof (*newarea) + n - 1);
-        newarea->size = n;
-        /*log_debug ("allocating area for type %d\n", type );*/
+	data[n0+2] = buflen+1;
+	data[n0+3] = type;
+	memcpy(data+n0+4, buffer, buflen );
     }
-    newarea->len = n;
 
-    p = newarea->data + n0;
-    if (nlen == 5) {
-	*p++ = 255;
-	*p++ = (buflen+1) >> 24;
-	*p++ = (buflen+1) >> 16;
-	*p++ = (buflen+1) >>  8;
-	*p++ = (buflen+1);
-	*p++ = type;
-	memcpy (p, buffer, buflen);
-    }
-    else if (nlen == 2) {
-	*p++ = (buflen+1-192) / 256 + 192;
-	*p++ = (buflen+1-192) % 256;
-	*p++ = type;
-	memcpy (p, buffer, buflen);
+    if( hashed ) {
+	if( !realloced )
+	    m_free(sig->hashed_data);
+	sig->hashed_data = data;
     }
     else {
-	*p++ = buflen+1;
-	*p++ = type;
-	memcpy (p, buffer, buflen);
+	if( !realloced )
+	    m_free(sig->unhashed_data);
+	sig->unhashed_data = data;
     }
-
-    if (hashed) 
-	sig->hashed = newarea;
-    else
-	sig->unhashed = newarea;
 }
 
 /****************
  * Put all the required stuff from SIG into subpackets of sig.
- * Hmmm, should we delete those subpackets which are in a wrong area?
  */
 void
 build_sig_subpkt_from_sig( PKT_signature *sig )
@@ -816,70 +743,8 @@ build_sig_subpkt_from_sig( PKT_signature *sig )
     buf[2] = (u >>  8) & 0xff;
     buf[3] = u & 0xff;
     build_sig_subpkt( sig, SIGSUBPKT_SIG_CREATED, buf, 4 );
-
-    if(sig->expiredate)
-      {
-	u = sig->expiredate-sig->timestamp;
-	buf[0] = (u >> 24) & 0xff;
-	buf[1] = (u >> 16) & 0xff;
-	buf[2] = (u >>  8) & 0xff;
-	buf[3] = u & 0xff;
-
-	/* Mark this CRITICAL, so if any implementation doesn't
-           understand sigs that can expire, it'll just disregard this
-           sig altogether. */
-
-	build_sig_subpkt( sig, SIGSUBPKT_SIG_EXPIRE | SIGSUBPKT_FLAG_CRITICAL,
-			  buf, 4 );
-      }
 }
 
-void
-build_attribute_subpkt(PKT_user_id *uid,byte type,
-		       const void *buf,int buflen,
-		       const void *header,int headerlen)
-{
-  byte *attrib;
-  int idx;
-
-  if(1+headerlen+buflen>8383)
-    idx=5;
-  else if(1+headerlen+buflen>191)
-    idx=2;
-  else
-    idx=1;
-
-  /* realloc uid->attrib_data to the right size */
-
-  uid->attrib_data=m_realloc(uid->attrib_data,
-			     uid->attrib_len+idx+1+headerlen+buflen);
-
-  attrib=&uid->attrib_data[uid->attrib_len];
-
-  if(idx==5)
-    {
-      attrib[0]=255;
-      attrib[1]=(1+headerlen+buflen) >> 24;
-      attrib[2]=(1+headerlen+buflen) >> 16;
-      attrib[3]=(1+headerlen+buflen) >> 8;
-      attrib[4]=1+headerlen+buflen;
-    }
-  else if(idx==2)
-    {
-      attrib[0]=(1+headerlen+buflen-192) / 256 + 192;
-      attrib[1]=(1+headerlen+buflen-192) % 256;
-    }
-  else
-    attrib[0]=1+headerlen+buflen; /* Good luck finding a JPEG this small! */
-
-  attrib[idx++]=type;
-
-  /* Tack on our data at the end */
-
-  memcpy(&attrib[idx],header,headerlen);
-  memcpy(&attrib[idx+headerlen],buf,buflen);
-  uid->attrib_len+=idx+headerlen+buflen;
-}
 
 static int
 do_signature( IOBUF out, int ctb, PKT_signature *sig )
@@ -907,14 +772,16 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
 	/* timestamp and keyid must have been packed into the
 	 * subpackets prior to the call of this function, because
 	 * these subpackets are hashed */
-	nn = sig->hashed? sig->hashed->len : 0;
+	nn = sig->hashed_data?((sig->hashed_data[0]<<8)
+				|sig->hashed_data[1])	:0;
 	write_16(a, nn);
 	if( nn )
-	    iobuf_write( a, sig->hashed->data, nn );
-	nn = sig->unhashed? sig->unhashed->len : 0;
+	    iobuf_write( a, sig->hashed_data+2, nn );
+	nn = sig->unhashed_data?((sig->unhashed_data[0]<<8)
+				  |sig->unhashed_data[1])   :0;
 	write_16(a, nn);
 	if( nn )
-	    iobuf_write( a, sig->unhashed->data, nn );
+	    iobuf_write( a, sig->unhashed_data+2, nn );
     }
     iobuf_put(a, sig->digest_start[0] );
     iobuf_put(a, sig->digest_start[1] );
