@@ -1,5 +1,5 @@
 /* cipher.c - En-/De-ciphering filter
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998,1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <assert.h>
 
+#include <gcrypt.h>
 #include "errors.h"
 #include "iobuf.h"
 #include "memory.h"
@@ -33,7 +34,6 @@
 #include "packet.h"
 #include "options.h"
 #include "main.h"
-#include "status.h"
 
 
 #define MIN_PARTIAL_SIZE 512
@@ -45,67 +45,59 @@ write_header( cipher_filter_context_t *cfx, IOBUF a )
     PACKET pkt;
     PKT_encrypted ed;
     byte temp[18];
-    unsigned blocksize;
+    int blocksize;
     unsigned nprefix;
-    int use_mdc;
-
-    blocksize = cipher_get_blocksize( cfx->dek->algo );
-    if( blocksize < 8 || blocksize > 16 )
-	log_fatal("unsupported blocksize %u\n", blocksize );
-
-    use_mdc = cfx->dek->use_mdc;
-
-    if( blocksize != 8 )
-	use_mdc = 1;  /* Hack: enable it for all modern ciphers */
-    /* Note: We should remove this hack as soon as a reasonable number of keys
-       are carrying the MDC flag.  But always keep the hack for conventional
-       encryption */
-
-    if (opt.force_mdc)
-        use_mdc = 1;
-        
-    if( opt.rfc2440 || opt.rfc1991 || opt.disable_mdc )
-	use_mdc = 0;  /* override - rfc2440 does not know about MDC */
+    int use_mdc = opt.force_mdc;
+    int rc;
 
     memset( &ed, 0, sizeof ed );
     ed.len = cfx->datalen;
-    ed.extralen = blocksize+2;
     ed.new_ctb = !ed.len && !opt.rfc1991;
     if( use_mdc ) {
 	ed.mdc_method = DIGEST_ALGO_SHA1;
-	cfx->mdc_hash = md_open( DIGEST_ALGO_SHA1, 0 );
-	if ( DBG_HASHING )
-	    md_start_debug( cfx->mdc_hash, "creatmdc" );
+	cfx->mdc_hash = gcry_md_open( DIGEST_ALGO_SHA1, 0 );
+	/*should we check the function works, or is it better to provide
+	  a flag which makes the function die itself ?? FIXME */
+	/*md_start_debug( cfx->mdc_hash, "mdccreat" );*/
     }
-
-    {
-        char buf[20];
-        
-        sprintf (buf, "%d %d", ed.mdc_method, cfx->dek->algo);
-        write_status_text (STATUS_BEGIN_ENCRYPTION, buf);
-    }
-
     init_packet( &pkt );
     pkt.pkttype = use_mdc? PKT_ENCRYPTED_MDC : PKT_ENCRYPTED;
     pkt.pkt.encrypted = &ed;
     if( build_packet( a, &pkt ))
 	log_bug("build_packet(ENCR_DATA) failed\n");
+    blocksize = gcry_cipher_get_algo_blklen( cfx->dek->algo );
+    if( blocksize < 8 || blocksize > 16 )
+	log_fatal("unsupported blocksize %d\n", blocksize );
     nprefix = blocksize;
     randomize_buffer( temp, nprefix, 1 );
     temp[nprefix] = temp[nprefix-2];
     temp[nprefix+1] = temp[nprefix-1];
     print_cipher_algo_note( cfx->dek->algo );
-    cfx->cipher_hd = cipher_open( cfx->dek->algo,
-				  use_mdc? CIPHER_MODE_CFB
-					 : CIPHER_MODE_AUTO_CFB, 1 );
+    if( !(cfx->cipher_hd = gcry_cipher_open( cfx->dek->algo,
+				       GCRY_CIPHER_MODE_CFB,
+				       GCRY_CIPHER_SECURE
+				       | (cfx->dek->algo >= 100 ?
+					     0 : GCRY_CIPHER_ENABLE_SYNC)))
+				     ) {
+	/* we should never get an error here cause we already checked, that
+	 * the algorithm is available. */
+	BUG();
+    }
+
 /*   log_hexdump( "thekey", cfx->dek->key, cfx->dek->keylen );*/
-    cipher_setkey( cfx->cipher_hd, cfx->dek->key, cfx->dek->keylen );
-    cipher_setiv( cfx->cipher_hd, NULL, 0 );
+    rc = gcry_cipher_setkey( cfx->cipher_hd, cfx->dek->key, cfx->dek->keylen );
+    if( !rc )
+	rc = gcry_cipher_setiv( cfx->cipher_hd, NULL, 0 );
+    if( rc )
+	log_fatal("set key or IV failed: %s\n", gcry_strerror(rc) );
 /*  log_hexdump( "prefix", temp, nprefix+2 ); */
-    if( cfx->mdc_hash ) /* hash the "IV" */
-	md_write( cfx->mdc_hash, temp, nprefix+2 );
-    cipher_encrypt( cfx->cipher_hd, temp, temp, nprefix+2);
-    cipher_sync( cfx->cipher_hd );
+    if( cfx->mdc_hash )
+	gcry_md_write( cfx->mdc_hash, temp, nprefix+2 );
+    rc = gcry_cipher_encrypt( cfx->cipher_hd, temp, nprefix+2, NULL, 0 );
+    if( !rc )
+	rc = gcry_cipher_sync( cfx->cipher_hd );
+    if( rc )
+	log_fatal("encrypt failed: %s\n", gcry_strerror(rc) );
     iobuf_write(a, temp, nprefix+2);
     cfx->header=1;
 }
@@ -132,33 +124,26 @@ cipher_filter( void *opaque, int control,
 	    write_header( cfx, a );
 	}
 	if( cfx->mdc_hash )
-	    md_write( cfx->mdc_hash, buf, size );
-	cipher_encrypt( cfx->cipher_hd, buf, buf, size);
+	    gcry_md_write( cfx->mdc_hash, buf, size );
+	rc = gcry_cipher_encrypt( cfx->cipher_hd, buf, size, NULL, 0);
+	if( rc )
+	    log_fatal("encrypt failed: %s\n", gcry_strerror(rc) );
 	if( iobuf_write( a, buf, size ) )
 	    rc = G10ERR_WRITE_FILE;
     }
     else if( control == IOBUFCTRL_FREE ) {
 	if( cfx->mdc_hash ) {
 	    byte *hash;
-	    int hashlen = md_digest_length( md_get_algo( cfx->mdc_hash ) );
-	    byte temp[22];
-
-	    assert( hashlen == 20 );
-	    /* we must hash the prefix of the MDC packet here */
-	    temp[0] = 0xd3;
-	    temp[1] = 0x14;
-	    md_putc( cfx->mdc_hash, temp[0] );
-	    md_putc( cfx->mdc_hash, temp[1] );
-
-	    md_final( cfx->mdc_hash );
-	    hash = md_read( cfx->mdc_hash, 0 );
-	    memcpy(temp+2, hash, 20);
-	    cipher_encrypt( cfx->cipher_hd, temp, temp, 22 );
-	    md_close( cfx->mdc_hash ); cfx->mdc_hash = NULL;
-	    if( iobuf_write( a, temp, 22 ) )
-		log_error("writing MDC packet failed\n" );
+	    int hashlen = gcry_md_get_algo_dlen( gcry_md_get_algo( cfx->mdc_hash ) );
+	    hash = gcry_md_read( cfx->mdc_hash, 0 );
+	    rc = gcry_cipher_encrypt( cfx->cipher_hd, hash, hashlen, NULL, 0 );
+	    if( rc )
+		log_fatal("encrypt failed: %s\n", gcry_strerror(rc) );
+	    if( iobuf_write( a, hash, hashlen ) )
+		rc = G10ERR_WRITE_FILE;
+	    gcry_md_close( cfx->mdc_hash ); cfx->mdc_hash = NULL;
 	}
-	cipher_close(cfx->cipher_hd);
+	gcry_cipher_close(cfx->cipher_hd);
     }
     else if( control == IOBUFCTRL_DESC ) {
 	*(char**)buf = "cipher_filter";
