@@ -1,5 +1,5 @@
 /* compress.c - compress filter
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -73,11 +73,7 @@ do_compress( compress_filter_context_t *zfx, z_stream *zs, int flush, IOBUF a )
     unsigned n;
 
     do {
-#ifndef __riscos__
 	zs->next_out = zfx->outbuf;
-#else /* __riscos__ */
-	zs->next_out = (Bytef *) zfx->outbuf;
-#endif /* __riscos__ */
 	zs->avail_out = zfx->outbufsize;
 	if( DBG_FILTER )
 	    log_debug("enter deflate: avail_in=%u, avail_out=%u, flush=%d\n",
@@ -147,11 +143,7 @@ do_uncompress( compress_filter_context_t *zfx, z_stream *zs,
 	if( zs->avail_in < zfx->inbufsize && refill ) {
 	    n = zs->avail_in;
 	    if( !n )
-#ifndef __riscos__
 		zs->next_in = zfx->inbuf;
-#else /* __riscos__ */
-		zs->next_in = (Bytef *) zfx->inbuf;
-#endif /* __riscos__ */
 	    count = zfx->inbufsize - n;
 	    nread = iobuf_read( a, zfx->inbuf + n, count );
 	    if( nread == -1 ) nread = 0;
@@ -209,11 +201,7 @@ compress_filter( void *opaque, int control,
 	    zfx->status = 1;
 	}
 
-#ifndef __riscos__
 	zs->next_out = buf;
-#else /* __riscos__ */
-	zs->next_out = (Bytef *) buf;
-#endif /* __riscos__ */
 	zs->avail_out = size;
 	zfx->outbufsize = size; /* needed only for calculation */
 	rc = do_uncompress( zfx, zs, a, ret_len );
@@ -238,11 +226,7 @@ compress_filter( void *opaque, int control,
 	    zfx->status = 2;
 	}
 
-#ifndef __riscos__
 	zs->next_in = buf;
-#else /* __riscos__ */
-	zs->next_in = (Bytef *) buf;
-#endif /* __riscos__ */
 	zs->avail_in = size;
 	rc = do_compress( zfx, zs, Z_NO_FLUSH, a );
     }
@@ -254,11 +238,7 @@ compress_filter( void *opaque, int control,
 	    m_free(zfx->outbuf); zfx->outbuf = NULL;
 	}
 	else if( zfx->status == 2 ) {
-#ifndef __riscos__
 	    zs->next_in = buf;
-#else /* __riscos__ */
-	    zs->next_in = (Bytef *) buf;
-#endif /* __riscos__ */
 	    zs->avail_in = 0;
 	    do_compress( zfx, zs, Z_FINISH, a );
 	    deflateEnd(zs);
@@ -266,21 +246,12 @@ compress_filter( void *opaque, int control,
 	    zfx->opaque = NULL;
 	    m_free(zfx->outbuf); zfx->outbuf = NULL;
 	}
-        if (zfx->release)
-          zfx->release (zfx);
     }
     else if( control == IOBUFCTRL_DESC )
 	*(char**)buf = "compress_filter";
     return rc;
 }
 
-
-static void
-release_context (compress_filter_context_t *ctx)
-{
-  m_free (ctx);
-}
-
 /****************
  * Handle a compressed packet
  */
@@ -288,19 +259,26 @@ int
 handle_compressed( void *procctx, PKT_compressed *cd,
 		   int (*callback)(IOBUF, void *), void *passthru )
 {
-    compress_filter_context_t *cfx;
+    compress_filter_context_t cfx;
     int rc;
 
+    memset( &cfx, 0, sizeof cfx );
     if( cd->algorithm < 1 || cd->algorithm > 2	)
 	return G10ERR_COMPR_ALGO;
-    cfx = m_alloc_clear (sizeof *cfx);
-    cfx->algo = cd->algorithm;
-    cfx->release = release_context;
-    iobuf_push_filter( cd->buf, compress_filter, cfx );
+    cfx.algo = cd->algorithm;
+
+    iobuf_push_filter( cd->buf, compress_filter, &cfx );
     if( callback )
 	rc = callback(cd->buf, passthru );
     else
 	rc = proc_packets(procctx, cd->buf);
+  #if 0
+    iobuf_pop_filter( cd->buf, compress_filter, &cfx );
+    if( cd->len )
+	iobuf_set_limit( cd->buf, 0 ); /* disable the readlimit */
+    else
+	iobuf_clear_eof( cd->buf );
+  #endif
     cd->buf = NULL;
     return rc;
 }
