@@ -1,5 +1,5 @@
 /* encr-data.c -  process an encrypted data packet
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <gcrypt.h>
 #include "util.h"
 #include "memory.h"
 #include "packet.h"
@@ -32,14 +33,14 @@
 #include "i18n.h"
 
 
-static int mdc_decode_filter( void *opaque, int control, IOBUF a,
-					      byte *buf, size_t *ret_len);
 static int decode_filter( void *opaque, int control, IOBUF a,
 					byte *buf, size_t *ret_len);
+static int mdc_decode_filter( void *opaque, int control, IOBUF a,
+					byte *buf, size_t *ret_len);
 
 typedef struct {
-    CIPHER_HANDLE cipher_hd;
-    MD_HANDLE mdc_hash;
+    GCRY_CIPHER_HD cipher_hd;
+    GCRY_MD_HD mdc_hash;
     char defer[20];
     int  defer_filled;
     int  eof_seen;
@@ -56,49 +57,58 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
     byte *p;
     int rc=0, c, i;
     byte temp[32];
-    unsigned blocksize;
+    int blocksize;
     unsigned nprefix;
 
     memset( &dfx, 0, sizeof dfx );
-    if( opt.verbose ) {
-	const char *s = cipher_algo_to_string( dek->algo );
-	if( s )
-	    log_info(_("%s encrypted data\n"), s );
-	else
+    if( gcry_cipher_test_algo( dek->algo ) ) {
+	if( opt.verbose )
 	    log_info(_("encrypted with unknown algorithm %d\n"), dek->algo );
-    }
-    if( (rc=check_cipher_algo(dek->algo)) )
+	rc = G10ERR_CIPHER_ALGO;
 	goto leave;
-    blocksize = cipher_get_blocksize(dek->algo);
-    if( !blocksize || blocksize > 16 )
+    }
+    if( opt.verbose )
+	log_info(_("%s encrypted data\n"), gcry_cipher_algo_name( dek->algo ) );
+
+    blocksize = gcry_cipher_get_algo_blklen( dek->algo );
+    if( blocksize < 1 || blocksize > 16 )
 	log_fatal("unsupported blocksize %u\n", blocksize );
     nprefix = blocksize;
     if( ed->len && ed->len < (nprefix+2) )
 	BUG();
 
     if( ed->mdc_method ) {
-	dfx.mdc_hash = md_open( ed->mdc_method, 0 );
-	if ( DBG_HASHING )
-	    md_start_debug(dfx.mdc_hash, "checkmdc");
+	dfx.mdc_hash = gcry_md_open( ed->mdc_method, 0 );
+	if( !dfx.mdc_hash )
+	    BUG();
     }
-    dfx.cipher_hd = cipher_open( dek->algo,
-				 ed->mdc_method? CIPHER_MODE_CFB
-					       : CIPHER_MODE_AUTO_CFB, 1 );
-/* log_hexdump( "thekey", dek->key, dek->keylen );*/
-    rc = cipher_setkey( dfx.cipher_hd, dek->key, dek->keylen );
-    if( rc == G10ERR_WEAK_KEY )
-	log_info(_("WARNING: message was encrypted with "
-		    "a weak key in the symmetric cipher.\n"));
-    else if( rc ) {
-	log_error("key setup failed: %s\n", g10_errstr(rc) );
-	goto leave;
-    }
-    if (!ed->buf) {
-        log_error(_("problem handling encrypted packet\n"));
-        goto leave;
+    if( !(dfx.cipher_hd = gcry_cipher_open( dek->algo,
+				      GCRY_CIPHER_MODE_CFB,
+				      GCRY_CIPHER_SECURE
+				      | (dek->algo >= 100 ?
+					   0 : GCRY_CIPHER_ENABLE_SYNC) ))
+				    ) {
+	/* we should never get an error here cause we already checked, that
+	 * the algorithm is available. What about a flag to let the function
+	 * die in this case? */
+	BUG();
     }
 
-    cipher_setiv( dfx.cipher_hd, NULL, 0 );
+
+/* log_hexdump( "thekey", dek->key, dek->keylen );*/
+    rc = gcry_cipher_setkey( dfx.cipher_hd, dek->key, dek->keylen );
+    if( rc == GCRYERR_WEAK_KEY ) {
+	log_info(_("WARNING: message was encrypted with "
+		    "a weak key in the symmetric cipher.\n"));
+	rc = 0;
+    }
+    else if( rc ) {
+	log_error("key setup failed: %s\n", gcry_strerror(rc) );
+	rc = map_gcry_rc(rc);
+	goto leave;
+    }
+
+    gcry_cipher_setiv( dfx.cipher_hd, NULL, 0 );
 
     if( ed->len ) {
 	for(i=0; i < (nprefix+2) && ed->len; i++, ed->len-- ) {
@@ -115,47 +125,39 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
 	    else
 		temp[i] = c;
     }
-    cipher_decrypt( dfx.cipher_hd, temp, temp, nprefix+2);
-    cipher_sync( dfx.cipher_hd );
+    gcry_cipher_decrypt( dfx.cipher_hd, temp, nprefix+2, NULL, 0 );
+    if( dfx.mdc_hash )
+	gcry_md_write( dfx.mdc_hash, temp, nprefix+2 );
+    gcry_cipher_sync( dfx.cipher_hd );
     p = temp;
 /* log_hexdump( "prefix", temp, nprefix+2 ); */
     if( p[nprefix-2] != p[nprefix] || p[nprefix-1] != p[nprefix+1] ) {
 	rc = G10ERR_BAD_KEY;
 	goto leave;
     }
-
-    if( dfx.mdc_hash )
-	md_write( dfx.mdc_hash, temp, nprefix+2 );
-
     if( ed->mdc_method )
 	iobuf_push_filter( ed->buf, mdc_decode_filter, &dfx );
     else
 	iobuf_push_filter( ed->buf, decode_filter, &dfx );
-
-    proc_packets( procctx, ed->buf );
+    proc_packets( procctx, ed->buf);
     ed->buf = NULL;
     if( ed->mdc_method && dfx.eof_seen == 2 )
 	rc = G10ERR_INVALID_PACKET;
     else if( ed->mdc_method ) { /* check the mdc */
-	int datalen = md_digest_length( ed->mdc_method );
-
-	cipher_decrypt( dfx.cipher_hd, dfx.defer, dfx.defer, 20);
-	md_final( dfx.mdc_hash );
+	int datalen = gcry_md_get_algo_dlen( ed->mdc_method );
 	if( datalen != 20
-	    || memcmp(md_read( dfx.mdc_hash, 0 ), dfx.defer, datalen) )
+	    || memcmp(gcry_md_read( dfx.mdc_hash, 0 ), dfx.defer, datalen) )
 	    rc = G10ERR_BAD_SIGN;
-	/*log_hexdump("MDC calculated:", md_read( dfx.mdc_hash, 0), datalen);*/
-	/*log_hexdump("MDC message   :", dfx.defer, 20);*/
+	log_hexdump("MDC calculated:", gcry_md_read( dfx.mdc_hash, 0), datalen);
+	log_hexdump("MDC message   :", dfx.defer, 20);
     }
-
   leave:
-    cipher_close(dfx.cipher_hd);
-    md_close( dfx.mdc_hash );
+    gcry_cipher_close(dfx.cipher_hd);
+    gcry_md_close( dfx.mdc_hash );
     return rc;
 }
 
 
-
 /* I think we should merge this with cipher_filter */
 static int
 mdc_decode_filter( void *opaque, int control, IOBUF a,
@@ -182,14 +184,11 @@ mdc_decode_filter( void *opaque, int control, IOBUF a,
 	}
 	if( n == 40 ) {
 	    /* we have enough stuff - flush the deferred stuff */
-	    /* (we have asserted that the buffer is large enough) */
-	    if( !dfx->defer_filled ) { /* the first time */
+	    /* (we have asserted that the buffer is large enough */
+	    if( !dfx->defer_filled ) /* the first time */
 		memcpy(buf, buf+20, 20 );
-		n = 20;
-	    }
-	    else {
+	    else
 		memcpy(buf, dfx->defer, 20 );
-	    }
 	    /* now fill up */
 	    for(; n < size; n++ ) {
 		if( (c = iobuf_get(a)) == -1 )
@@ -203,7 +202,7 @@ mdc_decode_filter( void *opaque, int control, IOBUF a,
 	    dfx->defer_filled = 1;
 	}
 	else if( !dfx->defer_filled ) { /* eof seen buf empty defer */
-	    /* this is bad because there is an incomplete hash */
+	    /* this is very bad because there is an incomplete hash */
 	    n -= 20;
 	    memcpy(buf, buf+20, n );
 	    dfx->eof_seen = 2; /* eof with incomplete hash */
@@ -216,8 +215,8 @@ mdc_decode_filter( void *opaque, int control, IOBUF a,
 	}
 
 	if( n ) {
-	    cipher_decrypt( dfx->cipher_hd, buf, buf, n);
-	    md_write( dfx->mdc_hash, buf, n );
+	    gcry_cipher_decrypt( dfx->cipher_hd, buf, n, NULL, 0);
+	    gcry_md_write( dfx->mdc_hash, buf, n );
 	}
 	else {
 	    assert( dfx->eof_seen );
@@ -243,7 +242,7 @@ decode_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len)
 	n = iobuf_read( a, buf, size );
 	if( n == -1 ) n = 0;
 	if( n )
-	    cipher_decrypt( fc->cipher_hd, buf, buf, n);
+	    gcry_cipher_decrypt( fc->cipher_hd, buf, n, NULL, 0);
 	else
 	    rc = -1; /* eof */
 	*ret_len = n;
