@@ -1,5 +1,5 @@
 /* filter.h
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -20,12 +20,15 @@
 #ifndef G10_FILTER_H
 #define G10_FILTER_H
 
-#include "types.h"
-#include "cipher.h"
+#include <gcrypt.h>
+
+#include "basicdefs.h"
+
+
 
 typedef struct {
-    MD_HANDLE md;      /* catch all */
-    MD_HANDLE md2;     /* if we want to calculate an alternate hash */
+    GCRY_MD_HD md;	/* catch all */
+    GCRY_MD_HD md2;	/* if we want to calculate an alternate hash */
     size_t maxbuf_size;
 } md_filter_context_t;
 
@@ -35,9 +38,6 @@ typedef struct {
     int only_keyblocks;     /* skip all headers but ".... key block" */
     const char *hdrlines;   /* write these headerlines */
 
-    /* these fileds must be initialized to zero */
-    int no_openpgp_data;    /* output flag: "No valid OpenPGP data found" */
-
     /* the following fields must be initialized to zero */
     int inp_checked;	    /* set if the input has been checked */
     int inp_bypass;	    /* set if the input is not armored */
@@ -59,14 +59,10 @@ typedef struct {
     u32 crc;
 
     int status; 	    /* an internal state flag */
-    int cancel;
     int any_data;	    /* any valid armored data seen */
     int pending_lf;	    /* used together with faked */
 } armor_filter_context_t;
 
-struct unarmor_pump_s;
-typedef struct unarmor_pump_s *UnarmorPump;
-
 
 typedef struct {
     int status;
@@ -83,14 +79,19 @@ typedef struct {
 typedef struct {
     DEK *dek;
     u32 datalen;
-    CIPHER_HANDLE cipher_hd;
+    GCRY_CIPHER_HD cipher_hd;
     int header;
-    MD_HANDLE mdc_hash;
-    byte enchash[20];
-    int create_mdc; /* flag will be set by the cipher filter */
+    GCRY_MD_HD mdc_hash;
 } cipher_filter_context_t;
 
 
+typedef struct {
+    int header_okay;
+    PK_LIST pk_list;
+    cipher_filter_context_t cfx;
+} encrypt_filter_context_t;
+
+
 
 typedef struct {
     byte *buffer;	    /* malloced buffer */
@@ -100,7 +101,7 @@ typedef struct {
     int truncated;	    /* number of truncated lines */
     int not_dash_escaped;
     int escape_from;
-    MD_HANDLE md;
+    GCRY_MD_HD md;
     int pending_lf;
     int pending_esc;
 } text_filter_context_t;
@@ -116,9 +117,6 @@ void free_md_filter_context( md_filter_context_t *mfx );
 int use_armor_filter( IOBUF a );
 int armor_filter( void *opaque, int control,
 		  IOBUF chain, byte *buf, size_t *ret_len);
-UnarmorPump unarmor_pump_new (void);
-void        unarmor_pump_release (UnarmorPump x);
-int         unarmor_pump (UnarmorPump x, int c);
 
 /*-- compress.c --*/
 int compress_filter( void *opaque, int control,
@@ -131,7 +129,7 @@ int cipher_filter( void *opaque, int control,
 /*-- textfilter.c --*/
 int text_filter( void *opaque, int control,
 		 IOBUF chain, byte *buf, size_t *ret_len);
-int copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
+int copy_clearsig_text( IOBUF out, IOBUF inp, GCRY_MD_HD md,
 			  int escape_dash, int escape_from, int pgp2mode );
 
 
