@@ -1,5 +1,5 @@
 /* import.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -35,10 +35,9 @@
 #include "main.h"
 #include "i18n.h"
 #include "status.h"
-#include "keyserver-internal.h"
 
-struct stats_s {
-    ulong count;
+
+static struct {
     ulong no_user_id;
     ulong imported;
     ulong imported_rsa;
@@ -50,23 +49,17 @@ struct stats_s {
     ulong secret_read;
     ulong secret_imported;
     ulong secret_dups;
-    ulong skipped_new_keys;
-};
+} stats;
 
 
-static int import( IOBUF inp, int fast, const char* fname,
-                   struct stats_s *stats );
+static int import( IOBUF inp, int fast, const char* fname );
 static int read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root );
-static void revocation_present(KBNODE keyblock);
-static void remove_bad_stuff (KBNODE keyblock);
-static int import_one( const char *fname, KBNODE keyblock, int fast,
-                       struct stats_s *stats);
-static int import_secret_one( const char *fname, KBNODE keyblock,
-                              struct stats_s *stats );
-static int import_revoke_cert( const char *fname, KBNODE node,
-                               struct stats_s *stats);
+static int import_one( const char *fname, KBNODE keyblock, int fast );
+static int import_secret_one( const char *fname, KBNODE keyblock );
+static int import_revoke_cert( const char *fname, KBNODE node );
 static int chk_self_sigs( const char *fname, KBNODE keyblock,
 			  PKT_public_key *pk, u32 *keyid );
+static void mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid );
 static int delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid );
 static int merge_blocks( const char *fname, KBNODE keyblock_orig,
 			 KBNODE keyblock, u32 *keyid,
@@ -81,18 +74,6 @@ static int merge_keysigs( KBNODE dst, KBNODE src, int *n_sigs,
 			     const char *fname, u32 *keyid );
 
 
-void *
-import_new_stats_handle (void)
-{
-    return m_alloc_clear ( sizeof (struct stats_s) );
-}
-
-void
-import_release_stats_handle (void *p)
-{
-    m_free (p);
-}
-
 /****************
  * Import the public keys from the given filename. Input may be armored.
  * This function rejects all keys which are not validly self signed on at
@@ -124,66 +105,42 @@ import_release_stats_handle (void *p)
  *  Key revocation certificates have special handling.
  *
  */
-void
-import_keys( char **fnames, int nnames, int fast, void *stats_handle )
-{
-    int i;
-    struct stats_s *stats = stats_handle;
-
-    if (!stats)
-        stats = import_new_stats_handle ();
-
-    if( !fnames && !nnames )
-	nnames = 1;  /* Ohh what a ugly hack to jump into the loop */
-
-    for(i=0; i < nnames; i++ ) {
-	const char *fname = fnames? fnames[i] : NULL;
-	IOBUF inp = iobuf_open(fname);
-	if( !fname )
-	    fname = "[stdin]";
-	if( !inp )
-	    log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
-	else {
-	    int rc = import( inp, fast, fname, stats );
-	    iobuf_close(inp);
-	    if( rc )
-		log_error("import from `%s' failed: %s\n", fname,
-							   g10_errstr(rc) );
-	}
-	if( !fname )
-	    break;
-    }
-    if (!stats_handle) {
-        import_print_stats (stats);
-        import_release_stats_handle (stats);
-    }
-
-}
-
 int
-import_keys_stream( IOBUF inp, int fast, void *stats_handle )
+import_keys( const char *fname, int fast )
 {
-    int rc = 0;
-    struct stats_s *stats = stats_handle;
+    IOBUF inp = NULL;
+    int rc;
 
-    if (!stats)
-        stats = import_new_stats_handle ();
-
-    rc = import( inp, fast, "[stream]", stats);
-    if (!stats_handle) {
-        import_print_stats (stats);
-        import_release_stats_handle (stats);
+    inp = iobuf_open(fname);
+    if( !fname )
+	fname = "[stdin]";
+    if( !inp ) {
+	log_error(_("can't open `%s': %s\n"), fname, strerror(errno) );
+	return G10ERR_OPEN_FILE;
     }
 
+    rc = import( inp, fast, fname );
+
+    iobuf_close(inp);
     return rc;
 }
 
+int
+import_keys_stream( IOBUF inp, int fast )
+{
+    return import( inp, fast, "[stream]" );
+}
+
 static int
-import( IOBUF inp, int fast, const char* fname, struct stats_s *stats )
+import( IOBUF inp, int fast, const char* fname )
 {
     PACKET *pending_pkt = NULL;
     KBNODE keyblock;
     int rc = 0;
+    ulong count=0;
+
+    /* fixme: don't use static variables */
+    memset( &stats, 0, sizeof( stats ) );
 
     getkey_disable_caches();
 
@@ -194,14 +151,13 @@ import( IOBUF inp, int fast, const char* fname, struct stats_s *stats )
     }
 
     while( !(rc = read_block( inp, &pending_pkt, &keyblock) )) {
-        remove_bad_stuff (keyblock);
 	if( keyblock->pkt->pkttype == PKT_PUBLIC_KEY )
-	    rc = import_one( fname, keyblock, fast, stats );
-	else if( keyblock->pkt->pkttype == PKT_SECRET_KEY ) 
-                rc = import_secret_one( fname, keyblock, stats );
+	    rc = import_one( fname, keyblock, fast );
+	else if( keyblock->pkt->pkttype == PKT_SECRET_KEY )
+	    rc = import_secret_one( fname, keyblock );
 	else if( keyblock->pkt->pkttype == PKT_SIGNATURE
 		 && keyblock->pkt->pkt.signature->sig_class == 0x20 )
-	    rc = import_revoke_cert( fname, keyblock, stats );
+	    rc = import_revoke_cert( fname, keyblock );
 	else {
 	    log_info( _("skipping block of type %d\n"),
 					    keyblock->pkt->pkttype );
@@ -209,72 +165,61 @@ import( IOBUF inp, int fast, const char* fname, struct stats_s *stats )
 	release_kbnode(keyblock);
 	if( rc )
 	    break;
-	if( !(++stats->count % 100) && !opt.quiet )
-	    log_info(_("%lu keys so far processed\n"), stats->count );
+	if( !(++count % 100) && !opt.quiet )
+	    log_info(_("%lu keys so far processed\n"), count );
     }
     if( rc == -1 )
 	rc = 0;
     else if( rc && rc != G10ERR_INV_KEYRING )
 	log_error( _("error reading `%s': %s\n"), fname, g10_errstr(rc));
 
-    return rc;
-}
-
-
-void
-import_print_stats (void *hd)
-{
-    struct stats_s *stats = hd;
-
     if( !opt.quiet ) {
-	log_info(_("Total number processed: %lu\n"), stats->count );
-	if( stats->skipped_new_keys )
-	    log_info(_("      skipped new keys: %lu\n"),
-						stats->skipped_new_keys );
-	if( stats->no_user_id )
-	    log_info(_("          w/o user IDs: %lu\n"), stats->no_user_id );
-	if( stats->imported || stats->imported_rsa ) {
-	    log_info(_("              imported: %lu"), stats->imported );
-	    if( stats->imported_rsa )
-		fprintf(stderr, "  (RSA: %lu)", stats->imported_rsa );
+	log_info(_("Total number processed: %lu\n"), count );
+	if( stats.no_user_id )
+	    log_info(_("          w/o user IDs: %lu\n"), stats.no_user_id );
+	if( stats.imported || stats.imported_rsa ) {
+	    log_info(_("              imported: %lu"), stats.imported );
+	    if( stats.imported_rsa )
+		fprintf(stderr, "  (RSA: %lu)", stats.imported_rsa );
 	    putc('\n', stderr);
 	}
-	if( stats->unchanged )
-	    log_info(_("             unchanged: %lu\n"), stats->unchanged );
-	if( stats->n_uids )
-	    log_info(_("          new user IDs: %lu\n"), stats->n_uids );
-	if( stats->n_subk )
-	    log_info(_("           new subkeys: %lu\n"), stats->n_subk );
-	if( stats->n_sigs )
-	    log_info(_("        new signatures: %lu\n"), stats->n_sigs );
-	if( stats->n_revoc )
-	    log_info(_("   new key revocations: %lu\n"), stats->n_revoc );
-	if( stats->secret_read )
-	    log_info(_("      secret keys read: %lu\n"), stats->secret_read );
-	if( stats->secret_imported )
-	    log_info(_("  secret keys imported: %lu\n"), stats->secret_imported );
-	if( stats->secret_dups )
-	    log_info(_(" secret keys unchanged: %lu\n"), stats->secret_dups );
+	if( stats.unchanged )
+	    log_info(_("             unchanged: %lu\n"), stats.unchanged );
+	if( stats.n_uids )
+	    log_info(_("          new user IDs: %lu\n"), stats.n_uids );
+	if( stats.n_subk )
+	    log_info(_("           new subkeys: %lu\n"), stats.n_subk );
+	if( stats.n_sigs )
+	    log_info(_("        new signatures: %lu\n"), stats.n_sigs );
+	if( stats.n_revoc )
+	    log_info(_("   new key revocations: %lu\n"), stats.n_revoc );
+	if( stats.secret_read )
+	    log_info(_("      secret keys read: %lu\n"), stats.secret_read );
+	if( stats.secret_imported )
+	    log_info(_("  secret keys imported: %lu\n"), stats.secret_imported );
+	if( stats.secret_dups )
+	    log_info(_(" secret keys unchanged: %lu\n"), stats.secret_dups );
     }
 
     if( is_status_enabled() ) {
-	char buf[13*20];
-	sprintf(buf, "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
-		stats->count,
-		stats->no_user_id,
-		stats->imported,
-		stats->imported_rsa,
-		stats->unchanged,
-		stats->n_uids,
-		stats->n_subk,
-		stats->n_sigs,
-		stats->n_revoc,
-		stats->secret_read,
-		stats->secret_imported,
-		stats->secret_dups,
-		stats->skipped_new_keys );
+	char buf[12*16];
+	sprintf(buf, "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
+		count,
+		stats.no_user_id,
+		stats.imported,
+		stats.imported_rsa,
+		stats.unchanged,
+		stats.n_uids,
+		stats.n_subk,
+		stats.n_sigs,
+		stats.n_revoc,
+		stats.secret_read,
+		stats.secret_imported,
+		stats.secret_dups);
 	write_status_text( STATUS_IMPORT_RES, buf );
     }
+
+    return rc;
 }
 
 
@@ -340,11 +285,6 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
 	    init_packet(pkt);
 	    break;
 
-          case PKT_RING_TRUST:
-            /* skip those packets */
-	    free_packet( pkt );
-	    init_packet(pkt);
-            break;
 
 	  case PKT_PUBLIC_KEY:
 	  case PKT_SECRET_KEY:
@@ -380,22 +320,6 @@ read_block( IOBUF a, PACKET **pending_pkt, KBNODE *ret_root )
 }
 
 
-static void
-remove_bad_stuff (KBNODE keyblock)
-{
-    KBNODE node;
-
-    for (node=keyblock; node; node = node->next ) {
-        if( node->pkt->pkttype == PKT_SIGNATURE ) {
-            /* delete the subpackets we used to use for the
-               verification cache */
-            delete_sig_subpkt (node->pkt->pkt.signature->unhashed,
-                               SIGSUBPKT_PRIV_VERIFY_CACHE);
-        }
-    }
-}
-
-
 /****************
  * Try to import one keyblock.	Return an error only in serious cases, but
  * never for an invalid keyblock.  It uses log_error to increase the
@@ -403,13 +327,13 @@ remove_bad_stuff (KBNODE keyblock)
  * which called g10.
  */
 static int
-import_one( const char *fname, KBNODE keyblock, int fast,
-            struct stats_s *stats )
+import_one( const char *fname, KBNODE keyblock, int fast )
 {
     PKT_public_key *pk;
     PKT_public_key *pk_orig;
     KBNODE node, uidnode;
     KBNODE keyblock_orig = NULL;
+    KBPOS kbpos;
     u32 keyid[2];
     int rc = 0;
     int new_key = 0;
@@ -430,8 +354,8 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 		  pubkey_letter( pk->pubkey_algo ),
 		  (ulong)keyid[1], datestr_from_pk(pk) );
 	if( uidnode )
-	    print_utf8_string( stderr, uidnode->pkt->pkt.user_id->name,
-				       uidnode->pkt->pkt.user_id->len );
+	    print_string( stderr, uidnode->pkt->pkt.user_id->name,
+				  uidnode->pkt->pkt.user_id->len, 0 );
 	putc('\n', stderr);
     }
     if( !uidnode ) {
@@ -444,18 +368,8 @@ import_one( const char *fname, KBNODE keyblock, int fast,
     if( rc )
 	return rc== -1? 0:rc;
 
-    /* If we allow such a thing, mark unsigned uids as valid */
     if( opt.allow_non_selfsigned_uid )
-      for( node=keyblock; node; node = node->next )
-	if( node->pkt->pkttype == PKT_USER_ID && !(node->flag & 1) )
-	  {
-	    char *user=utf8_to_native(node->pkt->pkt.user_id->name,
-				      node->pkt->pkt.user_id->len,0);
-	    node->flag |= 1;
-	    log_info( _("key %08lX: accepted non self-signed user ID '%s'\n"),
-		      (ulong)keyid[1],user);
-	    m_free(user);
-	  }
+	mark_non_selfsigned_uids_valid( keyblock, keyid );
 
     if( !delete_inv_parts( fname, keyblock, keyid ) ) {
 	if( !opt.quiet ) {
@@ -463,43 +377,34 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 							(ulong)keyid[1]);
 	    log_info(_("this may be caused by a missing self-signature\n"));
 	}
-	stats->no_user_id++;
+	stats.no_user_id++;
 	return 0;
     }
 
+
     /* do we have this key already in one of our pubrings ? */
     pk_orig = m_alloc_clear( sizeof *pk_orig );
     rc = get_pubkey( pk_orig, keyid );
-    if( rc && rc != G10ERR_NO_PUBKEY && rc != G10ERR_UNU_PUBKEY ) {
+    if( rc && rc != G10ERR_NO_PUBKEY ) {
 	log_error( _("key %08lX: public key not found: %s\n"),
 				(ulong)keyid[1], g10_errstr(rc));
     }
-    else if ( rc && opt.merge_only ) {
-	if( opt.verbose )
-	    log_info( _("key %08lX: new key - skipped\n"), (ulong)keyid[1] );
-	rc = 0;
-	fast = 1; /* so that we don't get into the trustdb update */
-	stats->skipped_new_keys++;
-    }
     else if( rc ) { /* insert this key */
-        KEYDB_HANDLE hd = keydb_new (0);
-
-        rc = keydb_locate_writable (hd, NULL);
-	if (rc) {
-	    log_error (_("no writable keyring found: %s\n"), g10_errstr (rc));
-            keydb_release (hd);
+	/* get default resource */
+	if( get_keyblock_handle( NULL, 0, &kbpos ) ) {
+	    log_error(_("no default public keyring\n"));
 	    return G10ERR_GENERAL;
 	}
 	if( opt.verbose > 1 )
-	    log_info (_("writing to `%s'\n"), keydb_get_resource_name (hd) );
-	rc = keydb_insert_keyblock (hd, keyblock );
-        if (rc)
-	   log_error (_("error writing keyring `%s': %s\n"),
-		       keydb_get_resource_name (hd), g10_errstr(rc));
-	else
-	   revalidation_mark ();
-        keydb_release (hd);
-
+	    log_info( _("writing to `%s'\n"),
+				keyblock_resource_name(&kbpos) );
+	if( (rc=lock_keyblock( &kbpos )) )
+	   log_error(_("can't lock keyring `%s': %s\n"),
+		       keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	else if( (rc=insert_keyblock( &kbpos, keyblock )) )
+	   log_error( _("error writing keyring `%s': %s\n"),
+		       keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	unlock_keyblock( &kbpos );
 	/* we are ready */
 	if( !opt.quiet )
 	    log_info( _("key %08lX: public key imported\n"), (ulong)keyid[1]);
@@ -508,16 +413,14 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 	    write_status_text( STATUS_IMPORTED, us );
 	    m_free(us);
 	}
-	stats->imported++;
+	stats.imported++;
 	if( is_RSA( pk->pubkey_algo ) )
-	    stats->imported_rsa++;
+	    stats.imported_rsa++;
 	new_key = 1;
     }
     else { /* merge */
-        KEYDB_HANDLE hd;
 	int n_uids, n_sigs, n_subk;
 
-
 	/* Compare the original against the new key; just to be sure nothing
 	 * weird is going on */
 	if( cmp_public_keys( pk_orig, pk ) ) {
@@ -528,27 +431,16 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 	}
 
 	/* now read the original keyblock */
-        hd = keydb_new (0);
-        {
-            byte afp[MAX_FINGERPRINT_LEN];
-            size_t an;
-
-            fingerprint_from_pk (pk_orig, afp, &an);
-            while (an < MAX_FINGERPRINT_LEN) 
-                afp[an++] = 0;
-            rc = keydb_search_fpr (hd, afp);
-        }
+	rc = find_keyblock_bypk( &kbpos, pk_orig );
 	if( rc ) {
-	    log_error (_("key %08lX: can't locate original keyblock: %s\n"),
+	    log_error( _("key %08lX: can't locate original keyblock: %s\n"),
 				     (ulong)keyid[1], g10_errstr(rc));
-            keydb_release (hd);
 	    goto leave;
 	}
-	rc = keydb_get_keyblock (hd, &keyblock_orig );
-	if (rc) {
-	    log_error (_("key %08lX: can't read original keyblock: %s\n"),
+	rc = read_keyblock( &kbpos, &keyblock_orig );
+	if( rc ) {
+	    log_error( _("key %08lX: can't read original keyblock: %s\n"),
 					    (ulong)keyid[1], g10_errstr(rc));
-            keydb_release (hd);
 	    goto leave;
 	}
 
@@ -559,20 +451,18 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 	n_uids = n_sigs = n_subk = 0;
 	rc = merge_blocks( fname, keyblock_orig, keyblock,
 				keyid, &n_uids, &n_sigs, &n_subk );
-	if( rc ) {
-            keydb_release (hd);
+	if( rc )
 	    goto leave;
-        }
 	if( n_uids || n_sigs || n_subk ) {
 	    mod_key = 1;
 	    /* keyblock_orig has been updated; write */
-	    rc = keydb_update_keyblock (hd, keyblock_orig);
-            if (rc)
-		log_error (_("error writing keyring `%s': %s\n"),
-			     keydb_get_resource_name (hd), g10_errstr(rc) );
-	    else
-	      revalidation_mark ();
-
+	    if( (rc=lock_keyblock( &kbpos )) )
+	       log_error( _("can't lock keyring `%s': %s\n"),
+			  keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	    else if( (rc=update_keyblock( &kbpos, keyblock_orig )) )
+		log_error( _("error writing keyring `%s': %s\n"),
+			     keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	    unlock_keyblock( &kbpos );
 	    /* we are ready */
 	    if( !opt.quiet ) {
 		if( n_uids == 1 )
@@ -595,40 +485,48 @@ import_one( const char *fname, KBNODE keyblock, int fast,
 					     (ulong)keyid[1], n_subk );
 	    }
 
-	    stats->n_uids +=n_uids;
-	    stats->n_sigs +=n_sigs;
-	    stats->n_subk +=n_subk;
+	    stats.n_uids +=n_uids;
+	    stats.n_sigs +=n_sigs;
+	    stats.n_subk +=n_subk;
 	}
 	else {
 	    if( !opt.quiet )
 		log_info( _("key %08lX: not changed\n"), (ulong)keyid[1] );
-	    stats->unchanged++;
+	    stats.unchanged++;
 	}
-        keydb_release (hd); hd = NULL;
+    }
+    if( !rc && !fast ) {
+	rc = query_trust_record( new_key? pk : pk_orig );
+	if( rc && rc != -1 )
+	    log_error("trustdb error: %s\n", g10_errstr(rc) );
+	else if( rc == -1 ) { /* not found trustdb */
+	    rc = insert_trust_record( new_key? keyblock : keyblock_orig );
+	    if( rc )
+		log_error("key %08lX: trustdb insert failed: %s\n",
+					(ulong)keyid[1], g10_errstr(rc) );
+	}
+	else if( mod_key )
+	    rc = update_trust_record( keyblock_orig, 1, NULL );
+	else
+	    rc = clear_trust_checked_flag( new_key? pk : pk_orig );
     }
 
   leave:
     release_kbnode( keyblock_orig );
     free_public_key( pk_orig );
-
-    revocation_present(keyblock);
-
     return rc;
 }
 
 
 /****************
  * Ditto for secret keys.  Handling is simpler than for public keys.
- * We allow secret key importing only when allow is true, this is so
- * that a secret key can not be imported accidently and thereby tampering
- * with the trust calculation.
  */
 static int
-import_secret_one( const char *fname, KBNODE keyblock, 
-                   struct stats_s *stats)
+import_secret_one( const char *fname, KBNODE keyblock )
 {
     PKT_secret_key *sk;
     KBNODE node, uidnode;
+    KBPOS kbpos;
     u32 keyid[2];
     int rc = 0;
 
@@ -647,12 +545,11 @@ import_secret_one( const char *fname, KBNODE keyblock,
 		  pubkey_letter( sk->pubkey_algo ),
 		  (ulong)keyid[1], datestr_from_sk(sk) );
 	if( uidnode )
-	    print_utf8_string( stderr, uidnode->pkt->pkt.user_id->name,
-				       uidnode->pkt->pkt.user_id->len );
+	    print_string( stderr, uidnode->pkt->pkt.user_id->name,
+				  uidnode->pkt->pkt.user_id->len, 0 );
 	putc('\n', stderr);
     }
-    stats->secret_read++;
-
+    stats.secret_read++;
     if( !uidnode ) {
 	log_error( _("key %08lX: no user ID\n"), (ulong)keyid[1]);
 	return 0;
@@ -662,30 +559,28 @@ import_secret_one( const char *fname, KBNODE keyblock,
 
     /* do we have this key already in one of our secrings ? */
     rc = seckey_available( keyid );
-    if( rc == G10ERR_NO_SECKEY && !opt.merge_only ) { /* simply insert this key */
-        KEYDB_HANDLE hd = keydb_new (1);
-
+    if( rc == G10ERR_NO_SECKEY ) { /* simply insert this key */
 	/* get default resource */
-        rc = keydb_locate_writable (hd, NULL);
-	if (rc) {
-	    log_error (_("no default secret keyring: %s\n"), g10_errstr (rc));
-            keydb_release (hd);
+	if( get_keyblock_handle( NULL, 1, &kbpos ) ) {
+	    log_error("no default secret keyring\n");
 	    return G10ERR_GENERAL;
 	}
-	rc = keydb_insert_keyblock (hd, keyblock );
-        if (rc)
-	    log_error (_("error writing keyring `%s': %s\n"),
-                       keydb_get_resource_name (hd), g10_errstr(rc) );
-        keydb_release (hd);
+	if( (rc=lock_keyblock( &kbpos )) )
+	    log_error( _("can't lock keyring `%s': %s\n"),
+			 keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	else if( (rc=insert_keyblock( &kbpos, keyblock )) )
+	    log_error( _("error writing keyring `%s': %s\n"),
+		      keyblock_resource_name(&kbpos), g10_errstr(rc) );
+	unlock_keyblock( &kbpos );
 	/* we are ready */
 	if( !opt.quiet )
 	    log_info( _("key %08lX: secret key imported\n"), (ulong)keyid[1]);
-	stats->secret_imported++;
+	stats.secret_imported++;
     }
     else if( !rc ) { /* we can't merge secret keys */
 	log_error( _("key %08lX: already in secret keyring\n"),
 							(ulong)keyid[1]);
-	stats->secret_dups++;
+	stats.secret_dups++;
     }
     else
 	log_error( _("key %08lX: secret key not found: %s\n"),
@@ -699,11 +594,11 @@ import_secret_one( const char *fname, KBNODE keyblock,
  * Import a revocation certificate; this is a single signature packet.
  */
 static int
-import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
+import_revoke_cert( const char *fname, KBNODE node )
 {
     PKT_public_key *pk=NULL;
     KBNODE onode, keyblock = NULL;
-    KEYDB_HANDLE hd = NULL;
+    KBPOS kbpos;
     u32 keyid[2];
     int rc = 0;
 
@@ -729,25 +624,16 @@ import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
     }
 
     /* read the original keyblock */
-    hd = keydb_new (0);
-    {
-        byte afp[MAX_FINGERPRINT_LEN];
-        size_t an;
-        
-        fingerprint_from_pk (pk, afp, &an);
-        while (an < MAX_FINGERPRINT_LEN) 
-            afp[an++] = 0;
-        rc = keydb_search_fpr (hd, afp);
-    }
-    if (rc) {
-	log_error (_("key %08lX: can't locate original keyblock: %s\n"),
-                   (ulong)keyid[1], g10_errstr(rc));
+    rc = find_keyblock_bypk( &kbpos, pk );
+    if( rc ) {
+	log_error( _("key %08lX: can't locate original keyblock: %s\n"),
+					(ulong)keyid[1], g10_errstr(rc));
 	goto leave;
     }
-    rc = keydb_get_keyblock (hd, &keyblock );
-    if (rc) {
-	log_error (_("key %08lX: can't read original keyblock: %s\n"),
-                   (ulong)keyid[1], g10_errstr(rc));
+    rc = read_keyblock( &kbpos, &keyblock );
+    if( rc ) {
+	log_error( _("key %08lX: can't read original keyblock: %s\n"),
+					(ulong)keyid[1], g10_errstr(rc));
 	goto leave;
     }
 
@@ -759,7 +645,6 @@ import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
     if( rc ) {
 	log_error( _("key %08lX: invalid revocation certificate"
 		  ": %s - rejected\n"), (ulong)keyid[1], g10_errstr(rc));
-	goto leave;
     }
 
 
@@ -781,20 +666,20 @@ import_revoke_cert( const char *fname, KBNODE node, struct stats_s *stats )
     insert_kbnode( keyblock, clone_kbnode(node), 0 );
 
     /* and write the keyblock back */
-    rc = keydb_update_keyblock (hd, keyblock );
-    if (rc)
-	log_error (_("error writing keyring `%s': %s\n"),
-                   keydb_get_resource_name (hd), g10_errstr(rc) );
-    keydb_release (hd); hd = NULL;
+    if( (rc=lock_keyblock( &kbpos )) )
+	log_error( _("can't lock keyring `%s': %s\n"),
+		   keyblock_resource_name(&kbpos), g10_errstr(rc) );
+    else if( (rc=update_keyblock( &kbpos, keyblock )) )
+	log_error( _("error writing keyring `%s': %s\n"),
+		    keyblock_resource_name(&kbpos), g10_errstr(rc) );
+    unlock_keyblock( &kbpos );
     /* we are ready */
     if( !opt.quiet )
 	log_info( _("key %08lX: revocation certificate imported\n"),
 					(ulong)keyid[1]);
-    stats->n_revoc++;
-    revalidation_mark ();
+    stats.n_revoc++;
 
   leave:
-    keydb_release (hd);
     release_kbnode( keyblock );
     free_public_key( pk );
     return rc;
@@ -827,25 +712,16 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 					    (ulong)keyid[1]);
 		    return -1;	/* the complete keyblock is invalid */
 		}
+		rc = check_key_signature( keyblock, n, NULL);
+		if( rc ) {
+		    log_info( rc == G10ERR_PUBKEY_ALGO ?
+			 _("key %08lX: unsupported public key algorithm\n"):
+			 _("key %08lX: invalid self-signature\n"),
+				     (ulong)keyid[1]);
 
-		/* If it hasn't been marked valid yet, keep trying */
-		if(!(unode->flag&1)) {
-		  rc = check_key_signature( keyblock, n, NULL);
-		  if( rc )
-		    {
-		      char *p=utf8_to_native(unode->pkt->pkt.user_id->name,
-				      strlen(unode->pkt->pkt.user_id->name),0);
-		      log_info( rc == G10ERR_PUBKEY_ALGO ?
-				_("key %08lX: unsupported public key "
-				  "algorithm on user id \"%s\"\n"):
-				_("key %08lX: invalid self-signature "
-				  "on user id \"%s\"\n"),
-				(ulong)keyid[1],p);
-		      m_free(p);
-		    }
-		  else
-		    unode->flag |= 1; /* mark that signature checked */
+		    unode->flag |= 2; /* mark as invalid */
 		}
+		unode->flag |= 1; /* mark that signature checked */
 	    }
 	    else if( sig->sig_class == 0x18 ) {
 		KBNODE knode = find_prev_kbnode( keyblock,
@@ -860,17 +736,16 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		    n->flag |= 4; /* delete this */
 		}
 		else {
-		  /* If it hasn't been marked valid yet, keep trying */
-		  if(!(knode->flag&1)) {
 		    rc = check_key_signature( keyblock, n, NULL);
-		    if( rc )
+		    if( rc ) {
 			log_info(  rc == G10ERR_PUBKEY_ALGO ?
 			   _("key %08lX: unsupported public key algorithm\n"):
 			   _("key %08lX: invalid subkey binding\n"),
 					 (ulong)keyid[1]);
-		    else
-		      knode->flag |= 1; /* mark that signature checked */
-		  }
+
+			knode->flag |= 2; /* mark as invalid */
+		    }
+		    knode->flag |= 1; /* mark that signature checked */
 		}
 	    }
 	}
@@ -878,6 +753,29 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
     return 0;
 }
 
+
+
+/****************
+ * If a user ID has at least one signature, mark it as valid
+ */
+static void
+mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid )
+{
+    KBNODE node;
+    for(node=keyblock->next; node; node = node->next ) {
+	if( node->pkt->pkttype == PKT_USER_ID && !(node->flag & 1) ) {
+	    if( node->next && node->next->pkt->pkttype == PKT_SIGNATURE ) {
+		node->flag |= 1;
+		log_info( _("key %08lX: accepted non self-signed user ID '"),
+							 (ulong)kid[1]);
+		print_string( log_stream(), node->pkt->pkt.user_id->name,
+					    node->pkt->pkt.user_id->len, 0 );
+		fputs("'\n", log_stream() );
+	    }
+	}
+    }
+}
+
 /****************
  * delete all parts which are invalid and those signatures whose
  * public key algorithm is not available in this implemenation;
@@ -890,6 +788,7 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 {
     KBNODE node;
     int nvalid=0, uid_seen=0;
+    const char *p;
 
     for(node=keyblock->next; node; node = node->next ) {
 	if( node->pkt->pkttype == PKT_USER_ID ) {
@@ -898,8 +797,8 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 		if( opt.verbose ) {
 		    log_info( _("key %08lX: skipped user ID '"),
 							 (ulong)keyid[1]);
-		    print_utf8_string( stderr, node->pkt->pkt.user_id->name,
-				       node->pkt->pkt.user_id->len );
+		    print_string( stderr, node->pkt->pkt.user_id->name,
+				      node->pkt->pkt.user_id->len, 0 );
 		    fputs("'\n", stderr );
 		}
 		delete_kbnode( node ); /* the user-id */
@@ -935,9 +834,11 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 		 && check_pubkey_algo( node->pkt->pkt.signature->pubkey_algo)
 		 && node->pkt->pkt.signature->pubkey_algo != PUBKEY_ALGO_RSA )
 	    delete_kbnode( node ); /* build_packet() can't handle this */
-	else if( node->pkt->pkttype == PKT_SIGNATURE &&
-		 !node->pkt->pkt.signature->flags.exportable &&
-		 seckey_available( node->pkt->pkt.signature->keyid ) ) {
+	else if( node->pkt->pkttype == PKT_SIGNATURE
+		 && (p = parse_sig_subpkt2( node->pkt->pkt.signature,
+					    SIGSUBPKT_EXPORTABLE, NULL ))
+		 && !*p
+		 && seckey_available( node->pkt->pkt.signature->keyid ) ) {
 	    /* here we violate the rfc a bit by still allowing
 	     * to import non-exportable signature when we have the
 	     * the secret key used to create this signature - it
@@ -957,22 +858,12 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 		delete_kbnode( node );
 	    }
 	    else {
-	      /* If the revocation cert is from a different key than
-                 the one we're working on don't check it - it's
-                 probably from a revocation key and won't be
-                 verifiable with this key anyway. */
-
-	      if(node->pkt->pkt.signature->keyid[0]==keyid[0] &&
-		 node->pkt->pkt.signature->keyid[1]==keyid[1])
-		{
-		  int rc = check_key_signature( keyblock, node, NULL);
-		  if( rc )
-		    {
-		      log_error( _("key %08lX: invalid revocation "
-				   "certificate: %s - skipped\n"),
-				 (ulong)keyid[1], g10_errstr(rc));
-		      delete_kbnode( node );
-		    }
+		int rc = check_key_signature( keyblock, node, NULL);
+		if( rc ) {
+		    log_error( _("key %08lX: invalid revocation "
+			      "certificate: %s - skipped\n"),
+			      (ulong)keyid[1], g10_errstr(rc));
+		    delete_kbnode( node );
 		}
 	    }
 	}
@@ -1075,84 +966,7 @@ collapse_uids( KBNODE *keyblock )
     return 1;
 }
 
-/* Check for a 0x20 revocation from a revocation key that is not
-   present.  This gets called without the benefit of merge_xxxx so you
-   can't rely on pk->revkey and friends. */
-static void
-revocation_present(KBNODE keyblock)
-{
-  KBNODE onode,inode;
-  PKT_public_key *pk=keyblock->pkt->pkt.public_key;
 
-  for(onode=keyblock->next;onode;onode=onode->next)
-    {
-      /* If we reach user IDs, we're done. */
-      if(onode->pkt->pkttype==PKT_USER_ID)
-	break;
-
-      if(onode->pkt->pkttype==PKT_SIGNATURE &&
-	 onode->pkt->pkt.signature->sig_class==0x1F &&
-	 onode->pkt->pkt.signature->revkey)
-	{
-	  int idx;
-	  PKT_signature *sig=onode->pkt->pkt.signature;
-
-	  for(idx=0;idx<sig->numrevkeys;idx++)
-	    {
-	      u32 keyid[2];
-
-	      keyid_from_fingerprint(sig->revkey[idx]->fpr,
-				     MAX_FINGERPRINT_LEN,keyid);
-
-	      for(inode=keyblock->next;inode;inode=inode->next)
-		{
-		  /* If we reach user IDs, we're done. */
-		  if(inode->pkt->pkttype==PKT_USER_ID)
-		    break;
-
-		  if(inode->pkt->pkttype==PKT_SIGNATURE &&
-		     inode->pkt->pkt.signature->sig_class==0x20 &&
-		     inode->pkt->pkt.signature->keyid[0]==keyid[0] &&
-		     inode->pkt->pkt.signature->keyid[1]==keyid[1])
-		    {
-		      /* Okay, we have a revocation key, and a
-                         revocation issued by it.  Do we have the key
-                         itself? */
-		      int rc;
-
-		      rc=get_pubkey_byfprint(NULL,sig->revkey[idx]->fpr,
-					     MAX_FINGERPRINT_LEN);
-		      if(rc==G10ERR_NO_PUBKEY || rc==G10ERR_UNU_PUBKEY)
-			{
-			  /* No, so try and get it */
-			  if(opt.keyserver_scheme &&
-			     opt.keyserver_options.auto_key_retrieve)
-			    {
-			      log_info(_("Warning: key %08lX may be revoked: "
-					 "fetching revocation key %08lX\n"),
-				       (ulong)keyid_from_pk(pk,NULL),
-				       (ulong)keyid[1]);
-			      keyserver_import_fprint(sig->revkey[idx]->fpr,
-						      MAX_FINGERPRINT_LEN);
-
-			      /* Do we have it now? */
-			      rc=get_pubkey_byfprint(NULL,
-						     sig->revkey[idx]->fpr,
-						     MAX_FINGERPRINT_LEN);
-			    }
-
-			  if(rc==G10ERR_NO_PUBKEY || rc==G10ERR_UNU_PUBKEY)
-			    log_info(_("Warning: key %08lX may be revoked: "
-				       "revocation key %08lX not present.\n"),
-				     (ulong)keyid_from_pk(pk,NULL),
-				     (ulong)keyid[1]);
-			}
-		    }
-		}
-	    }
-	}
-    }
-}
 
 /****************
  * compare and merge the blocks
@@ -1197,44 +1011,13 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 		KBNODE n2 = clone_kbnode(node);
 		insert_kbnode( keyblock_orig, n2, 0 );
 		n2->flag |= 1;
-                ++*n_sigs;
 		log_info( _("key %08lX: revocation certificate added\n"),
 					 (ulong)keyid[1]);
 	    }
 	}
     }
 
-    /* 2nd: merge in any direct key (0x1F) sigs */
-    for(node=keyblock->next; node; node=node->next ) {
-	if( node->pkt->pkttype == PKT_USER_ID )
-	    break;
-	else if( node->pkt->pkttype == PKT_SIGNATURE
-		 && node->pkt->pkt.signature->sig_class == 0x1F )  {
-	    /* check whether we already have this */
-	    found = 0;
-	    for(onode=keyblock_orig->next; onode; onode=onode->next ) {
-		if( onode->pkt->pkttype == PKT_USER_ID )
-		    break;
-		else if( onode->pkt->pkttype == PKT_SIGNATURE
-			 && onode->pkt->pkt.signature->sig_class == 0x1F
-			 && !cmp_signatures(onode->pkt->pkt.signature,
-					    node->pkt->pkt.signature)) {
-		    found = 1;
-		    break;
-		}
-	    }
-	    if( !found ) {
-		KBNODE n2 = clone_kbnode(node);
-		insert_kbnode( keyblock_orig, n2, 0 );
-		n2->flag |= 1;
-                ++*n_sigs;
-		log_info( _("key %08lX: direct key signature added\n"),
-					 (ulong)keyid[1]);
-	    }
-	}
-    }
-
-    /* 3rd: try to merge new certificates in */
+    /* 2nd: try to merge new certificates in */
     for(onode=keyblock_orig->next; onode; onode=onode->next ) {
 	if( !(onode->flag & 1) && onode->pkt->pkttype == PKT_USER_ID) {
 	    /* find the user id in the imported keyblock */
@@ -1251,7 +1034,7 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 	}
     }
 
-    /* 4th: add new user-ids */
+    /* 3rd: add new user-ids */
     for(node=keyblock->next; node; node=node->next ) {
 	if( node->pkt->pkttype == PKT_USER_ID) {
 	    /* do we have this in the original keyblock */
@@ -1269,7 +1052,7 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 	}
     }
 
-    /* 5th: add new subkeys */
+    /*	add new subkeys */
     for(node=keyblock->next; node; node=node->next ) {
 	onode = NULL;
 	if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
@@ -1302,7 +1085,7 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 	}
     }
 
-    /* 6th: merge subkey certificates */
+    /* merge subkey certificates */
     for(onode=keyblock_orig->next; onode; onode=onode->next ) {
 	if( !(onode->flag & 1)
 	    &&	(   onode->pkt->pkttype == PKT_PUBLIC_SUBKEY
@@ -1341,6 +1124,11 @@ append_uid( KBNODE keyblock, KBNODE node, int *n_sigs,
     KBNODE n, n_where=NULL;
 
     assert(node->pkt->pkttype == PKT_USER_ID );
+    if( !node->next || node->next->pkt->pkttype == PKT_USER_ID ) {
+	log_error( _("key %08lX: our copy has no self-signature\n"),
+						  (ulong)keyid[1]);
+	return G10ERR_GENERAL;
+    }
 
     /* find the position */
     for( n = keyblock; n; n_where = n, n = n->next ) {
@@ -1389,6 +1177,12 @@ merge_sigs( KBNODE dst, KBNODE src, int *n_sigs,
 
     assert(dst->pkt->pkttype == PKT_USER_ID );
     assert(src->pkt->pkttype == PKT_USER_ID );
+    if( !dst->next || dst->next->pkt->pkttype == PKT_USER_ID ) {
+	log_error( _("key %08lX: our copy has no self-signature\n"),
+						  (ulong)keyid[1]);
+	return 0;
+    }
+
 
     for(n=src->next; n && n->pkt->pkttype != PKT_USER_ID; n = n->next ) {
 	if( n->pkt->pkttype != PKT_SIGNATURE )
@@ -1508,3 +1302,4 @@ append_key( KBNODE keyblock, KBNODE node, int *n_sigs,
 
     return 0;
 }
+
