@@ -1,5 +1,5 @@
 /* main.h
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -19,22 +19,16 @@
  */
 #ifndef G10_MAIN_H
 #define G10_MAIN_H
-#include "types.h"
+
+#include <gcrypt.h>
+#include "basicdefs.h"
 #include "iobuf.h"
 #include "mpi.h"
-#include "cipher.h"
 #include "keydb.h"
 
-#define DEFAULT_CIPHER_ALGO  CIPHER_ALGO_CAST5
-#define DEFAULT_PUBKEY_ALGO  PUBKEY_ALGO_ELGAMAL
-#define DEFAULT_DIGEST_ALGO  DIGEST_ALGO_SHA1
-
-
-typedef struct {
-    int header_okay;
-    PK_LIST pk_list;
-    cipher_filter_context_t cfx;
-} encrypt_filter_context_t;
+#define DEFAULT_CIPHER_ALGO  GCRY_CIPHER_BLOWFISH
+#define DEFAULT_PUBKEY_ALGO  GCRY_PUBKEY_ELGAMAL
+#define DEFAULT_DIGEST_ALGO  GCRY_MD_RMD160
 
 
 /*-- g10.c --*/
@@ -54,21 +48,16 @@ char *make_radix64_string( const byte *data, size_t len );
 
 /*-- misc.c --*/
 void trap_unaligned(void);
-int disable_core_dumps(void);
+void disable_core_dumps(void);
 u16 checksum_u16( unsigned n );
 u16 checksum( byte *p, unsigned n );
 u16 checksum_mpi( MPI a );
 u16 checksum_mpi_counted_nbits( MPI a );
 u32 buffer_to_u32( const byte *buffer );
-const byte *get_session_marker( size_t *rlen );
+
 int openpgp_cipher_test_algo( int algo );
-int openpgp_pk_test_algo( int algo, unsigned int usage_flags );
-int openpgp_pk_algo_usage ( int algo );
+int openpgp_pk_test_algo( int algo );
 int openpgp_md_test_algo( int algo );
-int check_permissions(const char *path,int extension,int checkonly);
-void idea_cipher_warn( int show );
-char *pct_expando(const char *string,PKT_public_key *pk);
-int hextobyte( const char *s );
 
 /*-- helptext.c --*/
 void display_online_help( const char *keyword );
@@ -77,39 +66,33 @@ void display_online_help( const char *keyword );
 int encode_symmetric( const char *filename );
 int encode_store( const char *filename );
 int encode_crypt( const char *filename, STRLIST remusr );
-void encode_crypt_files(int nfiles, char **files, STRLIST remusr);
 int encrypt_filter( void *opaque, int control,
 		    IOBUF a, byte *buf, size_t *ret_len);
 
 
 /*-- sign.c --*/
-int complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md );
+int complete_sig( PKT_signature *sig, PKT_secret_key *sk, GCRY_MD_HD md );
 int sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	       int do_encrypt, STRLIST remusr, const char *outfile );
 int clearsign_file( const char *fname, STRLIST locusr, const char *outfile );
-int sign_symencrypt_file (const char *fname, STRLIST locusr);
 
 /*-- sig-check.c --*/
 int check_key_signature( KBNODE root, KBNODE node, int *is_selfsig );
 int check_key_signature2( KBNODE root, KBNODE node,
-			  int *is_selfsig, u32 *r_expiredate, int *r_expired );
+			  int *is_selfsig, u32 *r_expire );
 
 /*-- delkey.c --*/
-int delete_keys( STRLIST names, int secret, int allow_both );
+int delete_key( const char *username, int secure );
 
 /*-- keyedit.c --*/
 void keyedit_menu( const char *username, STRLIST locusr, STRLIST cmds,
 							    int sign_mode );
 
 /*-- keygen.c --*/
-u32 ask_expire_interval(int object);
 u32 ask_expiredate(void);
-void generate_keypair( const char *fname );
-int keygen_set_std_prefs (const char *string);
-char *keygen_get_std_prefs (void);
+void generate_keypair(void);
 int keygen_add_key_expire( PKT_signature *sig, void *opaque );
 int keygen_add_std_prefs( PKT_signature *sig, void *opaque );
-int keygen_upd_std_prefs( PKT_signature *sig, void *opaque );
 int generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock );
 
 /*-- openfile.c --*/
@@ -118,69 +101,49 @@ char *make_outfile_name( const char *iname );
 char *ask_outfile_name( const char *name, size_t namelen );
 int   open_outfile( const char *iname, int mode, IOBUF *a );
 IOBUF open_sigfile( const char *iname );
-void try_make_homedir( const char *fname );
+void copy_options_file( const char *destdir );
 
 /*-- seskey.c --*/
 void make_session_key( DEK *dek );
 MPI encode_session_key( DEK *dek, unsigned nbits );
-MPI encode_md_value( int pubkey_algo,  MD_HANDLE md,
-		     int hash_algo, unsigned nbits, int v3compathack );
+MPI encode_md_value( int pubkey_algo,  GCRY_MD_HD md,
+		     int hash_algo, unsigned nbits );
 
 /*-- comment.c --*/
 KBNODE make_comment_node( const char *s );
 KBNODE make_mpi_comment_node( const char *s, MPI a );
 
 /*-- import.c --*/
-void import_keys( char **fnames, int nnames, int fast, void *stats_hd );
-int import_keys_stream( IOBUF inp, int fast, void *stats_hd );
-void *import_new_stats_handle (void);
-void import_release_stats_handle (void *p);
-void import_print_stats (void *hd);
-
+int import_keys( const char *filename, int fast );
+int import_keys_stream( IOBUF inp, int fast );
 int collapse_uids( KBNODE *keyblock );
 
 /*-- export.c --*/
 int export_pubkeys( STRLIST users, int onlyrfc );
 int export_pubkeys_stream( IOBUF out, STRLIST users, int onlyrfc );
 int export_seckeys( STRLIST users );
-int export_secsubkeys( STRLIST users );
 
 /* dearmor.c --*/
 int dearmor_file( const char *fname );
 int enarmor_file( const char *fname );
 
 /*-- revoke.c --*/
-struct revocation_reason_info;
 int gen_revoke( const char *uname );
-int revocation_reason_build_cb( PKT_signature *sig, void *opaque );
-struct revocation_reason_info *
-		ask_revocation_reason( int key_rev, int cert_rev, int hint );
-void release_revocation_reason_info( struct revocation_reason_info *reason );
 
 /*-- keylist.c --*/
-void public_key_list( STRLIST list );
-void secret_key_list( STRLIST list );
-void list_keyblock( KBNODE keyblock, int secret, int fpr, void *opaque );
-void print_fingerprint (PKT_public_key *pk, PKT_secret_key *sk, int mode);
-void show_policy_url(PKT_signature *sig,int indent);
-void show_notation(PKT_signature *sig,int indent);
+void public_key_list( int nnames, char **names );
+void secret_key_list( int nnames, char **names );
 
 /*-- verify.c --*/
-void print_file_status( int status, const char *name, int what );
 int verify_signatures( int nfiles, char **files );
-int verify_files( int nfiles, char **files );
 
 /*-- decrypt.c --*/
 int decrypt_message( const char *filename );
-void decrypt_messages(int nfiles, char **files);
 
 /*-- plaintext.c --*/
-int hash_datafiles( MD_HANDLE md, MD_HANDLE md2,
+int hash_datafiles( GCRY_MD_HD md, GCRY_MD_HD md2,
 		    STRLIST files, const char *sigfilename, int textmode );
 
-/*-- pipemode.c --*/
-void run_in_pipemode (void);
-
 /*-- signal.c --*/
 void init_signals(void);
 void pause_on_sigusr( int which );
