@@ -1,5 +1,5 @@
 /* openfile.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,9 +24,6 @@
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include "util.h"
 #include "memory.h"
@@ -42,16 +39,13 @@
   #define SKELEXT ".skel"
 #endif
 
+
 #ifdef HAVE_DRIVE_LETTERS
   #define CMP_FILENAME(a,b) stricmp( (a), (b) )
 #else
   #define CMP_FILENAME(a,b) strcmp( (a), (b) )
 #endif
 
-#ifdef MKDIR_TAKES_ONE_ARG
-# undef mkdir
-# define mkdir(a,b) mkdir(a)
-#endif
 
 /* FIXME:  Implement opt.interactive. */
 
@@ -70,11 +64,6 @@ overwrite_filep( const char *fname )
     if( access( fname, F_OK ) )
 	return 1; /* does not exist */
 
-#ifndef HAVE_DOSISH_SYSTEM
-    if ( !strcmp ( fname, "/dev/null" ) )
-        return 1; /* does not do any harm */
-#endif
-
     /* fixme: add some backup stuff in case of overwrite */
     if( opt.answer_yes )
 	return 1;
@@ -89,6 +78,7 @@ overwrite_filep( const char *fname )
 }
 
 
+
 /****************
  * Strip know extensions from iname and return a newly allocated
  * filename.  Return NULL if we can't do that.
@@ -110,11 +100,6 @@ make_outfile_name( const char *iname )
 	buf[n-4] = 0;
 	return buf;
     }
-    else if( n > 5 && !CMP_FILENAME(iname+n-5,".sign") ) {
-	char *buf = m_strdup( iname );
-	buf[n-5] = 0;
-	return buf;
-    }
 
     log_info(_("%s: unknown suffix\n"), iname );
     return NULL;
@@ -155,8 +140,6 @@ ask_outfile_name( const char *name, size_t namelen )
 	fname = defname; defname = NULL;
     }
     m_free(defname);
-    if (fname)
-        trim_spaces (fname);
     return fname;
 }
 
@@ -193,7 +176,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    name = opt.outfile;
 	else {
 	  #ifdef USE_ONLY_8DOT3
-	    /* It is quite common DOS system to have only one dot in a
+	    /* It is quite common for DOS system to have only one dot in a
 	     * a filename So if we have something like this, we simple
 	     * replace the suffix execpt in cases where the suffix is
 	     * larger than 3 characters and not the same as.
@@ -212,7 +195,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 		strcpy(dot, newsfx );
 	    }
 	    else if( dot && !dot[1] ) /* don't duplicate a dot */
-		strcpy( dot, newsfx+1 );
+		strcat( dot, newsfx+1 );
 	    else
 		strcat( buf, newsfx );
 	  #else
@@ -223,19 +206,7 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    name = buf;
 	}
 
-        rc = 0;
-	while( !overwrite_filep (name) ) {
-            char *tmp = ask_outfile_name (NULL, 0);
-            if ( !tmp || !*tmp ) {
-                m_free (tmp);
-                rc = G10ERR_FILE_EXISTS;
-                break;
-            }
-            m_free (buf);
-            name = buf = tmp;
-        }
-
-	if( !rc ) {
+	if( overwrite_filep( name ) ) {
 	    if( !(*a = iobuf_create( name )) ) {
 		log_error(_("%s: can't create: %s\n"), name, strerror(errno) );
 		rc = G10ERR_CREATE_FILE;
@@ -243,6 +214,8 @@ open_outfile( const char *iname, int mode, IOBUF *a )
 	    else if( opt.verbose )
 		log_info(_("writing to `%s'\n"), name );
 	}
+	else
+	    rc = G10ERR_FILE_EXISTS;
 	m_free(buf);
     }
     return rc;
@@ -263,13 +236,12 @@ open_sigfile( const char *iname )
     if( iname && !(*iname == '-' && !iname[1]) ) {
 	len = strlen(iname);
 	if( len > 4 && ( !strcmp(iname + len - 4, ".sig")
-                        || ( len > 5 && !strcmp(iname + len - 5, ".sign") )
-                        || !strcmp(iname + len - 4, ".asc")) ) {
+			|| !strcmp(iname + len - 4, ".asc")) ) {
 	    char *buf;
 	    buf = m_strdup(iname);
-	    buf[len-(buf[len-1]=='n'?5:4)] = 0 ;
+	    buf[len-4] = 0 ;
 	    a = iobuf_open( buf );
-	    if( a && opt.verbose )
+	    if( opt.verbose )
 		log_info(_("assuming signed data in `%s'\n"), buf );
 	    m_free(buf);
 	}
@@ -281,7 +253,7 @@ open_sigfile( const char *iname )
 /****************
  * Copy the option file skeleton to the given directory.
  */
-static void
+void
 copy_options_file( const char *destdir )
 {
     const char *datadir = GNUPG_DATADIR;
@@ -324,36 +296,3 @@ copy_options_file( const char *destdir )
     m_free(fname);
 }
 
-
-void
-try_make_homedir( const char *fname )
-{
-    const char *defhome = GNUPG_HOMEDIR;
-
-    /* Create the directory only if the supplied directory name
-     * is the same as the default one.  This way we avoid to create
-     * arbitrary directories when a non-default homedirectory is used.
-     * To cope with HOME, we do compare only the suffix if we see that
-     * the default homedir does start with a tilde.
-     */
-    if( opt.dry_run )
-	return;
-
-    if ( ( *defhome == '~'
-           && ( strlen(fname) >= strlen (defhome+1)
-                && !strcmp(fname+strlen(fname)-strlen(defhome+1),
-                           defhome+1 ) ))
-         || ( *defhome != '~'
-              && !compare_filenames( fname, defhome ) )
-        ) {
-	if( mkdir( fname, S_IRUSR|S_IWUSR|S_IXUSR ) )
-	    log_fatal( _("%s: can't create directory: %s\n"),
-					fname,	strerror(errno) );
-	else if( !opt.quiet )
-	    log_info( _("%s: directory created\n"), fname );
-	copy_options_file( fname );
-	log_info(_("you have to start GnuPG again, "
-		   "so it can read the new options file\n") );
-	g10_exit(1);
-    }
-}
