@@ -1,5 +1,5 @@
-/* plaintext.c -  process plaintext packets
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/* plaintext.c -  process an plaintext packet
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -24,10 +24,6 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
-#ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
-#endif
-
 #include "util.h"
 #include "memory.h"
 #include "options.h"
@@ -85,21 +81,10 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     else if( !*fname || (*fname=='-' && !fname[1])) {
 	/* no filename or "-" given; write to stdout */
 	fp = stdout;
-      #ifdef HAVE_DOSISH_SYSTEM
-	setmode ( fileno(fp) , O_BINARY );
-      #endif
     }
-    else {
-	while( !overwrite_filep (fname) ) {
-            char *tmp = ask_outfile_name (NULL, 0);
-            if ( !tmp || !*tmp ) {
-                m_free (tmp);
-                rc = G10ERR_CREATE_FILE;
-                goto leave;
-            }
-            m_free (fname);
-            fname = tmp;
-        }
+    else if( !overwrite_filep( fname ) ) {
+	rc = G10ERR_CREATE_FILE;
+	goto leave;
     }
 
     if( fp || nooutput )
@@ -121,11 +106,9 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    goto leave;
 		}
 		if( mfx->md )
-		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
-		if( c == '\r' )  /* convert to native line ending */
-		    continue;	 /* fixme: this hack might be too simple */
-	      #endif
+		    gcry_md_putc(mfx->md, c );
+		if( c == '\r' )
+		    continue; /* fixme: this hack might be too simple */
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
@@ -149,7 +132,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		    goto leave;
 		}
 		if( mfx->md )
-		    md_write( mfx->md, buffer, len );
+		    gcry_md_write( mfx->md, buffer, len );
 		if( fp ) {
 		    if( fwrite( buffer, 1, len, fp ) != len ) {
 			log_error("Error writing to `%s': %s\n",
@@ -168,11 +151,9 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	if( convert ) { /* text mode */
 	    while( (c = iobuf_get(pt->buf)) != -1 ) {
 		if( mfx->md )
-		    md_putc(mfx->md, c );
-	      #ifndef HAVE_DOSISH_SYSTEM
+		    gcry_md_putc(mfx->md, c );
 		if( convert && c == '\r' )
 		    continue; /* fixme: this hack might be too simple */
-	      #endif
 		if( fp ) {
 		    if( putc( c, fp ) == EOF ) {
 			log_error("Error writing to `%s': %s\n",
@@ -188,10 +169,10 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    int eof;
 	    for( eof=0; !eof; ) {
 		/* Why do we check for len < 32768:
-		 * If we won't, we would practically read 2 EOFs but
+		 * If we won we would practically read 2 EOFS but
 		 * the first one has already popped the block_filter
 		 * off and therefore we don't catch the boundary.
-		 * So, always assume EOF if iobuf_read returns less bytes
+		 * Always assume EOF if iobuf_read returns less bytes
 		 * then requested */
 		int len = iobuf_read( pt->buf, buffer, 32768 );
 		if( len == -1 )
@@ -199,7 +180,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		if( len < 32768 )
 		    eof = 1;
 		if( mfx->md )
-		    md_write( mfx->md, buffer, len );
+		    gcry_md_write( mfx->md, buffer, len );
 		if( fp ) {
 		    if( fwrite( buffer, 1, len, fp ) != len ) {
 			log_error("Error writing to `%s': %s\n",
@@ -229,28 +210,26 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	    if( !mfx->md )
 		continue;
 	    if( state == 2 ) {
-		md_putc(mfx->md, '\r' );
-		md_putc(mfx->md, '\n' );
+		gcry_md_putc(mfx->md, '\r' );
+		gcry_md_putc(mfx->md, '\n' );
 		state = 0;
 	    }
 	    if( !state ) {
 		if( c == '\r'  )
 		    state = 1;
-		else if( c == '\n'  )
-		    state = 2;
 		else
-		    md_putc(mfx->md, c );
+		    gcry_md_putc(mfx->md, c );
 	    }
 	    else if( state == 1 ) {
 		if( c == '\n'  )
 		    state = 2;
 		else {
-		    md_putc(mfx->md, '\r' );
+		    gcry_md_putc(mfx->md, '\r' );
 		    if( c == '\r'  )
 			state = 1;
 		    else {
 			state = 0;
-			md_putc(mfx->md, c );
+			gcry_md_putc(mfx->md, c );
 		    }
 		}
 	    }
@@ -274,7 +253,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 }
 
 static void
-do_hash( MD_HANDLE md, MD_HANDLE md2, IOBUF fp, int textmode )
+do_hash( GCRY_MD_HD md, GCRY_MD_HD md2, IOBUF fp, int textmode )
 {
     text_filter_context_t tfx;
     int c;
@@ -288,27 +267,27 @@ do_hash( MD_HANDLE md, MD_HANDLE md2, IOBUF fp, int textmode )
 	int lc = -1;
 	while( (c = iobuf_get(fp)) != -1 ) {
 	    if( c == '\n' && lc == '\r' )
-		md_putc(md2, c);
+		gcry_md_putc(md2, c);
 	    else if( c == '\n' ) {
-		md_putc(md2, '\r');
-		md_putc(md2, c);
+		gcry_md_putc(md2, '\r');
+		gcry_md_putc(md2, c);
 	    }
 	    else if( c != '\n' && lc == '\r' ) {
-		md_putc(md2, '\n');
-		md_putc(md2, c);
+		gcry_md_putc(md2, '\n');
+		gcry_md_putc(md2, c);
 	    }
 	    else
-		md_putc(md2, c);
+		gcry_md_putc(md2, c);
 
 	    if( md )
-		md_putc(md, c );
+		gcry_md_putc(md, c );
 	    lc = c;
 	}
     }
     else {
 	while( (c = iobuf_get(fp)) != -1 ) {
 	    if( md )
-		md_putc(md, c );
+		gcry_md_putc(md, c );
 	}
     }
 }
@@ -319,7 +298,7 @@ do_hash( MD_HANDLE md, MD_HANDLE md2, IOBUF fp, int textmode )
  * INFILE is the name of the input file.
  */
 int
-ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
+ask_for_detached_datafile( GCRY_MD_HD md, GCRY_MD_HD md2,
 			   const char *inname, int textmode )
 {
     char *answer = NULL;
@@ -374,11 +353,11 @@ ask_for_detached_datafile( MD_HANDLE md, MD_HANDLE md2,
  * If FILES is NULL, hash stdin.
  */
 int
-hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
+hash_datafiles( GCRY_MD_HD md, GCRY_MD_HD md2, STRLIST files,
 		const char *sigfilename, int textmode )
 {
     IOBUF fp;
-    STRLIST sl;
+    STRLIST sl=NULL;
 
     if( !files ) {
 	/* check whether we can open the signed material */
@@ -388,26 +367,28 @@ hash_datafiles( MD_HANDLE md, MD_HANDLE md2, STRLIST files,
 	    iobuf_close(fp);
 	    return 0;
 	}
-        log_error (_("no signed data\n"));
-        return G10ERR_OPEN_FILE;
+	/* no we can't (no sigfile) - read signed stuff from stdin */
+	add_to_strlist( &sl, "-");
     }
+    else
+	sl = files;
 
-
-    for (sl=files; sl; sl = sl->next ) {
+    for( ; sl; sl = sl->next ) {
 	fp = iobuf_open( sl->d );
 	if( !fp ) {
 	    log_error(_("can't open signed data `%s'\n"),
 						print_fname_stdin(sl->d));
+	    if( !files )
+		free_strlist(sl);
 	    return G10ERR_OPEN_FILE;
 	}
 	do_hash( md, md2, fp, textmode );
 	iobuf_close(fp);
     }
 
+    if( !files )
+	free_strlist(sl);
     return 0;
 }
 
 
-
-
-
