@@ -1,5 +1,5 @@
 /* ringedit.c -  Function for key ring editing
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -63,6 +63,10 @@
 #include "i18n.h"
 
 
+#ifdef MKDIR_TAKES_ONE_ARG
+# undef mkdir
+# define mkdir(a,b) mkdir(a)
+#endif
 
 
 struct resource_table_struct {
@@ -181,6 +185,7 @@ enum_keyblock_resources( int *sequence, int secret )
 }
 
 
+
 /****************
  * Register a resource (which currently may only be a keyring file).
  * The first keyring which is added by this function is
@@ -198,7 +203,7 @@ add_keyblock_resource( const char *url, int force, int secret )
     char *filename = NULL;
     int rc = 0;
     enum resource_type rt = rt_UNKNOWN;
-    const char *created_fname = NULL;
+
 
     /* Do we have an URL?
      *	gnupg-gdbm:filename  := this is a GDBM resource
@@ -286,13 +291,24 @@ add_keyblock_resource( const char *url, int force, int secret )
 	    *last_slash_in_filename = 0;
 
 	    if( access(filename, F_OK) ) {
-		/* on the first time we try to create the default homedir and
-		 * in this case the process will be terminated, so that on the
-		 * next invocation it can read the options file in on startup
-		 */
-		try_make_homedir( filename );
-		rc = G10ERR_OPEN_FILE;
-		goto leave;
+		if( strlen(filename) >= 7
+		    && !strcmp(filename+strlen(filename)-7, "/.gnupg") ) {
+		    if( mkdir(filename, S_IRUSR|S_IWUSR|S_IXUSR) )
+		    {
+			log_error( _("%s: can't create directory: %s\n"),
+				  filename, strerror(errno));
+			rc = G10ERR_OPEN_FILE;
+			goto leave;
+		    }
+		    else if( !opt.quiet )
+			log_info( _("%s: directory created\n"), filename );
+		    copy_options_file( filename );
+		}
+		else
+		{
+		    rc = G10ERR_OPEN_FILE;
+		    goto leave;
+		}
 	    }
 
 	    *last_slash_in_filename = '/';
@@ -317,14 +333,12 @@ add_keyblock_resource( const char *url, int force, int secret )
 	      #endif
 		if( !opt.quiet )
 		    log_info(_("%s: keyring created\n"), filename );
-                created_fname = filename;
 	    }
 	}
       #if HAVE_DOSISH_SYSTEM || 1
 	iobuf_close( iobuf );
 	iobuf = NULL;
-        if (created_fname) /* must invalidate that ugly cache */
-            iobuf_ioctl (NULL, 2, 0, (char*)created_fname);
+	/* must close it again */
       #endif
 	break;
 
@@ -421,35 +435,6 @@ get_keyblock_handle( const char *filename, int secret, KBPOS *kbpos )
 }
 
 
-/****************
- * Return the filename of the firstkeyblock resource which is intended
- * for write access. This will either be the default resource or in
- * case this is not writable one of the others.  If no writable is found,
- * the default filename in the homedirectory will be returned.
- * Caller must free, will never return NULL.
- */
-char *
-get_writable_keyblock_file( int secret )
-{
-    int i = secret? default_secret_resource : default_public_resource;
-
-    if( resource_table[i].used && !resource_table[i].secret == !secret ) {
-	if( !access( resource_table[i].fname, R_OK|W_OK ) ) {
-	    return m_strdup( resource_table[i].fname );
-	}
-    }
-    for(i=0; i < MAX_RESOURCES; i++ ) {
-	if( resource_table[i].used && !resource_table[i].secret == !secret ) {
-	    if( !access( resource_table[i].fname, R_OK|W_OK ) ) {
-		return m_strdup( resource_table[i].fname );
-	    }
-	}
-    }
-    /* Assume the home dir is always writable */
-    return  make_filename(opt.homedir, secret? "secring.gpg"
-					     : "pubring.gpg", NULL );
-}
-
 
 /****************
  * Search a keyblock which starts with the given packet and puts all
@@ -1072,13 +1057,7 @@ cmp_seckey( PKT_secret_key *req_sk, PKT_secret_key *sk )
 
     n = pubkey_get_nskey( req_sk->pubkey_algo );
     for(i=0; i < n; i++ ) {
-        /* Note: becuase v4 protected keys have nothing in the
-         * mpis except for the first one, we skip all NULL MPIs.
-         * This might not be always correct in cases where the both
-         * keys do not match in their secret parts but we can ignore that
-         * because the need for this function is quite ugly. */
-	if( req_sk->skey[1] && sk->skey[i]
-             && mpi_cmp( req_sk->skey[i], sk->skey[i] ) )
+	if( mpi_cmp( req_sk->skey[i], sk->skey[i] ) )
 	    return -1;
     }
     return 0;
@@ -1108,7 +1087,7 @@ keyring_search( PACKET *req, KBPOS *kbpos, IOBUF iobuf, const char *fname )
     int rc;
     PACKET pkt;
     int save_mode;
-    off_t offset;
+    ulong offset;
     int pkttype = req->pkttype;
     PKT_public_key *req_pk = req->pkt.public_key;
     PKT_secret_key *req_sk = req->pkt.secret_key;
@@ -1190,9 +1169,9 @@ keyring_read( KBPOS *kbpos, KBNODE *ret_root )
     }
 
     if( !kbpos->valid )
-       log_debug("kbpos not valid in keyring_read\n" );
+	log_debug("kbpos not valid in keyring_read, want %d\n", (int)kbpos->offset );
     if( iobuf_seek( a, kbpos->offset ) ) {
-       log_error("can't seek\n");
+	log_error("can't seek to %lu\n", kbpos->offset);
 	iobuf_close(a);
 	return G10ERR_KEYRING_OPEN;
     }
@@ -1531,8 +1510,6 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 
     /* rename and make backup file */
     if( !rentry->secret ) {  /* but not for secret keyrings */
-        iobuf_ioctl (NULL, 2, 0, bakfname );
-        iobuf_ioctl (NULL, 2, 0, rentry->fname );
       #ifdef HAVE_DOSISH_SYSTEM
 	remove( bakfname );
       #endif
@@ -1543,8 +1520,6 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	    goto leave;
 	}
     }
-    iobuf_ioctl (NULL, 2, 0, tmpfname );
-    iobuf_ioctl (NULL, 2, 0, rentry->fname );
   #ifdef HAVE_DOSISH_SYSTEM
     remove( rentry->fname );
   #endif
