@@ -1,5 +1,5 @@
 /* seckey-cert.c -  secret key certificate packet handling
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,12 +23,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+
+#include <gcrypt.h>
 #include "util.h"
 #include "memory.h"
 #include "packet.h"
 #include "mpi.h"
 #include "keydb.h"
-#include "cipher.h"
 #include "main.h"
 #include "options.h"
 #include "i18n.h"
@@ -36,7 +37,7 @@
 
 
 static int
-do_check( PKT_secret_key *sk, const char *tryagain_text )
+do_check( PKT_secret_key *sk )
 {
     byte *buffer;
     u16 csum=0;
@@ -46,113 +47,81 @@ do_check( PKT_secret_key *sk, const char *tryagain_text )
     if( sk->is_protected ) { /* remove the protection */
 	DEK *dek = NULL;
 	u32 keyid[4]; /* 4! because we need two of them */
-	CIPHER_HANDLE cipher_hd=NULL;
+	GCRY_CIPHER_HD cipher_hd=NULL;
 	PKT_secret_key *save_sk;
 
-	if( sk->protect.s2k.mode == 1001 ) {
-	    log_info(_("secret key parts are not available\n"));
-	    return G10ERR_GENERAL;
-	}
-	if( sk->protect.algo == CIPHER_ALGO_NONE )
+	if( sk->protect.algo == GCRY_CIPHER_NONE )
 	    BUG();
-	if( check_cipher_algo( sk->protect.algo ) ) {
-	    log_info(_("protection algorithm %d%s is not supported\n"),
-			sk->protect.algo,sk->protect.algo==1?" (IDEA)":"" );
-	    idea_cipher_warn(0);
+	if( openpgp_cipher_test_algo( sk->protect.algo ) ) {
+	    log_info(_("protection algorithm %d is not supported\n"),
+			sk->protect.algo );
 	    return G10ERR_CIPHER_ALGO;
 	}
 	keyid_from_sk( sk, keyid );
 	keyid[2] = keyid[3] = 0;
 	if( !sk->is_primary ) {
-            keyid[2] = sk->main_keyid[0];
-            keyid[3] = sk->main_keyid[1];
+	    PKT_secret_key *sk2 = m_alloc_clear( sizeof *sk2 );
+	    if( !get_primary_seckey( sk2, keyid ) )
+		keyid_from_sk( sk2, keyid+2 );
+	    free_secret_key( sk2 );
 	}
 	dek = passphrase_to_dek( keyid, sk->pubkey_algo, sk->protect.algo,
-				 &sk->protect.s2k, 0, tryagain_text );
-	cipher_hd = cipher_open( sk->protect.algo,
-				 CIPHER_MODE_AUTO_CFB, 1);
-	cipher_setkey( cipher_hd, dek->key, dek->keylen );
+				 &sk->protect.s2k, 0 );
+	if( !(cipher_hd = gcry_cipher_open( sk->protect.algo,
+				      GCRY_CIPHER_MODE_CFB,
+				      GCRY_CIPHER_SECURE
+				      | (sk->protect.algo >= 100 ?
+					   0 : GCRY_CIPHER_ENABLE_SYNC) ) )
+				    ) {
+	    BUG();
+	}
+
+	if( gcry_cipher_setkey( cipher_hd, dek->key, dek->keylen ) )
+	    log_fatal("set key failed: %s\n", gcry_strerror(-1) );
 	m_free(dek);
 	save_sk = copy_secret_key( NULL, sk );
-	cipher_setiv( cipher_hd, sk->protect.iv, sk->protect.ivlen );
+	if( gcry_cipher_setiv( cipher_hd, sk->protect.iv, sk->protect.ivlen ))
+	    log_fatal("set IV failed: %s\n", gcry_strerror(-1) );
 	csum = 0;
 	if( sk->version >= 4 ) {
 	    int ndata;
 	    byte *p, *data;
-            u16 csumc = 0;
 
 	    i = pubkey_get_npkey(sk->pubkey_algo);
 	    assert( mpi_is_opaque( sk->skey[i] ) );
 	    p = mpi_get_opaque( sk->skey[i], &ndata );
-            if ( ndata > 1 )
-                csumc = p[ndata-2] << 8 | p[ndata-1];
 	    data = m_alloc_secure( ndata );
-	    cipher_decrypt( cipher_hd, data, p, ndata );
+	    gcry_cipher_decrypt( cipher_hd, data, ndata, p, ndata );
 	    mpi_free( sk->skey[i] ); sk->skey[i] = NULL ;
 	    p = data;
-            if (sk->protect.sha1chk) {
-                /* This is the new SHA1 checksum method to detect
-                   tampering with the key as used by the Klima/Rosa
-                   attack */
-                sk->csum = 0;
-                csum = 1;
-                if( ndata < 20 ) 
-                    log_error("not enough bytes for SHA-1 checksum\n");
-                else {
-                    MD_HANDLE h = md_open (DIGEST_ALGO_SHA1, 1);
-                    if (!h)
-                        BUG(); /* algo not available */
-                    md_write (h, data, ndata - 20);
-                    md_final (h);
-                    if (!memcmp (md_read (h, DIGEST_ALGO_SHA1),
-                                 data + ndata - 20, 20) ) {
-                        /* digest does match.  We have to keep the old
-                           style checksum in sk->csum, so that the
-                           test used for unprotected keys does work.
-                           This test gets used when we are adding new
-                           keys. */
-                        sk->csum = csum = checksum (data, ndata-20);
-                    }
-                    md_close (h);
-                }
-            }
-            else {
-                if( ndata < 2 ) {
-                    log_error("not enough bytes for checksum\n");
-                    sk->csum = 0;
-                    csum = 1;
-                }
-                else {
-                    csum = checksum( data, ndata-2);
-                    sk->csum = data[ndata-2] << 8 | data[ndata-1];
-                    if ( sk->csum != csum ) {
-                        /* This is a PGP 7.0.0 workaround */
-                        sk->csum = csumc; /* take the encrypted one */
-                    }
-                }
-            }
-                
-            /* must check it here otherwise the mpi_read_xx would fail
-               because the length may have an arbitrary value */
-            if( sk->csum == csum ) {
-                for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
-                    nbytes = ndata;
-                    sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
-                    ndata -= nbytes;
-                    p += nbytes;
-                }
-                /* Note: at this point ndata should be 2 for a simple
-                   checksum or 20 for the sha1 digest */
-            }
+	    if( ndata < 2 ) {
+		log_error("not enough bytes for checksum\n");
+		sk->csum = 0;
+		csum = 1;
+	    }
+	    else {
+		csum = checksum( data, ndata-2);
+		sk->csum = data[ndata-2] << 8 | data[ndata-1];
+	    }
+	    /* must check it here otherwise the mpi_read_xx would fail
+	     * because the length das an abritary value */
+	    if( sk->csum == csum ) {
+		for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+		    nbytes = ndata;
+		    sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
+		    ndata -= nbytes;
+		    p += nbytes;
+		}
+	    }
 	    m_free(data);
 	}
 	else {
 	    for(i=pubkey_get_npkey(sk->pubkey_algo);
 		    i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
 		buffer = mpi_get_secure_buffer( sk->skey[i], &nbytes, NULL );
-		cipher_sync( cipher_hd );
+		gcry_cipher_sync( cipher_hd );
 		assert( mpi_is_protected(sk->skey[i]) );
-		cipher_decrypt( cipher_hd, buffer, buffer, nbytes );
+		gcry_cipher_decrypt( cipher_hd, buffer, nbytes, NULL, 0 );
 		mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
 		mpi_clear_protect_flag( sk->skey[i] );
 		csum += checksum_mpi( sk->skey[i] );
@@ -162,11 +131,10 @@ do_check( PKT_secret_key *sk, const char *tryagain_text )
 	       csum = sk->csum;
 	    }
 	}
-	cipher_close( cipher_hd );
+	gcry_cipher_close( cipher_hd );
 	/* now let's see whether we have used the right passphrase */
 	if( csum != sk->csum ) {
 	    copy_secret_key( sk, save_sk );
-            passphrase_clear_cache ( keyid, sk->pubkey_algo );
 	    free_secret_key( save_sk );
 	    return G10ERR_BAD_PASS;
 	}
@@ -174,7 +142,6 @@ do_check( PKT_secret_key *sk, const char *tryagain_text )
 	res = pubkey_check_secret_key( sk->pubkey_algo, sk->skey );
 	if( res ) {
 	    copy_secret_key( sk, save_sk );
-            passphrase_clear_cache ( keyid, sk->pubkey_algo );
 	    free_secret_key( save_sk );
 	    return G10ERR_BAD_PASS;
 	}
@@ -207,15 +174,12 @@ check_secret_key( PKT_secret_key *sk, int n )
     int i;
 
     if( n < 1 )
-	n = (opt.batch && !opt.use_agent)? 1 : 3; /* use the default value */
+	n = opt.batch? 1 : 3; /* use the default value */
 
     for(i=0; i < n && rc == G10ERR_BAD_PASS; i++ ) {
-        const char *tryagain = NULL;
-	if (i) {
-            tryagain = _("Invalid passphrase; please try again");
-            log_info (_("%s ...\n"), tryagain);
-        }
-	rc = do_check( sk, tryagain );
+	if( i )
+	    log_info(_("Invalid passphrase; please try again ...\n"));
+	rc = do_check( sk );
 	if( rc == G10ERR_BAD_PASS && is_status_enabled() ) {
 	    u32 kid[2];
 	    char buf[50];
@@ -261,27 +225,46 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 	return 0;
 
     if( !sk->is_protected ) { /* okay, apply the protection */
-	CIPHER_HANDLE cipher_hd=NULL;
+	GCRY_CIPHER_HD cipher_hd=NULL;
 
-	if( check_cipher_algo( sk->protect.algo ) )
+	if( openpgp_cipher_test_algo( sk->protect.algo ) )
 	    rc = G10ERR_CIPHER_ALGO; /* unsupport protection algorithm */
 	else {
 	    print_cipher_algo_note( sk->protect.algo );
-	    cipher_hd = cipher_open( sk->protect.algo,
-				     CIPHER_MODE_AUTO_CFB, 1 );
-	    if( cipher_setkey( cipher_hd, dek->key, dek->keylen ) )
+	    if( !(cipher_hd = gcry_cipher_open( sk->protect.algo,
+					  GCRY_CIPHER_MODE_CFB,
+					  GCRY_CIPHER_SECURE
+					  | (sk->protect.algo >= 100 ?
+					      0 : GCRY_CIPHER_ENABLE_SYNC) ))
+					 ) {
+		BUG();
+	    }
+
+
+	    rc = gcry_cipher_setkey( cipher_hd, dek->key, dek->keylen );
+	    if( rc == GCRYERR_WEAK_KEY ) {
 		log_info(_("WARNING: Weak key detected"
 			   " - please change passphrase again.\n"));
-	    sk->protect.ivlen = cipher_get_blocksize( sk->protect.algo );
+		rc = 0;
+	    }
+	    else if( rc )
+		BUG();
+
+	    /* set the IV length */
+	    {	int blocksize = gcry_cipher_get_algo_blklen( sk->protect.algo );
+		if( blocksize != 8 && blocksize != 16 )
+		    log_fatal("unsupported blocksize %d\n", blocksize );
+		sk->protect.ivlen = blocksize;
+	    }
+
 	    assert( sk->protect.ivlen <= DIM(sk->protect.iv) );
-	    if( sk->protect.ivlen != 8 && sk->protect.ivlen != 16 )
-		BUG(); /* yes, we are very careful */
 	    randomize_buffer(sk->protect.iv, sk->protect.ivlen, 1);
-	    cipher_setiv( cipher_hd, sk->protect.iv, sk->protect.ivlen );
+	    gcry_cipher_setiv( cipher_hd, sk->protect.iv, sk->protect.ivlen );
 	    if( sk->version >= 4 ) {
-                byte *bufarr[PUBKEY_MAX_NSKEY];
-		unsigned narr[PUBKEY_MAX_NSKEY];
-		unsigned nbits[PUBKEY_MAX_NSKEY];
+	      #define NMPIS (PUBKEY_MAX_NSKEY - PUBKEY_MAX_NPKEY)
+		byte *bufarr[NMPIS];
+		unsigned narr[NMPIS];
+		unsigned nbits[NMPIS];
 		int ndata=0;
 		byte *p, *data;
 
@@ -292,13 +275,13 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    nbits[j]  = mpi_get_nbits( sk->skey[i] );
 		    ndata += narr[j] + 2;
 		}
-		for( ; j < PUBKEY_MAX_NSKEY; j++ )
+		for( ; j < NMPIS; j++ )
 		    bufarr[j] = NULL;
-		ndata += opt.simple_sk_checksum? 2 : 20; /* for checksum */
+		ndata += 2; /* for checksum */
 
 		data = m_alloc_secure( ndata );
 		p = data;
-		for(j=0; j < PUBKEY_MAX_NSKEY && bufarr[j]; j++ ) {
+		for(j=0; j < NMPIS && bufarr[j]; j++ ) {
 		    p[0] = nbits[j] >> 8 ;
 		    p[1] = nbits[j];
 		    p += 2;
@@ -306,31 +289,13 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    p += narr[j];
 		    m_free(bufarr[j]);
 		}
-                
-                if (opt.simple_sk_checksum) {
-                    log_info (_("generating the deprecated 16-bit checksum"
-                              " for secret key protection\n")); 
-                    csum = checksum( data, ndata-2);
-                    sk->csum = csum;
-                    *p++ =	csum >> 8;
-                    *p++ =	csum;
-                    sk->protect.sha1chk = 0;
-                }
-                else {
-                    MD_HANDLE h = md_open (DIGEST_ALGO_SHA1, 1);
-                    if (!h)
-                        BUG(); /* algo not available */
-                    md_write (h, data, ndata - 20);
-                    md_final (h);
-                    memcpy (p, md_read (h, DIGEST_ALGO_SHA1), 20);
-                    p += 20;
-                    md_close (h);
-                    sk->csum = csum = 0;
-                    sk->protect.sha1chk = 1;
-                }
-                assert( p == data+ndata );
-
-		cipher_encrypt( cipher_hd, data, data, ndata );
+	      #undef NMPIS
+		csum = checksum( data, ndata-2);
+		sk->csum = csum;
+		*p++ =	csum >> 8;
+		*p++ =	csum;
+		assert( p == data+ndata );
+		gcry_cipher_encrypt( cipher_hd, data, ndata, NULL, 0 );
 		for(i = pubkey_get_npkey(sk->pubkey_algo);
 			i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
 		    mpi_free( sk->skey[i] );
@@ -347,9 +312,9 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 			i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
 		    csum += checksum_mpi_counted_nbits( sk->skey[i] );
 		    buffer = mpi_get_buffer( sk->skey[i], &nbytes, NULL );
-		    cipher_sync( cipher_hd );
+		    gcry_cipher_sync( cipher_hd );
 		    assert( !mpi_is_protected(sk->skey[i]) );
-		    cipher_encrypt( cipher_hd, buffer, buffer, nbytes );
+		    gcry_cipher_encrypt( cipher_hd, buffer, nbytes, NULL, 0 );
 		    mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
 		    mpi_set_protect_flag( sk->skey[i] );
 		    m_free( buffer );
@@ -357,7 +322,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		sk->csum = csum;
 	    }
 	    sk->is_protected = 1;
-	    cipher_close( cipher_hd );
+	    gcry_cipher_close( cipher_hd );
 	}
     }
     return rc;
