@@ -1,5 +1,5 @@
 /* sig-check.c -  Check a signature
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,51 +23,56 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+
+#include <gcrypt.h>
 #include "util.h"
 #include "packet.h"
 #include "memory.h"
 #include "mpi.h"
 #include "keydb.h"
-#include "cipher.h"
 #include "main.h"
 #include "status.h"
 #include "i18n.h"
-#include "options.h"
 
 struct cmp_help_context_s {
     PKT_signature *sig;
-    MD_HANDLE md;
+    GCRY_MD_HD md;
 };
 
+
+static int do_signature_check( PKT_signature *sig, GCRY_MD_HD digest,
+						      u32 *r_expire );
 static int do_check( PKT_public_key *pk, PKT_signature *sig,
-					 MD_HANDLE digest, int *r_expired );
+						GCRY_MD_HD digest );
+
 
 /****************
  * Check the signature which is contained in SIG.
- * The MD_HANDLE should be currently open, so that this function
+ * The GCRY_MD_HD should be currently open, so that this function
  * is able to append some data, before finalizing the digest.
  */
 int
-signature_check( PKT_signature *sig, MD_HANDLE digest )
+signature_check( PKT_signature *sig, GCRY_MD_HD digest )
 {
     u32 dummy;
-    int dum2;
-    return signature_check2( sig, digest, &dummy, &dum2 );
+    return do_signature_check( sig, digest, &dummy );
 }
 
-int
-signature_check2( PKT_signature *sig, MD_HANDLE digest,
-		  u32 *r_expiredate, int *r_expired )
+static int
+do_signature_check( PKT_signature *sig, GCRY_MD_HD digest, u32 *r_expire )
 {
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
     int rc=0;
 
-    *r_expiredate = 0;
+    if( is_RSA(sig->pubkey_algo) )
+	write_status(STATUS_RSA_OR_IDEA);
+
+    *r_expire = 0;
     if( get_pubkey( pk, sig->keyid ) )
 	rc = G10ERR_NO_PUBKEY;
     else {
-	*r_expiredate = pk->expiredate;
-	rc = do_check( pk, sig, digest, r_expired );
+	*r_expire = pk->expiredate;
+	rc = do_check( pk, sig, digest );
     }
 
     free_public_key( pk );
@@ -78,44 +83,133 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest,
 	 * this sig-id we could have also used the hash of the document
 	 * and the timestamp, but the drawback of this is, that it is
 	 * not possible to sign more than one identical document within
-	 * one second.	Some remote batch processing applications might
+	 * one second.	Some remote bacth processing applications might
 	 * like this feature here */
-	MD_HANDLE md;
+	GCRY_MD_HD md;
 	u32 a = sig->timestamp;
 	int i, nsig = pubkey_get_nsig( sig->pubkey_algo );
 	byte *p, *buffer;
 
-	md = md_open( DIGEST_ALGO_RMD160, 0);
-	md_putc( digest, sig->pubkey_algo );
-	md_putc( digest, sig->digest_algo );
-	md_putc( digest, (a >> 24) & 0xff );
-	md_putc( digest, (a >> 16) & 0xff );
-	md_putc( digest, (a >>	8) & 0xff );
-	md_putc( digest,  a	   & 0xff );
+	if( !(md = gcry_md_open( DIGEST_ALGO_RMD160, 0)) )
+	    BUG();
+	gcry_md_putc( digest, sig->pubkey_algo );
+	gcry_md_putc( digest, sig->digest_algo );
+	gcry_md_putc( digest, (a >> 24) & 0xff );
+	gcry_md_putc( digest, (a >> 16) & 0xff );
+	gcry_md_putc( digest, (a >>  8) & 0xff );
+	gcry_md_putc( digest,  a	& 0xff );
 	for(i=0; i < nsig; i++ ) {
 	    unsigned n = mpi_get_nbits( sig->data[i]);
 
-	    md_putc( md, n>>8);
-	    md_putc( md, n );
+	    gcry_md_putc( md, n>>8);
+	    gcry_md_putc( md, n );
 	    p = mpi_get_buffer( sig->data[i], &n, NULL );
-	    md_write( md, p, n );
+	    gcry_md_write( md, p, n );
 	    m_free(p);
 	}
-	md_final( md );
-	p = make_radix64_string( md_read( md, 0 ), 20 );
+	gcry_md_final( md );
+	p = make_radix64_string( gcry_md_read( md, 0 ), 20 );
 	buffer = m_alloc( strlen(p) + 60 );
 	sprintf( buffer, "%s %s %lu",
 		 p, strtimestamp( sig->timestamp ), (ulong)sig->timestamp );
 	write_status_text( STATUS_SIG_ID, buffer );
 	m_free(buffer);
 	m_free(p);
-	md_close(md);
+	gcry_md_close(md);
     }
 
     return rc;
 }
 
 
+#if 0 /* not anymore used */
+/****************
+ * Check the MDC which is contained in SIG.
+ * The GCRY_MD_HD should be currently open, so that this function
+ * is able to append some data, before finalizing the digest.
+ */
+int
+mdc_kludge_check( PKT_signature *sig, GCRY_MD_HD digest )
+{
+    int rc=0;
+
+    if( (rc=check_digest_algo(sig->digest_algo)) )
+	return rc;
+
+    /* make sure the digest algo is enabled (in case of a detached mdc??) */
+    md_enable( digest, sig->digest_algo );
+
+    /* complete the digest */
+    if( sig->version >= 4 )
+	gcry_md_putc( digest, sig->version );
+    gcry_md_putc( digest, sig->sig_class );
+    if( sig->version < 4 ) {
+	u32 a = sig->timestamp;
+	gcry_md_putc( digest, (a >> 24) & 0xff );
+	gcry_md_putc( digest, (a >> 16) & 0xff );
+	gcry_md_putc( digest, (a >>  8) & 0xff );
+	gcry_md_putc( digest,  a	& 0xff );
+    }
+    else {
+	byte buf[6];
+	size_t n;
+	gcry_md_putc( digest, sig->pubkey_algo );
+	gcry_md_putc( digest, sig->digest_algo );
+	if( sig->hashed_data ) {
+	    n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
+	    gcry_md_write( digest, sig->hashed_data, n+2 );
+	    n += 6;
+	}
+	else
+	    n = 6;
+	/* add some magic */
+	buf[0] = sig->version;
+	buf[1] = 0xff;
+	buf[2] = n >> 24;
+	buf[3] = n >> 16;
+	buf[4] = n >>  8;
+	buf[5] = n;
+	gcry_md_write( digest, buf, 6 );
+    }
+    md_final( digest );
+
+    rc = G10ERR_BAD_SIGN;
+    {	const byte *s1 = md_read( digest, sig->digest_algo );
+	int s1len = md_digest_length( sig->digest_algo );
+
+	log_hexdump( "MDC calculated", s1, s1len );
+
+	if( !sig->data[0] )
+	    log_debug("sig_data[0] is NULL\n");
+	else {
+	    unsigned s2len;
+	    byte *s2;
+	    s2 = mpi_get_buffer( sig->data[0], &s2len, NULL );
+	    log_hexdump( "MDC stored    ", s2, s2len );
+
+	    if( s2len != s1len )
+		log_debug("MDC check: len differ: %d/%d\n", s1len, s2len);
+	    else if( memcmp( s1, s2, s1len ) )
+		log_debug("MDC check: hashs differ\n");
+	    else
+		rc = 0;
+	    m_free(s2);
+	}
+    }
+
+    if( !rc && sig->flags.unknown_critical ) {
+	log_info(_("assuming bad MDC due to an unknown critical bit\n"));
+	rc = G10ERR_BAD_SIGN;
+    }
+    sig->flags.checked = 1;
+    sig->flags.valid = !rc;
+
+    /* FIXME: check that we are actually in an encrypted packet */
+
+    return rc;
+}
+#endif
+
 /****************
  * This function gets called by pubkey_verify() if the algorithm needs it.
  */
@@ -129,7 +223,7 @@ cmp_help( void *opaque, MPI result )
     size_t mdlen, asnlen;
     struct cmp_help_context_s *ctx = opaque;
     PKT_signature *sig = ctx->sig;
-    MD_HANDLE digest = ctx->md;
+    GCRY_MD_HD digest = ctx->md;
 
     old_enc = 0;
     for(i=j=0; (c=mpi_getbyte(result, i)) != -1; i++ ) {
@@ -192,15 +286,13 @@ cmp_help( void *opaque, MPI result )
 
 
 static int
-do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
-						    int *r_expired )
+do_check( PKT_public_key *pk, PKT_signature *sig, GCRY_MD_HD digest )
 {
     MPI result = NULL;
     int rc=0;
     struct cmp_help_context_s ctx;
     u32 cur_time;
 
-    *r_expired = 0;
     if( pk->version == 4 && pk->pubkey_algo == PUBKEY_ALGO_ELGAMAL_E ) {
 	log_info(_("this is a PGP generated "
 		  "ElGamal key which is NOT secure for signatures!\n"));
@@ -213,8 +305,7 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 		  ? _("public key is %lu second newer than the signature\n")
 		  : _("public key is %lu seconds newer than the signature\n"),
 		       d );
-	if( !opt.ignore_time_conflict )
-	    return G10ERR_TIME_CONFLICT; /* pubkey newer than signature */
+	return G10ERR_TIME_CONFLICT; /* pubkey newer than signature */
     }
 
     cur_time = make_timestamp();
@@ -224,56 +315,43 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 			   "in future (time warp or clock problem)\n")
 		       : _("key has been created %lu seconds "
 			   "in future (time warp or clock problem)\n"), d );
-	if( !opt.ignore_time_conflict )
-	    return G10ERR_TIME_CONFLICT;
+	return G10ERR_TIME_CONFLICT;
     }
 
     if( pk->expiredate && pk->expiredate < cur_time ) {
-        char buf[11];
-        if (opt.verbose) {
-	    u32 tmp_kid[2];
-
-	    keyid_from_pk( pk, tmp_kid );
-            log_info(_("NOTE: signature key %08lX expired %s\n"),
-                     (ulong)tmp_kid[1], asctimestamp( pk->expiredate ) );
-        }
-	/* SIGEXPIRED is deprecated.  Use KEYEXPIRED. */
-	sprintf(buf,"%lu",(ulong)pk->expiredate);
-	write_status_text(STATUS_KEYEXPIRED,buf);
+	log_info(_("NOTE: signature key expired %s\n"),
+					asctimestamp( pk->expiredate ) );
 	write_status(STATUS_SIGEXPIRED);
-	*r_expired = 1;
     }
 
 
-    if( (rc=check_digest_algo(sig->digest_algo)) )
+    if( (rc=openpgp_md_test_algo(sig->digest_algo)) )
 	return rc;
-    if( (rc=check_pubkey_algo(sig->pubkey_algo)) )
+    if( (rc=openpgp_pk_test_algo(sig->pubkey_algo)) )
 	return rc;
 
     /* make sure the digest algo is enabled (in case of a detached signature)*/
-    md_enable( digest, sig->digest_algo );
+    gcry_md_enable( digest, sig->digest_algo );
 
     /* complete the digest */
     if( sig->version >= 4 )
-	md_putc( digest, sig->version );
-    md_putc( digest, sig->sig_class );
+	gcry_md_putc( digest, sig->version );
+    gcry_md_putc( digest, sig->sig_class );
     if( sig->version < 4 ) {
 	u32 a = sig->timestamp;
-	md_putc( digest, (a >> 24) & 0xff );
-	md_putc( digest, (a >> 16) & 0xff );
-	md_putc( digest, (a >>	8) & 0xff );
-	md_putc( digest,  a	   & 0xff );
+	gcry_md_putc( digest, (a >> 24) & 0xff );
+	gcry_md_putc( digest, (a >> 16) & 0xff );
+	gcry_md_putc( digest, (a >>  8) & 0xff );
+	gcry_md_putc( digest,  a	& 0xff );
     }
     else {
 	byte buf[6];
 	size_t n;
-	md_putc( digest, sig->pubkey_algo );
-	md_putc( digest, sig->digest_algo );
-	if( sig->hashed ) {
-	    n = sig->hashed->len;
-            md_putc (digest, (n >> 8) );
-            md_putc (digest,  n       );
-	    md_write (digest, sig->hashed->data, n);
+	gcry_md_putc( digest, sig->pubkey_algo );
+	gcry_md_putc( digest, sig->digest_algo );
+	if( sig->hashed_data ) {
+	    n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
+	    gcry_md_write( digest, sig->hashed_data, n+2 );
 	    n += 6;
 	}
 	else
@@ -285,91 +363,45 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
 	buf[3] = n >> 16;
 	buf[4] = n >>  8;
 	buf[5] = n;
-	md_write( digest, buf, 6 );
+	gcry_md_write( digest, buf, 6 );
     }
-    md_final( digest );
+    gcry_md_final( digest );
 
     result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
-			      mpi_get_nbits(pk->pkey[0]), 0 );
-    if (!result)
-        return G10ERR_GENERAL;
+				      mpi_get_nbits(pk->pkey[0]));
+
     ctx.sig = sig;
     ctx.md = digest;
     rc = pubkey_verify( pk->pubkey_algo, result, sig->data, pk->pkey,
 			cmp_help, &ctx );
     mpi_free( result );
-    if( (opt.emulate_bugs & EMUBUG_MDENCODE)
-	&& rc == G10ERR_BAD_SIGN && is_ELGAMAL(pk->pubkey_algo) ) {
-	/* In this case we try again because old GnuPG versions didn't encode
-	 * the hash right. There is no problem with DSA however  */
-	result = encode_md_value( pk->pubkey_algo, digest, sig->digest_algo,
-			      mpi_get_nbits(pk->pkey[0]), (sig->version < 5) );
-        if (!result)
-            rc = G10ERR_GENERAL;
-        else {
-            ctx.sig = sig;
-            ctx.md = digest;
-            rc = pubkey_verify( pk->pubkey_algo, result, sig->data, pk->pkey,
-                                cmp_help, &ctx );
-        }
-    }
-
     if( !rc && sig->flags.unknown_critical ) {
 	log_info(_("assuming bad signature due to an unknown critical bit\n"));
 	rc = G10ERR_BAD_SIGN;
     }
+    sig->flags.checked = 1;
+    sig->flags.valid = !rc;
 
     return rc;
 }
 
 
 static void
-hash_uid_node( KBNODE unode, MD_HANDLE md, PKT_signature *sig )
+hash_uid_node( KBNODE unode, GCRY_MD_HD md, PKT_signature *sig )
 {
     PKT_user_id *uid = unode->pkt->pkt.user_id;
 
     assert( unode->pkt->pkttype == PKT_USER_ID );
-    if( uid->attrib_data ) {
-	if( sig->version >=4 ) {
-	    byte buf[5];
-	    buf[0] = 0xd1;		     /* packet of type 17 */
-	    buf[1] = uid->attrib_len >> 24;  /* always use 4 length bytes */
-	    buf[2] = uid->attrib_len >> 16;
-	    buf[3] = uid->attrib_len >>  8;
-	    buf[4] = uid->attrib_len;
-	    md_write( md, buf, 5 );
-	}
-	md_write( md, uid->attrib_data, uid->attrib_len );
-    }
-    else {
-	if( sig->version >=4 ) {
-	    byte buf[5];
-	    buf[0] = 0xb4;	      /* indicates a userid packet */
-	    buf[1] = uid->len >> 24;  /* always use 4 length bytes */
-	    buf[2] = uid->len >> 16;
-	    buf[3] = uid->len >>  8;
-	    buf[4] = uid->len;
-	    md_write( md, buf, 5 );
-	}
-	md_write( md, uid->name, uid->len );
-    }
-}
-
-static void
-cache_sig_result ( PKT_signature *sig, int result )
-{
-    if ( !result ) {
-        sig->flags.checked = 1;
-        sig->flags.valid = 1;
-    }
-    else if ( result == G10ERR_BAD_SIGN ) {
-        sig->flags.checked = 1;
-        sig->flags.valid = 0;
-    }
-    else {
-        sig->flags.checked = 0;
-        sig->flags.valid = 0;
+    if( sig->version >=4 ) {
+	byte buf[5];
+	buf[0] = 0xb4; /* indicates a userid packet */
+	buf[1] = uid->len >> 24;  /* always use 4 length bytes */
+	buf[2] = uid->len >> 16;
+	buf[3] = uid->len >>  8;
+	buf[4] = uid->len;
+	gcry_md_write( md, buf, 5 );
     }
+    gcry_md_write( md, uid->name, uid->len );
 }
 
 /****************
@@ -381,15 +413,13 @@ int
 check_key_signature( KBNODE root, KBNODE node, int *is_selfsig )
 {
     u32 dummy;
-    int dum2;
-    return check_key_signature2(root, node, is_selfsig, &dummy, &dum2 );
+    return check_key_signature2(root, node, is_selfsig, &dummy );
 }
 
 int
-check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
-				       u32 *r_expiredate, int *r_expired )
+check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig, u32 *r_expire)
 {
-    MD_HANDLE md;
+    GCRY_MD_HD md;
     PKT_public_key *pk;
     PKT_signature *sig;
     int algo;
@@ -397,8 +427,7 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 
     if( is_selfsig )
 	*is_selfsig = 0;
-    *r_expiredate = 0;
-    *r_expired = 0;
+    *r_expire = 0;
     assert( node->pkt->pkttype == PKT_SIGNATURE );
     assert( root->pkt->pkttype == PKT_PUBLIC_KEY );
 
@@ -406,49 +435,39 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
     sig = node->pkt->pkt.signature;
     algo = sig->digest_algo;
 
-    /* check whether we have cached the result of a previous signature check.*/
-    if ( !opt.no_sig_cache ) {
-        if (sig->flags.checked) { /*cached status available*/
-	    if( is_selfsig ) {	
-		u32 keyid[2];	
+  #if 0 /* I am not sure whether this is a good thing to do */
+    if( sig->flags.checked )
+	log_debug("check_key_signature: already checked: %s\n",
+		      sig->flags.valid? "good":"bad" );
+  #endif
 
-		keyid_from_pk( pk, keyid );
-		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
-		    *is_selfsig = 1;
-	    }
-            return sig->flags.valid? 0 : G10ERR_BAD_SIGN;
-        }
-    }
-
-    if( (rc=check_digest_algo(algo)) )
+    if( (rc=openpgp_md_test_algo(algo)) )
 	return rc;
 
-    if( sig->sig_class == 0x20 ) { /* key revocation */
-	md = md_open( algo, 0 );
+    if( sig->sig_class == 0x20 ) {
+	if( !(md = gcry_md_open( algo, 0 )) )
+	    BUG();
 	hash_public_key( md, pk );
-	rc = do_check( pk, sig, md, r_expired );
-        cache_sig_result ( sig, rc );
-	md_close(md);
+	rc = do_check( pk, sig, md );
+	gcry_md_close(md);
     }
     else if( sig->sig_class == 0x28 ) { /* subkey revocation */
 	KBNODE snode = find_prev_kbnode( root, node, PKT_PUBLIC_SUBKEY );
 
 	if( snode ) {
-	    md = md_open( algo, 0 );
+	    if( !(md = gcry_md_open( algo, 0 )) )
+		BUG();
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md, r_expired );
-            cache_sig_result ( sig, rc );
-	    md_close(md);
+	    rc = do_check( pk, sig, md );
+	    gcry_md_close(md);
 	}
 	else {
-            if (!opt.quiet)
-                log_info ("key %08lX: no subkey for subkey revocation packet\n",
-                          (ulong)keyid_from_pk (pk, NULL));
+	    log_error("no subkey for subkey revocation packet\n");
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
-    else if( sig->sig_class == 0x18 ) { /* key binding */
+    else if( sig->sig_class == 0x18 ) {
 	KBNODE snode = find_prev_kbnode( root, node, PKT_PUBLIC_SUBKEY );
 
 	if( snode ) {
@@ -459,53 +478,40 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 		if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] )
 		    *is_selfsig = 1;
 	    }
-	    md = md_open( algo, 0 );
+	    if( !(md = gcry_md_open( algo, 0 )) )
+		BUG();
 	    hash_public_key( md, pk );
 	    hash_public_key( md, snode->pkt->pkt.public_key );
-	    rc = do_check( pk, sig, md, r_expired );
-            cache_sig_result ( sig, rc );
-	    md_close(md);
+	    rc = do_check( pk, sig, md );
+	    gcry_md_close(md);
 	}
 	else {
-            if (!opt.quiet)
-                log_info ("key %08lX: no subkey for subkey binding packet\n",
-                          (ulong)keyid_from_pk (pk, NULL));
+	    log_error("no subkey for key signature packet\n");
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
-    else if( sig->sig_class == 0x1f ) { /* direct key signature */
-	md = md_open( algo, 0 );
-	hash_public_key( md, pk );
-	rc = do_check( pk, sig, md, r_expired );
-        cache_sig_result ( sig, rc );
-	md_close(md);
-    }
-    else { /* all other classes */
+    else {
 	KBNODE unode = find_prev_kbnode( root, node, PKT_USER_ID );
 
 	if( unode ) {
 	    u32 keyid[2];
 
 	    keyid_from_pk( pk, keyid );
-	    md = md_open( algo, 0 );
+	    if( !(md = gcry_md_open( algo, 0 )) )
+		BUG();
 	    hash_public_key( md, pk );
 	    hash_uid_node( unode, md, sig );
 	    if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] ) {
 		if( is_selfsig )
 		    *is_selfsig = 1;
-		rc = do_check( pk, sig, md, r_expired );
+		rc = do_check( pk, sig, md );
 	    }
-	    else {
-		rc = signature_check2( sig, md, r_expiredate, r_expired );
-	    }
-            cache_sig_result ( sig, rc );
-	    md_close(md);
+	    else
+		rc = do_signature_check( sig, md, r_expire );
+	    gcry_md_close(md);
 	}
 	else {
-            if (!opt.quiet)
-                log_info ("key %08lX: no user ID for key signature packet "
-                          "of class %02x\n",
-                          (ulong)keyid_from_pk (pk, NULL), sig->sig_class );
+	    log_error("no user ID for key signature packet\n");
 	    rc = G10ERR_SIG_CLASS;
 	}
     }
