@@ -1,5 +1,5 @@
 /* tdbdump.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -43,8 +43,8 @@
 #include "tdbio.h"
 
 
-#define HEXTOBIN(x) ( (x) >= '0' && (x) <= '9' ? ((x)-'0') : \
-		      (x) >= 'A' && (x) <= 'F' ? ((x)-'A'+10) : ((x)-'a'+10))
+#define HEXTOBIN(a) ( (a) >= '0' && (a) <= '9' ? ((a)-'0') : \
+		      (a) >= 'A' && (a) <= 'F' ? ((a)-'A'+10) : ((a)-'a'+10))
 
 /****************
  * Read a record but die if it does not exist
@@ -520,6 +520,5 @@ import_ownertrust( const char *fname )
     if( !is_stdin )
 	fclose(fp);
     do_sync();
-    sync_trustdb();
 }
 
