@@ -1,5 +1,5 @@
 /* trustdb.c
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -109,7 +109,7 @@ static TN used_tns;
 static int alloced_tns;
 static int max_alloced_tns;
 
-static struct keyid_list *trusted_key_list;
+
 
 static LOCAL_ID_TABLE new_lid_table(void);
 static int ins_lid_table_item( LOCAL_ID_TABLE tbl, ulong lid, unsigned flag );
@@ -125,22 +125,13 @@ static int do_check( TRUSTREC *drec, unsigned *trustlevel,
 						unsigned *retflgs);
 static int get_dir_record( PKT_public_key *pk, TRUSTREC *rec );
 static int do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
-				   int sigs_only, int *modified );
-static int check_trust_record( TRUSTREC *drec, int sigs_only );
-static void mark_fresh_keys(void);
+					  int recheck, int *modified );
+static int check_trust_record( TRUSTREC *drec );
 
 /* a table used to keep track of ultimately trusted keys
  * which are the ones from our secrings and the trusted keys */
 static LOCAL_ID_TABLE ultikey_table;
 
-
-/* a table to keep track of newly importted keys.  This one is
- * create by the insert_trust_record function and from time to time
- * used to verify key signature which have been done with these new keys */
-static LOCAL_ID_TABLE fresh_imported_keys;
-static int fresh_imported_keys_count;
-#define FRESH_KEY_CHECK_THRESHOLD 200
-
 /* list of unused lid items and tables */
 static LOCAL_ID_TABLE unused_lid_tables;
 static struct local_id_item *unused_lid_items;
@@ -254,27 +245,6 @@ release_lid_table( LOCAL_ID_TABLE tbl )
 }
 #endif
 
-
-/****************
- * Remove all items from a LID table
- */
-static void
-clear_lid_table( LOCAL_ID_TABLE tbl )
-{
-    struct local_id_item *a, *a2;
-    int i;
-
-    for(i=0; i < 16; i++ ) {
-	for(a=tbl->items[i]; a; a = a2 ) {
-	    a2 = a->next;
-	    a->next = unused_lid_items;
-	    unused_lid_items = a;
-	}
-	tbl->items[i] = NULL;
-    }
-}
-
-
 /****************
  * Add a new item to the table or return 1 if we already have this item
  */
@@ -464,63 +434,6 @@ lid_from_keyid_no_sdir( u32 *keyid )
  *************	Initialization	****************
  ***********************************************/
 
-void
-register_trusted_key( const char *string )
-{
-    KEYDB_SEARCH_DESC desc;
-    struct keyid_list *r;
-
-    if (classify_user_id (string, &desc) != KEYDB_SEARCH_MODE_LONG_KID ) {
-        log_error(_("`%s' is not a valid long keyID\n"), string );
-        return;
-    }
-
-    for( r = trusted_key_list; r; r = r->next )
-        if( r->keyid[0] == desc.u.kid[0] && r->keyid[1] == desc.u.kid[1] )
-            return;
-    r = m_alloc( sizeof *r );
-    r->keyid[0] = desc.u.kid[0];
-    r->keyid[1] = desc.u.kid[1];
-    r->next = trusted_key_list;
-    trusted_key_list = r;
-}
-
-
-static void
-add_ultimate_key( PKT_public_key *pk, u32 *keyid )
-{
-    int rc;
-
-    /* first make sure that the pubkey is in the trustdb */
-    rc = query_trust_record( pk );
-    if( rc == -1 && opt.dry_run )
-	return;
-    if( rc == -1 ) { /* put it into the trustdb */
-        rc = insert_trust_record_by_pk( pk );
-        if( rc ) {
-            log_error(_("key %08lX: can't put it into the trustdb\n"),
-                      (ulong)keyid[1] );
-            return;
-        }
-    }
-    else if( rc ) {
-        log_error(_("key %08lX: query record failed\n"), (ulong)keyid[1] );
-        return;
-    }
-
-    if( DBG_TRUST )
-        log_debug("key %08lX.%lu: stored into ultikey_table\n",
-                  (ulong)keyid[1], pk->local_id );
-
-    if( ins_lid_table_item( ultikey_table, pk->local_id, 0 ) )
-        log_info(_("key %08lX: already in trusted key table\n"),
-                  (ulong)keyid[1]);
-    else if( opt.verbose > 1 )
-        log_info(_("key %08lX: accepted as trusted key.\n"),
-                 (ulong)keyid[1]);
-
-}
-
 /****************
  * Verify that all our public keys are in the trustdb.
  */
@@ -532,27 +445,7 @@ verify_own_keys(void)
     PKT_secret_key *sk = m_alloc_clear( sizeof *sk );
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
     u32 keyid[2];
-    struct keyid_list *kl;
 
-
-    /* put the trusted keys into the ultikey table */
-    for( kl = trusted_key_list; kl; kl = kl->next ) {
-        keyid[0] = kl->keyid[0];
-        keyid[1] = kl->keyid[1];
-        /* get the public key */
-        memset( pk, 0, sizeof *pk );
-        rc = get_pubkey( pk, keyid );
-        if( rc ) {
-            log_info(_("key %08lX: no public key for trusted key - skipped\n"),
-                                                            (ulong)keyid[1] );
-        }
-        else {
-            add_ultimate_key( pk, keyid );
-            release_public_key_parts( pk );
-        }
-    }
-
-    /* And now add all secret keys to the ultikey table */
     while( !(rc=enum_secret_keys( &enum_context, sk, 0 ) ) ) {
 	int have_pk = 0;
 
@@ -561,22 +454,17 @@ verify_own_keys(void)
 	if( DBG_TRUST )
 	    log_debug("key %08lX: checking secret key\n", (ulong)keyid[1] );
 
-	if( !opt.quiet && is_secret_key_protected( sk ) < 1 )
+	if( is_secret_key_protected( sk ) < 1 )
 	    log_info(_("NOTE: secret key %08lX is NOT protected.\n"),
 							    (ulong)keyid[1] );
 
-        for( kl = trusted_key_list; kl; kl = kl->next ) {
-            if( kl->keyid[0] == keyid[0] && kl->keyid[1] == keyid[1] )
-                goto skip; /* already in trusted key table */
-        }
 
 	/* see whether we can access the public key of this secret key */
 	memset( pk, 0, sizeof *pk );
 	rc = get_pubkey( pk, keyid );
 	if( rc ) {
-            if (!opt.quiet)
-                log_info(_("key %08lX: secret key without public key "
-                           "- skipped\n"),  (ulong)keyid[1] );
+	    log_info(_("key %08lX: secret key without public key - skipped\n"),
+							    (ulong)keyid[1] );
 	    goto skip;
 	}
 	have_pk=1;
@@ -587,8 +475,33 @@ verify_own_keys(void)
 	    goto skip;
 	}
 
-	add_ultimate_key( pk, keyid );
+	/* make sure that the pubkey is in the trustdb */
+	rc = query_trust_record( pk );
+	if( rc == -1 && opt.dry_run )
+	    goto skip;
+	if( rc == -1 ) { /* put it into the trustdb */
+	    rc = insert_trust_record_by_pk( pk );
+	    if( rc ) {
+		log_error(_("key %08lX: can't put it into the trustdb\n"),
+							    (ulong)keyid[1] );
+		goto skip;
+	    }
+	}
+	else if( rc ) {
+	    log_error(_("key %08lX: query record failed\n"), (ulong)keyid[1] );
+	    goto skip;
 
+	}
+
+	if( DBG_TRUST )
+	    log_debug("key %08lX.%lu: stored into ultikey_table\n",
+				    (ulong)keyid[1], pk->local_id );
+	if( ins_lid_table_item( ultikey_table, pk->local_id, 0 ) )
+	    log_error(_("key %08lX: already in trusted key table\n"),
+							(ulong)keyid[1]);
+	else if( opt.verbose > 1 )
+	    log_info(_("key %08lX: accepted as trusted key.\n"),
+							(ulong)keyid[1]);
       skip:
 	release_secret_key_parts( sk );
 	if( have_pk )
@@ -599,15 +512,6 @@ verify_own_keys(void)
     else
 	rc = 0;
 
-    /* release the trusted keyid table */
-    {   struct keyid_list *kl2;
-        for( kl = trusted_key_list; kl; kl = kl2 ) {
-            kl2 = kl->next;
-            m_free( kl );
-        }
-        trusted_key_list = NULL;
-    }
-
     enum_secret_keys( &enum_context, NULL, 0 ); /* free context */
     free_secret_key( sk );
     free_public_key( pk );
@@ -668,18 +572,6 @@ init_trustdb()
 
 
 
-/****************
- * This function should be called in certain cases to sync the internal state
- * of the trustdb with the file image.	Currently it is needed after
- * a sequence of insert_trust_record() calls.
- */
-void
-sync_trustdb()
-{
-    if( fresh_imported_keys && fresh_imported_keys_count )
-	mark_fresh_keys();
-}
-
 
 
 /***********************************************
@@ -760,7 +652,7 @@ print_path( int pathlen, TN ME .........., FILE *fp, ulong highlight )
 	p = get_user_id( keyid, &n );
 	putc(' ', fp);
 	putc('\"', fp);
-	print_utf8_string( fp, p, n > 40? 40:n );
+	print_utf8_string( fp, p, n > 40? 40:n, 0 );
 	putc('\"', fp);
 	m_free(p);
 	putc('\n', fp );
@@ -791,10 +683,7 @@ print_uid_from_keyblock( FILE *fp, KBNODE keyblock, ulong urecno )
 	if( node->pkt->pkttype == PKT_USER_ID ) {
 	    PKT_user_id *uidpkt = node->pkt->pkt.user_id;
 
-	    if( uidpkt->photo )
-		rmd160_hash_buffer( uhash, uidpkt->photo, uidpkt->photolen );
-	    else
-		rmd160_hash_buffer( uhash, uidpkt->name, uidpkt->len );
+	    rmd160_hash_buffer( uhash, uidpkt->name, uidpkt->len );
 	    if( !memcmp( uhash, urec.r.uid.namehash, 20 ) ) {
 		print_string( fp,  uidpkt->name, uidpkt->len, ':' );
 		return;
@@ -1111,7 +1000,7 @@ check_uidsigs( KBNODE keyblock, KBNODE keynode, u32 *mainkid, ulong lid,
 			    (ulong)mainkid[1], lid );
 	assert(keynode->pkt->pkttype == PKT_USER_ID );
 	uid = keynode->pkt->pkt.user_id;
-	print_string( log_stream(), uid->name, uid->len, '\"' );
+	print_utf8_string( log_stream(), uid->name, uid->len );
 	fputs("\"\n", log_stream());
     }
 
@@ -1193,17 +1082,17 @@ check_uidsigs( KBNODE keyblock, KBNODE keynode, u32 *mainkid, ulong lid,
 static unsigned int
 check_sig_record( KBNODE keyblock, KBNODE signode,
 		  ulong siglid, int sigidx, u32 *keyid, ulong lid,
-		  u32 *r_expiretime, int *mod_down, int *mod_up )
+							u32 *r_expire )
 {
     PKT_signature *sig = signode->pkt->pkt.signature;
     unsigned int sigflag = 0;
     TRUSTREC tmp;
-    int revocation=0, expired=0, rc;
+    int revocation=0, rc;
 
     if( DBG_TRUST )
 	log_debug("check_sig_record: %08lX.%lu %lu[%d]\n",
 			    (ulong)keyid[1], lid, siglid, sigidx );
-    *r_expiretime = 0;
+    *r_expire = 0;
     if( (sig->sig_class&~3) == 0x10 ) /* regular certification */
 	;
     else if( sig->sig_class == 0x30 ) /* cert revocation */
@@ -1214,8 +1103,7 @@ check_sig_record( KBNODE keyblock, KBNODE signode,
     read_record( siglid, &tmp, 0 );
     if( tmp.rectype == RECTYPE_DIR ) {
 	/* the public key is in the trustdb: check sig */
-	rc = check_key_signature2( keyblock, signode, NULL,
-					     r_expiretime, &expired );
+	rc = check_key_signature2( keyblock, signode, NULL, r_expire );
 	if( !rc ) { /* valid signature */
 	    if( opt.verbose )
 		log_info("sig %08lX.%lu/%lu[%d]/%08lX: %s\n",
@@ -1224,25 +1112,18 @@ check_sig_record( KBNODE keyblock, KBNODE signode,
 			revocation? _("Valid certificate revocation")
 				  : _("Good certificate") );
 	    sigflag |= SIGF_CHECKED | SIGF_VALID;
-	    if( expired ) {
-		sigflag |= SIGF_EXPIRED;
-		/* We have to reset the expiretime, so that this signature
-		 * does not get checked over and over due to the reached
-		 * expiretime */
-		*r_expiretime = 0;
-	    }
 	    if( revocation ) {
 		sigflag |= SIGF_REVOKED;
-		*mod_down = 1;
+		/**mod_down = 1;*/
 	    }
 	    else
-		*mod_up = 1;
+		/**mod_up = 1*/;
 	}
 	else if( rc == G10ERR_NO_PUBKEY ) {
 	    /* This may happen if the key is still in the trustdb
 	     * but not available in the keystorage */
 	    sigflag |= SIGF_NOPUBKEY;
-	    *mod_down = 1;
+	    /**mod_down = 1;*/
 	    if( revocation )
 		sigflag |= SIGF_REVOKED;
 	}
@@ -1256,7 +1137,7 @@ check_sig_record( KBNODE keyblock, KBNODE signode,
 	    sigflag |= SIGF_CHECKED;
 	    if( revocation ) {
 		sigflag |= SIGF_REVOKED;
-		*mod_down = 1;
+		/**mod_down = 1;*/
 	    }
 	}
     }
@@ -1287,15 +1168,14 @@ check_sig_record( KBNODE keyblock, KBNODE signode,
  */
 static ulong
 make_sig_records( KBNODE keyblock, KBNODE uidnode,
-		  ulong lid, u32 *mainkid, u32 *min_expire,
-					int *mod_down, int *mod_up  )
+		  ulong lid, u32 *mainkid, u32 *min_expire )
 {
     TRUSTREC *srecs, **s_end, *s=NULL, *s2;
     KBNODE  node;
     PKT_signature *sig;
     ulong sigrecno, siglid;
     int i, sigidx = 0;
-    u32 expiretime;
+    u32 expire;
 
     srecs = NULL; s_end = &srecs;
     for( node=uidnode->next; node; node = node->next ) {
@@ -1310,12 +1190,6 @@ make_sig_records( KBNODE keyblock, KBNODE uidnode,
 
 	siglid = find_or_create_lid( sig );
 	/* smash dups */
-	/* FIXME: Here we have a problem:
-	 *  We can't distinguish between a certification and a certification
-	 *  revocation without looking at class of the signature - we have
-	 *  to see how we can store the sigclass in the sigrecord..
-	 *  Argg- I hope I can get rid of this ugly trustdb ASAP.
-	 */
 	for( s2 = s; s2 ; s2 = s2->next ) {
 	    for(i=0; i < sigidx; i++ ) {
 		if( s2->r.sig.sig[i].lid == siglid )
@@ -1344,8 +1218,7 @@ make_sig_records( KBNODE keyblock, KBNODE uidnode,
 	s->r.sig.sig[sigidx].lid = siglid;
 	s->r.sig.sig[sigidx].flag= check_sig_record( keyblock, node,
 						     siglid, sigidx,
-						     mainkid, lid, &expiretime,
-						     mod_down, mod_up );
+						     mainkid, lid, &expire );
 
 	sigidx++;
 	if( sigidx == SIGS_PER_RECORD ) {
@@ -1355,8 +1228,8 @@ make_sig_records( KBNODE keyblock, KBNODE uidnode,
 	    sigidx = 0;
 	}
 	/* keep track of signers pk expire time */
-	if( expiretime && (!*min_expire || *min_expire > expiretime ) )
-	    *min_expire = expiretime;
+	if( expire && (!*min_expire || *min_expire > expire ) )
+	    *min_expire = expire;
     }
     if( sigidx ) {
        s->recnum = tdbio_new_recnum();
@@ -1378,10 +1251,75 @@ make_sig_records( KBNODE keyblock, KBNODE uidnode,
 
 
 
+/****************
+ * Make a preference record (or a list of them) according to the supplied
+ * signature.
+ * Returns: The record number of the first pref record.
+ */
+static ulong
+make_pref_record( PKT_signature *sig, ulong lid )
+{
+    static struct {
+	sigsubpkttype_t subpkttype;
+	int preftype;
+    } ptable[] = {
+	{ SIGSUBPKT_PREF_SYM,	PREFTYPE_SYM	},
+	{ SIGSUBPKT_PREF_HASH,	PREFTYPE_HASH	},
+	{ SIGSUBPKT_PREF_COMPR, PREFTYPE_COMPR	},
+	{ 0, 0 }
+    };
+    TRUSTREC *precs, **p_end, *p=NULL, *p2;
+    ulong precno;
+    int k, idx=0;
+    const byte *s;
+    size_t n;
+
+  #if (ITEMS_PER_PREF_RECORD % 2) != 0
+    #error ITEMS_PER_PREF_RECORD must have an even value
+  #endif
+
+    precs = NULL; p_end = &precs;
+    for(k=0; ptable[k].subpkttype; k++ ) {
+	s = parse_sig_subpkt2( sig, ptable[k].subpkttype, &n );
+	if( !s )
+	    continue;
+	for( ; n; n--, s++ ) {
+	    if( !idx ) {
+		p = m_alloc_clear( sizeof *p );
+		p->rectype = RECTYPE_PREF;
+		p->r.pref.lid = lid;
+	    }
+	    p->r.pref.data[idx++] = ptable[k].preftype;
+	    p->r.pref.data[idx++] = *s;
+	    if( idx >= ITEMS_PER_PREF_RECORD ) {
+		p->recnum = tdbio_new_recnum();
+		*p_end = p;
+		p_end = &p->next;
+		idx = 0;
+	    }
+	}
+    }
+    if( idx ) {
+       p->recnum = tdbio_new_recnum();
+       *p_end = p;
+       p_end = &p->next;
+    }
+
+    precno = precs? precs->recnum : 0;
+    /* write the precs and release the memory */
+    for( p = precs; p ; p = p2 ) {
+	if( p->next )
+	    p->r.pref.next = p->next->recnum;
+	write_record( p );
+	p2 = p->next;
+	m_free( p );
+    }
+    return precno;
+}
+
 
 static ulong
-make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire,
-						 int *mod_down, int *mod_up )
+make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire )
 {
     TRUSTREC *urecs, **uend, *u, *u2;
     KBNODE  node;
@@ -1396,10 +1334,7 @@ make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire,
 	if( node->pkt->pkttype != PKT_USER_ID )
 	    continue;
 	uid = node->pkt->pkt.user_id;
-	if( uid->photo )
-	    rmd160_hash_buffer( uidhash, uid->photo, uid->photolen );
-	else
-	    rmd160_hash_buffer( uidhash, uid->name, uid->len );
+	rmd160_hash_buffer( uidhash, uid->name, uid->len );
 
 	/* create the uid record */
 	u = m_alloc_clear( sizeof *u );
@@ -1414,23 +1349,11 @@ make_uid_records( KBNODE keyblock, ulong lid, u32 *keyid, u32 *min_expire,
 						     lid, &bestsig );
 	if( (u->r.uid.uidflags & UIDF_CHECKED)
 	    && (u->r.uid.uidflags & UIDF_VALID) ) {
-	    u->r.uid.prefrec = 0;
+	    u->r.uid.prefrec = bestsig? make_pref_record( bestsig, lid ) : 0;
 	}
-
-	/* the next test is really bad because we should modify
-	 * out modification timestamps only if we really have a change.
-	 * But because we are deleting the uid records first it is somewhat
-	 * difficult to track those changes.  fixme */
-	if(   !( u->r.uid.uidflags & UIDF_VALID )
-	    || ( u->r.uid.uidflags & UIDF_REVOKED ) )
-	    *mod_down=1;
-	else
-	    *mod_up=1;
-
 	/* create the list of signatures */
 	u->r.uid.siglist = make_sig_records( keyblock, node,
-					     lid, keyid, min_expire,
-					     mod_down, mod_up );
+					     lid, keyid, min_expire );
     }
 
     uidrecno = urecs? urecs->recnum : 0;
@@ -1457,8 +1380,6 @@ update_trust_record( KBNODE keyblock, int recheck, int *modified )
     TRUSTREC drec;
     int rc;
 
-    /* NOTE: We don't need recheck anymore, but this might chnage again in
-     * the future */
     if( opt.dry_run )
 	return 0;
     if( modified )
@@ -1469,27 +1390,26 @@ update_trust_record( KBNODE keyblock, int recheck, int *modified )
     if( rc )
 	return rc;
 
-    rc = do_update_trust_record( keyblock, &drec, 0, modified );
+    rc = do_update_trust_record( keyblock, &drec, recheck, modified );
     return rc;
 }
 
 /****************
- * Same as update_trust_record, but this functions expects the dir record.
- * On exit the dir record will reflect any changes made.
- * With sigs_only set only foreign key signatures are checked.
+ * Same as update_trust_record, but tghis functions expects the dir record.
+ * On exit the dirrecord will reflect any changes made.
  */
 static int
 do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
-			int sigs_only, int *modified )
+			int recheck, int *modified )
 {
     PKT_public_key *primary_pk;
     TRUSTREC krec, urec, prec, helprec;
     int i, rc = 0;
     u32 keyid[2]; /* keyid of primary key */
-    int mod_up = 0;
-    int mod_down = 0;
+/*    int mod_up = 0;
+    int mod_down = 0; */
     ulong recno, r2;
-    u32 expiretime;
+    u32 expire;
 
     primary_pk = find_kbnode( keyblock, PKT_PUBLIC_KEY )->pkt->pkt.public_key;
     if( !primary_pk->local_id )
@@ -1504,7 +1424,7 @@ do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
     if( rc )
 	return rc;
 
-    /* delete the old stuff FIXME: implementend sigs_only */
+    /* delete the old stuff */
     for( recno=drec->r.dir.keylist; recno; recno = krec.r.key.next ) {
 	read_record( recno, &krec, RECTYPE_KEY );
 	delete_record( recno );
@@ -1513,8 +1433,6 @@ do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
     for( recno=drec->r.dir.uidlist; recno; recno = urec.r.uid.next ) {
 	read_record( recno, &urec, RECTYPE_UID );
 	for(r2=urec.r.uid.prefrec ; r2; r2 = prec.r.pref.next ) {
-            /* we don't use preference records any more, but all ones might
-             * still be there */
 	    read_record( r2, &prec, RECTYPE_PREF );
 	    delete_record( r2 );
 	}
@@ -1529,13 +1447,22 @@ do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
 
     /* insert new stuff */
     drec->r.dir.dirflags &= ~DIRF_REVOKED;
-    drec->r.dir.dirflags &= ~DIRF_NEWKEYS;
     drec->r.dir.keylist = make_key_records( keyblock, drec->recnum, keyid, &i );
     if( i ) /* primary key has been revoked */
-	drec->r.dir.dirflags |= DIRF_REVOKED;
-    expiretime = 0;
+	drec->r.dir.dirflags &= DIRF_REVOKED;
+    expire = 0;
     drec->r.dir.uidlist = make_uid_records( keyblock, drec->recnum, keyid,
-					    &expiretime, &mod_down, &mod_up );
+								  &expire );
+    #if 0
+	if( orig_uidflags != urec.r.uid.uidflags ) {
+	    write_record( &urec );
+	    if(   !( urec.r.uid.uidflags & UIDF_VALID )
+		|| ( urec.r.uid.uidflags & UIDF_REVOKED ) )
+		*mod_down=1;
+	    else
+		*mod_up=1; /*(maybe a new user id)*/
+    #endif
+
     if( rc )
 	rc = tdbio_cancel_transaction();
     else {
@@ -1543,9 +1470,9 @@ do_update_trust_record( KBNODE keyblock, TRUSTREC *drec,
 	    *modified = 1;
 	drec->r.dir.dirflags |= DIRF_CHECKED;
 	drec->r.dir.valcheck = 0;
-	drec->r.dir.checkat = expiretime;
+	drec->r.dir.checkat = expire;
 	write_record( drec );
-	tdbio_write_modify_stamp( mod_up, mod_down );
+	/*tdbio_write_modify_stamp( mod_up, mod_down );*/
 	rc = tdbio_end_transaction();
     }
     return rc;
@@ -1610,28 +1537,16 @@ insert_trust_record( KBNODE keyblock )
 	}
     }
 
-
     /* mark tdb as modified upwards */
     tdbio_write_modify_stamp( 1, 0 );
 
     /* and put all the other stuff into the keydb */
-    rc = do_update_trust_record( keyblock, &dirrec, 0, NULL );
+    rc = do_update_trust_record( keyblock, &dirrec, 1, NULL );
 
     do_sync();
-
-    /* keep track of new keys */
-    if( !fresh_imported_keys )
-	fresh_imported_keys = new_lid_table();
-    ins_lid_table_item( fresh_imported_keys, pk->local_id, 0 );
-    if( ++fresh_imported_keys_count > FRESH_KEY_CHECK_THRESHOLD )
-	mark_fresh_keys();
-
     return rc;
 }
 
-
-
-
 /****************
  * Insert a trust record indentified by a PK into the TrustDB
  */
@@ -1669,7 +1584,7 @@ insert_trust_record_by_pk( PKT_public_key *pk )
  * Currently we only do an update_trust_record.
  */
 static int
-check_trust_record( TRUSTREC *drec, int sigs_only )
+check_trust_record( TRUSTREC *drec )
 {
     KBNODE keyblock;
     int modified, rc;
@@ -1681,7 +1596,7 @@ check_trust_record( TRUSTREC *drec, int sigs_only )
 	return rc;
     }
 
-    rc = do_update_trust_record( keyblock, drec, sigs_only, &modified );
+    rc = do_update_trust_record( keyblock, drec, 0, &modified );
     release_kbnode( keyblock );
 
     return rc;
@@ -1697,28 +1612,22 @@ void
 update_trustdb()
 {
     KBNODE keyblock = NULL;
-    KEYDB_HANDLE kdbhd;
+    KBPOS kbpos;
     int rc;
 
     if( opt.dry_run )
 	return;
 
     init_trustdb();
-    kdbhd = keydb_new (0);
-    rc = keydb_search_first (kdbhd);
-    if (!rc) {
+    rc = enum_keyblocks( 0, &kbpos, &keyblock );
+    if( !rc ) {
 	ulong count=0, err_count=0, new_count=0;
 
-	do {
-	    TRUSTREC drec;
-	    PKT_public_key *pk;
+	while( !(rc = enum_keyblocks( 1, &kbpos, &keyblock )) ) {
 	    /*int modified;*/
-
-            rc = keydb_get_keyblock (kdbhd, &keyblock );
-            if (rc)
-                break;
-
-	    pk = find_kbnode (keyblock, PKT_PUBLIC_KEY)->pkt->pkt.public_key;
+	    TRUSTREC drec;
+	    PKT_public_key *pk = find_kbnode( keyblock, PKT_PUBLIC_KEY )
+					->pkt->pkt.public_key;
 
 	    rc = get_dir_record( pk, &drec );
 	    if( rc == -1 ) { /* not in trustdb: insert */
@@ -1747,7 +1656,7 @@ update_trustdb()
 	    release_kbnode( keyblock ); keyblock = NULL;
 	    if( !(++count % 100) )
 		log_info(_("%lu keys so far processed\n"), count);
-	} while ( !(rc = keydb_search_next (kdbhd)));
+	}
 	log_info(_("%lu keys processed\n"), count);
 	if( err_count )
 	    log_info(_("\t%lu keys with errors\n"), err_count);
@@ -1757,14 +1666,14 @@ update_trustdb()
     if( rc && rc != -1 )
 	log_error(_("enumerate keyblocks failed: %s\n"), g10_errstr(rc));
 
-    keydb_release (kdbhd);
+    enum_keyblocks( 2, &kbpos, &keyblock ); /* close */
     release_kbnode( keyblock );
 }
 
 
 
 /****************
- * Do all required checks in the trustdb.  This function walks over all
+ * Do all required check in the trustdb.  This function walks over all
  * records in the trustdb and does scheduled processing.
  */
 void
@@ -1772,7 +1681,7 @@ check_trustdb( const char *username )
 {
     TRUSTREC rec;
     ulong recnum;
-    ulong count=0, upd_count=0, err_count=0, skip_count=0, sigonly_count=0;
+    ulong count=0, upd_count=0, err_count=0, skip_count=0;
     ulong current_time = make_timestamp();
 
     if( username )
@@ -1781,25 +1690,15 @@ check_trustdb( const char *username )
     init_trustdb();
 
     for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ ) {
-	int sigs_only;
-
 	if( rec.rectype != RECTYPE_DIR )
 	    continue; /* we only want the dir records */
 
 	if( count && !(count % 100) && !opt.quiet )
 	    log_info(_("%lu keys so far processed\n"), count);
 	count++;
-	sigs_only = 0;
-
-	if( !(rec.r.dir.dirflags & DIRF_CHECKED) )
-	    ;
-	else if( !rec.r.dir.checkat || rec.r.dir.checkat > current_time ) {
-	    if( !(rec.r.dir.dirflags & DIRF_NEWKEYS) ) {
-		skip_count++;
-		continue;  /* not scheduled for checking */
-	    }
-	    sigs_only = 1; /* new public keys - check them */
-	    sigonly_count++;
+	if( !rec.r.dir.checkat || rec.r.dir.checkat > current_time ) {
+	    skip_count++;
+	    continue;  /* not scheduled for checking */
 	}
 
 	if( !rec.r.dir.keylist ) {
@@ -1808,12 +1707,11 @@ check_trustdb( const char *username )
 	    continue;
 	}
 
-	check_trust_record( &rec, sigs_only );
+	check_trust_record( &rec );
+
     }
 
     log_info(_("%lu keys processed\n"), count);
-    if( sigonly_count )
-	log_info(_("\t%lu due to new pubkeys\n"), sigonly_count);
     if( skip_count )
 	log_info(_("\t%lu keys skipped\n"), skip_count);
     if( err_count )
@@ -1873,12 +1771,8 @@ build_cert_tree( ulong lid, int depth, int max_depth, TN helproot )
 	return NULL;
     }
 
-    if( dirrec.r.dir.checkat && dirrec.r.dir.checkat <= make_timestamp() ) {
-	check_trust_record( &dirrec, 0 );
-    }
-    else if( (dirrec.r.dir.dirflags & DIRF_NEWKEYS) ) {
-	check_trust_record( &dirrec, 1 );
-    }
+    if( dirrec.r.dir.checkat && dirrec.r.dir.checkat <= make_timestamp() )
+	check_trust_record( &dirrec );
 
     keynode->n.k.ownertrust = dirrec.r.dir.ownertrust & TRUST_MASK;
 
@@ -2029,10 +1923,10 @@ propagate_validity( TN root, TN node, int (*add_fnc)(ulong), unsigned *retflgs )
     }
 
     /* loop over all user ids */
-    for( ur=node->list; ur && max_validity <= TRUST_FULLY; ur = ur->next ) {
+    for( ur=node->list; ur && max_validity < TRUST_FULLY; ur = ur->next ) {
 	assert( ur->is_uid );
 	/* loop over all signators */
-	for(kr=ur->list; kr && max_validity <= TRUST_FULLY; kr = kr->next ) {
+	for(kr=ur->list; kr && max_validity < TRUST_FULLY; kr = kr->next ) {
 	    if( propagate_validity( root, kr, add_fnc, retflgs ) )
 		return -1; /* quit */
 	    if( kr->n.k.validity == TRUST_ULTIMATE ) {
@@ -2106,12 +2000,8 @@ verify_key( int max_depth, TRUSTREC *drec, const char *namehash,
     if( !tree )
 	return TRUST_UNDEFINED;
     pv_result = propagate_validity( tree, tree, add_fnc, retflgs );
-    if( namehash && tree->n.k.validity != TRUST_ULTIMATE ) {
+    if( namehash ) {
 	/* find the matching user id.
-	 * We don't do this here if the key is ultimately trusted; in
-	 * this case there will be no lids for the user IDs and frankly
-	 * it does not make sense to compare by the name if we do
-	 * have the secret key.
 	 * fixme: the way we handle this is too inefficient */
 	TN ur;
 	TRUSTREC rec;
@@ -2184,7 +2074,6 @@ do_check( TRUSTREC *dr, unsigned *validity,
     }
     else if( !add_fnc
 	&& tdbio_db_matches_options()
-	    /* FIXME, TODO: This comparision is WRONG ! */
 	&& dr->r.dir.valcheck
 	    > tdbio_read_modify_stamp( (dr->r.dir.validity < TRUST_FULLY) )
 	&& dr->r.dir.validity )
@@ -2307,11 +2196,11 @@ query_trust_record( PKT_public_key *pk )
  *	     is not necessary to check this if we use a local pubring. Hmmmm.
  */
 int
-check_trust( PKT_public_key *pk, int *r_trustlevel,
+check_trust( PKT_public_key *pk, unsigned *r_trustlevel,
 	     const byte *namehash, int (*add_fnc)(ulong), unsigned *retflgs )
 {
     TRUSTREC rec;
-    unsigned int trustlevel = TRUST_UNKNOWN;
+    unsigned trustlevel = TRUST_UNKNOWN;
     int rc=0;
     u32 cur_time;
     u32 keyid[2];
@@ -2350,16 +2239,10 @@ check_trust( PKT_public_key *pk, int *r_trustlevel,
 	log_info(_("key %08lX.%lu: created in future "
 		   "(time warp or clock problem)\n"),
 					  (ulong)keyid[1], pk->local_id );
-	if( !opt.ignore_time_conflict )
-	    return G10ERR_TIME_CONFLICT;
+	return G10ERR_TIME_CONFLICT;
     }
-
-    if( !(rec.r.dir.dirflags & DIRF_CHECKED) )
-	check_trust_record( &rec, 0 );
-    else if( rec.r.dir.checkat && rec.r.dir.checkat <= cur_time )
-	check_trust_record( &rec, 0 );
-    else if( (rec.r.dir.dirflags & DIRF_NEWKEYS) )
-	check_trust_record( &rec, 1 );
+    if( rec.r.dir.checkat && rec.r.dir.checkat <= cur_time )
+	check_trust_record( &rec );
 
     if( pk->expiredate && pk->expiredate <= cur_time ) {
 	log_info(_("key %08lX.%lu: expired at %s\n"),
@@ -2415,55 +2298,10 @@ check_trust( PKT_public_key *pk, int *r_trustlevel,
 }
 
 
-/****************
- * scan the whole trustdb and mark all signature records whose keys
- * are freshly imported.
- */
-static void
-mark_fresh_keys()
-{
-    TRUSTREC dirrec, rec;
-    ulong recnum, lid;
-    int i;
-
-    memset( &dirrec, 0, sizeof dirrec );
-
-    for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ ) {
-	if( rec.rectype != RECTYPE_SIG )
-	    continue;
-	/* if we have already have the dir record, we can check it now */
-	if( dirrec.recnum == rec.r.sig.lid
-	    && (dirrec.r.dir.dirflags & DIRF_NEWKEYS) )
-	    continue; /* flag is already set */
-
-	for(i=0; i < SIGS_PER_RECORD; i++ ) {
-	    if( !(lid=rec.r.sig.sig[i].lid) )
-		continue; /* skip deleted sigs */
-	    if( !(rec.r.sig.sig[i].flag & SIGF_CHECKED) )
-		continue; /* skip checked signatures */
-	    if( qry_lid_table_flag( fresh_imported_keys, lid, NULL ) )
-		continue; /* not in the list of new keys */
-	    read_record( rec.r.sig.lid, &dirrec, RECTYPE_DIR );
-	    if( !(dirrec.r.dir.dirflags & DIRF_NEWKEYS) ) {
-		dirrec.r.dir.dirflags |= DIRF_NEWKEYS;
-		write_record( &dirrec );
-	    }
-	    break;
-	}
-    }
-
-    do_sync();
-
-    clear_lid_table( fresh_imported_keys );
-    fresh_imported_keys_count = 0;
-}
-
-
-
 int
 query_trust_info( PKT_public_key *pk, const byte *namehash )
 {
-    int trustlevel;
+    unsigned trustlevel;
     int c;
 
     init_trustdb();
@@ -2520,7 +2358,7 @@ list_trust_path( const char *username )
     PKT_public_key *pk = m_alloc_clear( sizeof *pk );
 
     init_trustdb();
-    if( (rc = get_pubkey_byname (pk, username, NULL, NULL )) )
+    if( (rc = get_pubkey_byname(NULL, pk, username, NULL )) )
 	log_error(_("user '%s' not found: %s\n"), username, g10_errstr(rc) );
     else if( (rc=tdbio_search_dir_bypk( pk, &rec )) && rc != -1 )
 	log_error(_("problem finding '%s' in trustdb: %s\n"),
@@ -2690,3 +2528,69 @@ enum_cert_paths_print( void **context, FILE *fp,
 }
 
 
+/*
+ * Return an allocated buffer with the preference values for
+ * the key with LID and the userid which is identified by the
+ * HAMEHASH or the firstone if namehash is NULL.  ret_n receives
+ * the length of the allocated buffer.	Structure of the buffer is
+ * a repeated sequences of 2 bytes; where the first byte describes the
+ * type of the preference and the second one the value.  The constants
+ * PREFTYPE_xxxx should be used to reference a type.
+ */
+byte *
+get_pref_data( ulong lid, const byte *namehash, size_t *ret_n )
+{
+    TRUSTREC rec;
+    ulong recno;
+
+    init_trustdb();
+    read_record( lid, &rec, RECTYPE_DIR );
+    for( recno=rec.r.dir.uidlist; recno; recno = rec.r.uid.next ) {
+	read_record( recno, &rec, RECTYPE_UID );
+	if( rec.r.uid.prefrec
+	    && ( !namehash || !memcmp(namehash, rec.r.uid.namehash, 20) ))  {
+	    byte *buf;
+	    /* found the correct one or the first one */
+	    read_record( rec.r.uid.prefrec, &rec, RECTYPE_PREF );
+	    if( rec.r.pref.next )
+		log_info(_("WARNING: can't yet handle long pref records\n"));
+	    buf = m_alloc( ITEMS_PER_PREF_RECORD );
+	    memcpy( buf, rec.r.pref.data, ITEMS_PER_PREF_RECORD );
+	    *ret_n = ITEMS_PER_PREF_RECORD;
+	    return buf;
+	}
+    }
+    return NULL;
+}
+
+
+
+/****************
+ * Check whether the algorithm is in one of the pref records
+ */
+int
+is_algo_in_prefs( ulong lid, int preftype, int algo )
+{
+    TRUSTREC rec;
+    ulong recno;
+    int i;
+    byte *pref;
+
+    init_trustdb();
+    read_record( lid, &rec, RECTYPE_DIR );
+    for( recno=rec.r.dir.uidlist; recno; recno = rec.r.uid.next ) {
+	read_record( recno, &rec, RECTYPE_UID );
+	if( rec.r.uid.prefrec ) {
+	    read_record( rec.r.uid.prefrec, &rec, RECTYPE_PREF );
+	    if( rec.r.pref.next )
+		log_info(_("WARNING: can't yet handle long pref records\n"));
+	    pref = rec.r.pref.data;
+	    for(i=0; i+1 < ITEMS_PER_PREF_RECORD; i+=2 ) {
+		if( pref[i] == preftype && pref[i+1] == algo )
+		    return 1;
+	    }
+	}
+    }
+    return 0;
+}
+
