@@ -1,14 +1,18 @@
-/* g10lib.h -  GNU digital encryption libray interface
+/* g10lib.h -  internal defintions for libgcrypt
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This header is to be used inside of libgcrypt in place of gcrypt.h.
+ * This way we can easily distinguish between internal and external
+ * usage of gcrypt.h
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * This file is part of GnuPG.
+ *
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -18,19 +22,34 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef _g10lib_G10LIB_H
-#define _g10lib_G10LIB_H
-#ifdef __cplusplus
-extern "C" {
+#ifndef G10LIB_H
+#define G10LIB_H 1
+
+#ifdef _GCRYPT_H
+  #error  gcrypt.h already included
 #endif
 
+#include <gcrypt.h>
 
-
-#ifndef _g10lib_INTERNAL
-struct g10mpi_struct { int hidden_stuff; };
-typedef struct g10mpi_struct *MPI;
+#ifdef G10_I18N_H
+  #error i18n should not be included here
 #endif
 
+#define _(a)  g10_gettext(a)
+#define N_(a) (a)
+
+/*-- gcrypt/global.c --*/
+int set_lasterr( int ec );
+
+/*-- gcrypt/misc.c --*/
+const char *g10_gettext( const char *key );
+int fatal_invalid_arg(const char *text);
+
+#if 0
+/* This used to be the old include/g10lib.h */
+
+#include "mpi.h"
+
 int g10c_debug_mode;
 int g10_opt_verbose;
 
@@ -187,8 +206,7 @@ void g10_log_mpidump( const char *text, MPI a );
 #define DBG_CIPHER  g10c_debug_mode
 #define OPT_VERBOSE g10_opt_verbose
 
+#endif /* if 0 */
 
-#ifdef __cplusplus
-}
-#endif
-#endif /* _g10lib_G10LIB_H */
+
+#endif /* G10LIB_H */
