@@ -1,6 +1,5 @@
 /* mpi.h  -  Multi Precision Integers
- *	Copyright (C) 1994, 1996, 1998, 1999,
- *                    2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1994, 1996, 1998 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -30,40 +29,19 @@
 #ifndef G10_MPI_H
 #define G10_MPI_H
 
-#include <config.h>
 #include <stdio.h>
 #include "iobuf.h"
 #include "types.h"
 #include "memory.h"
 
-#if BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_INT
-  typedef unsigned int mpi_limb_t;
-  typedef   signed int mpi_limb_signed_t;
-#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG
-  typedef unsigned long int mpi_limb_t;
-  typedef   signed long int mpi_limb_signed_t;
-#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_LONG_LONG
-  typedef unsigned long long int mpi_limb_t;
-  typedef   signed long long int mpi_limb_signed_t;
-#elif BYTES_PER_MPI_LIMB == SIZEOF_UNSIGNED_SHORT
-  typedef unsigned short int mpi_limb_t;
-  typedef   signed short int mpi_limb_signed_t;
-#else
-  #error BYTES_PER_MPI_LIMB does not match any C type
-#endif
-#define BITS_PER_MPI_LIMB    (8*BYTES_PER_MPI_LIMB)
-
-#ifndef EXTERN_UNLESS_MAIN_MODULE
- #if defined (__riscos__) && !defined (INCLUDED_BY_MAIN_MODULE)
-  #define EXTERN_UNLESS_MAIN_MODULE extern
- #else
-  #define EXTERN_UNLESS_MAIN_MODULE 
- #endif
-#endif
 
 #define DBG_MPI     mpi_debug_mode
-EXTERN_UNLESS_MAIN_MODULE int mpi_debug_mode;
+int mpi_debug_mode;
 
+#define BITS_PER_MPI_LIMB    (8*SIZEOF_UNSIGNED_LONG)
+#define BYTES_PER_MPI_LIMB   SIZEOF_UNSIGNED_LONG
+typedef unsigned long int mpi_limb_t;
+typedef   signed long int mpi_limb_signed_t;
 
 struct gcry_mpi {
     int alloced;    /* array size (# of allocated limbs) */
@@ -76,7 +54,10 @@ struct gcry_mpi {
     mpi_limb_t *d;  /* array with the limbs */
 };
 
-typedef struct gcry_mpi *MPI;
+#ifndef DID_MPI_TYPEDEF
+  typedef struct gcry_mpi *MPI;
+  #define DID_MPI_TYPEDEF
+#endif
 
 #define MPI_NULL NULL
 
