@@ -1,5 +1,5 @@
 /* util.h
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -20,10 +20,6 @@
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
 
-#if defined (__MINGW32__) || defined (__CYGWIN32__)
-# include <stdarg.h>
-#endif
-
 #include "types.h"
 #include "errors.h"
 #include "types.h"
@@ -69,11 +65,9 @@ void log_set_name( const char *name );
 const char *log_get_name(void);
 void log_set_pid( int pid );
 int  log_get_errorcount( int clear );
-void log_inc_errorcount(void);
 void g10_log_hexdump( const char *text, const char *buf, size_t len );
 
-#if !defined (__riscos__) \
-    && (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 ))
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
   void g10_log_bug( const char *fmt, ... )
 			    __attribute__ ((noreturn, format (printf,1,2)));
   void g10_log_bug0( const char *, int, const char * ) __attribute__ ((noreturn));
@@ -137,11 +131,10 @@ const char *strusage( int level );
 struct dotlock_handle;
 typedef struct dotlock_handle *DOTLOCK;
 
-void disable_dotlock(void);
 DOTLOCK create_dotlock( const char *file_to_lock );
 int make_dotlock( DOTLOCK h, long timeout );
 int release_dotlock( DOTLOCK h );
-void remove_lockfiles (void);
+
 
 /*-- fileutil.c --*/
 char * make_basename(const char *filepath);
@@ -150,7 +143,6 @@ char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
 const char *print_fname_stdin( const char *s );
 const char *print_fname_stdout( const char *s );
-int  is_file_compressed(const char *s, int *r_status);
 
 
 /*-- miscutil.c --*/
@@ -162,9 +154,7 @@ const char *strtimestamp( u32 stamp ); /* GMT */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, const byte *p, size_t n, int delim );
 void  print_utf8_string( FILE *fp, const byte *p, size_t n );
-void  print_utf8_string2( FILE *fp, const byte *p, size_t n, int delim);
 char *make_printable_string( const byte *p, size_t n, int delim );
-int answer_is_yes_no_default( const char *s, int def_answer );
 int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
 
@@ -178,29 +168,17 @@ STRLIST append_to_strlist2( STRLIST *list, const char *string, int is_utf8 );
 STRLIST strlist_prev( STRLIST head, STRLIST node );
 STRLIST strlist_last( STRLIST node );
 const char *memistr( const char *buf, size_t buflen, const char *sub );
-const char *ascii_memistr( const char *buf, size_t buflen, const char *sub );
 char *mem2str( char *, const void *, size_t);
 char *trim_spaces( char *string );
-unsigned int trim_trailing_chars( byte *line, unsigned int len,
-                                  const char *trimchars);
-unsigned int trim_trailing_ws( byte *line, unsigned len );
-unsigned int check_trailing_chars( const byte *line, unsigned int len,
-                                   const char *trimchars );
-unsigned int check_trailing_ws( const byte *line, unsigned int len );
+unsigned trim_trailing_chars( byte *line, unsigned len, const char *trimchars);
+unsigned trim_trailing_ws( byte *line, unsigned len );
 int string_count_chr( const char *string, int c );
 int set_native_charset( const char *newset );
 const char* get_native_charset(void);
 char *native_to_utf8( const char *string );
-char *utf8_to_native( const char *string, size_t length, int delim);
+char *utf8_to_native( const char *string, size_t length );
 int  check_utf8_string( const char *string );
 
-int ascii_isupper (int c);
-int ascii_islower (int c);
-int ascii_toupper (int c);
-int ascii_tolower (int c);
-int ascii_strcasecmp( const char *a, const char *b );
-int ascii_memcasecmp( const char *a, const char *b, size_t n );
-
 #ifndef HAVE_MEMICMP
 int memicmp( const char *a, const char *b, size_t n );
 #endif
@@ -210,12 +188,6 @@ char *stpcpy(char *a,const char *b);
 #ifndef HAVE_STRLWR
 char *strlwr(char *a);
 #endif
-#ifndef HAVE_STRSEP
-char *strsep (char **stringp, const char *delim);
-#endif
-#ifndef HAVE_STRCASECMP
-int strcasecmp( const char *, const char *b);
-#endif
 #ifndef HAVE_STRTOUL
   #define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
 #endif
@@ -226,17 +198,6 @@ int strcasecmp( const char *, const char *b);
   #define stricmp(a,b)	 strcasecmp( (a), (b) )
 #endif
 
-#if defined (__MINGW32__) || defined (__CYGWIN32__)
-/*-- w32reg.c --*/
-char *read_w32_registry_string( const char *root,
-				const char *dir, const char *name );
-int write_w32_registry_string(const char *root, const char *dir,
-                              const char *name, const char *value);
-
-/*-- strgutil.c --*/
-int vasprintf ( char **result, const char *format, va_list args);
-#endif
-
 /**** other missing stuff ****/
 #ifndef HAVE_ATEXIT  /* For SunOS */
   #define atexit(a)    (on_exit((a),0))
@@ -254,26 +215,4 @@ int vasprintf ( char **result, const char *format, va_list args);
 #define DIM(v) (sizeof(v)/sizeof((v)[0]))
 #define DIMof(type,member)   DIM(((type *)0)->member)
 
-/******* RISC OS stuff ***********/
-#ifdef __riscos__
-pid_t riscos_getpid(void);
-int riscos_kill(pid_t pid, int sig);
-int riscos_access(const char *path, int amode);
-int fdopenfile(const char *filename, const int allow_write);
-void close_fds(void);
-int renamefile(const char *old, const char *new);
-char *gstrans(const char *old);
-void not_implemented(const char *feature);
-#ifdef DEBUG
-void dump_fdlist(void);
-void list_openfiles(void);
-#endif
-#ifndef __RISCOS__C__
-  #define getpid riscos_getpid
-  #define kill(a,b) riscos_kill((a),(b))
-  #define access(a,b) riscos_access((a),(b))
-#endif /* !__RISCOS__C__ */
-#endif /* __riscos__ */
-
 #endif /*G10_UTIL_H*/
-
