@@ -1,5 +1,5 @@
 /* iobuf.c  -  file handling
- *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,89 +23,21 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <ctype.h>
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h> 
 #include <unistd.h>
-#ifdef HAVE_DOSISH_SYSTEM
- #include <windows.h>
-#endif
-#ifdef __riscos__
-#include <kernel.h>
-#include <swis.h>
-#endif /* __riscos__ */
 
 #include "memory.h"
 #include "util.h"
 #include "iobuf.h"
 
-#undef FILE_FILTER_USES_STDIO
-
-#ifdef HAVE_DOSISH_SYSTEM
- #define USE_SETMODE 1
-#endif
-
-#ifdef FILE_FILTER_USES_STDIO
- #define my_fileno(a)  fileno ((a))
- #define my_fopen_ro(a,b) fopen ((a),(b))
- #define my_fopen(a,b)    fopen ((a),(b))
- typedef FILE *FILEP_OR_FD;
- #define INVALID_FP    NULL
- #define FILEP_OR_FD_FOR_STDIN  (stdin)
- #define FILEP_OR_FD_FOR_STDOUT  (stdout)
- typedef struct {
-     FILE *fp;	   /* open file handle */
-     int keep_open;
-     int no_cache;
-     int  print_only_name; /* flags indicating that fname is not a real file*/
-     char fname[1]; /* name of the file */
- } file_filter_ctx_t ;
-#else
- #define my_fileno(a)  (a)
- #define my_fopen_ro(a,b) fd_cache_open ((a),(b)) 
- #define my_fopen(a,b) direct_open ((a),(b)) 
- #ifdef HAVE_DOSISH_SYSTEM
-   typedef HANDLE FILEP_OR_FD;
-   #define INVALID_FP  ((HANDLE)-1)
-   #define FILEP_OR_FD_FOR_STDIN  (GetStdHandle (STD_INPUT_HANDLE))
-   #define FILEP_OR_FD_FOR_STDOUT (GetStdHandle (STD_OUTPUT_HANDLE))
-   #undef USE_SETMODE
- #else
-   typedef int FILEP_OR_FD;
-   #define INVALID_FP  (-1)
-   #define FILEP_OR_FD_FOR_STDIN  (0)
-   #define FILEP_OR_FD_FOR_STDOUT (1)
- #endif
- typedef struct {
-     FILEP_OR_FD  fp;	   /* open file handle */
-     int keep_open;
-     int no_cache;
-     int eof_seen;
-     int  print_only_name; /* flags indicating that fname is not a real file*/
-     char fname[1]; /* name of the file */
- } file_filter_ctx_t ;
-
- struct close_cache_s { 
-    struct close_cache_s *next;
-    FILEP_OR_FD fp;
-    char fname[1];
- };
- typedef struct close_cache_s *CLOSE_CACHE;
- static CLOSE_CACHE close_cache;
-#endif
-
-#ifdef __MINGW32__
 typedef struct {
-    int sock;
-    int keep_open;
-    int no_cache;
-    int eof_seen;
+    FILE *fp;	   /* open file handle */
     int  print_only_name; /* flags indicating that fname is not a real file*/
     char fname[1]; /* name of the file */
-} sock_filter_ctx_t ;
-#endif /*__MINGW32__*/
+} file_filter_ctx_t ;
+
 
 /* The first partial length header block must be of size 512
  * to make it easier (and efficienter) we use a min. block size of 512
@@ -124,185 +56,8 @@ typedef struct {
     int eof;
 } block_filter_ctx_t;
 
-static int special_names_enabled;
 
 static int underflow(IOBUF a);
-static int translate_file_handle ( int fd, int for_write );
-
-#ifndef FILE_FILTER_USES_STDIO
-
-/*
- * Invalidate (i.e. close) a cached iobuf
- */
-static void
-fd_cache_invalidate (const char *fname)
-{
-    CLOSE_CACHE cc;
-
-    assert (fname);
-    if( DBG_IOBUF )
-        log_debug ("fd_cache_invalidate (%s)\n", fname);
-
-    for (cc=close_cache; cc; cc = cc->next ) {
-        if ( cc->fp != INVALID_FP && !strcmp (cc->fname, fname) ) {
-            if( DBG_IOBUF )
-                log_debug ("                did (%s)\n", cc->fname);
-          #ifdef HAVE_DOSISH_SYSTEM
-            CloseHandle (cc->fp);
-          #else
-	    close(cc->fp);
-	  #endif
-            cc->fp = INVALID_FP;
-        }
-    }
-}
-
-
-
-static FILEP_OR_FD
-direct_open (const char *fname, const char *mode)
-{
-#ifdef HAVE_DOSISH_SYSTEM
-    unsigned long da, cd, sm;
-    HANDLE hfile;
-
-    /* Note, that we do not handle all mode combinations */
-
-    /* According to the ReactOS source it seems that open() of the
-     * standard MSW32 crt does open the file in share mode which is
-     * something new for MS applications ;-)
-     */
-    if ( strchr (mode, '+') ) {
-        fd_cache_invalidate (fname);
-        da = GENERIC_READ|GENERIC_WRITE;
-        cd = OPEN_EXISTING;
-        sm = FILE_SHARE_READ | FILE_SHARE_WRITE;
-    }
-    else if ( strchr (mode, 'w') ) {
-        fd_cache_invalidate (fname);
-        da = GENERIC_WRITE;
-        cd = CREATE_ALWAYS;
-        sm = FILE_SHARE_WRITE;
-    }
-    else {
-        da = GENERIC_READ;
-        cd = OPEN_EXISTING;
-        sm = FILE_SHARE_READ;
-    }
-
-    hfile = CreateFile (fname, da, sm, NULL, cd, FILE_ATTRIBUTE_NORMAL, NULL);
-    return hfile;
-#else
-    int oflag;
-    int cflag = S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
-
-    /* Note, that we do not handle all mode combinations */
-    if ( strchr (mode, '+') ) {
-        fd_cache_invalidate (fname);
-        oflag = O_RDWR;
-    }
-    else if ( strchr (mode, 'w') ) {
-        fd_cache_invalidate (fname);
-        oflag = O_WRONLY | O_CREAT | O_TRUNC;
-    }
-    else {
-        oflag = O_RDONLY;
-    }
-#ifndef __riscos__
-    return open (fname, oflag, cflag );
-#else
-    {
-        struct stat buf;
-        int rc = stat( fname, &buf );
-        
-        /* Don't allow iobufs on directories */
-        if( !rc && S_ISDIR(buf.st_mode) && !S_ISREG(buf.st_mode) )
-            return __set_errno( EISDIR );
-        else
-            return open( fname, oflag, cflag );
-    }
-#endif
-#endif
-}
-
-
-/*
- * Instead of closing an FD we keep it open and cache it for later reuse 
- * Note that this caching strategy only works if the process does not chdir.
- */
-static void
-fd_cache_close (const char *fname, FILEP_OR_FD fp)
-{
-    CLOSE_CACHE cc;
-
-    assert (fp);
-    if ( !fname || !*fname ) {
-#ifdef HAVE_DOSISH_SYSTEM
-        CloseHandle (fp);
-#else
-        close(fp);
-#endif
-        if( DBG_IOBUF )
-            log_debug ("fd_cache_close (%p) real\n", (void*)fp);
-        return;
-    }
-    /* try to reuse a slot */
-    for (cc=close_cache; cc; cc = cc->next ) {
-        if ( cc->fp == INVALID_FP && !strcmp (cc->fname, fname) ) {
-            cc->fp = fp;
-            if( DBG_IOBUF )
-                log_debug ("fd_cache_close (%s) used existing slot\n", fname);
-            return;
-        }
-    }
-    /* add a new one */
-    if( DBG_IOBUF )
-        log_debug ("fd_cache_close (%s) new slot created\n", fname);
-    cc = m_alloc_clear (sizeof *cc + strlen (fname));
-    strcpy (cc->fname, fname);
-    cc->fp = fp;
-    cc->next = close_cache;
-    close_cache = cc;
-}
-
-/*
- * Do an direct_open on FNAME but first try to reuse one from the fd_cache
- */
-static FILEP_OR_FD
-fd_cache_open (const char *fname, const char *mode)
-{
-    CLOSE_CACHE cc;
-
-    assert (fname);
-    for (cc=close_cache; cc; cc = cc->next ) {
-        if ( cc->fp != INVALID_FP && !strcmp (cc->fname, fname) ) {
-            FILEP_OR_FD fp = cc->fp;
-            cc->fp = INVALID_FP;
-            if( DBG_IOBUF )
-                log_debug ("fd_cache_open (%s) using cached fp\n", fname);
-          #ifdef HAVE_DOSISH_SYSTEM
-            if (SetFilePointer (fp, 0, NULL, FILE_BEGIN) == 0xffffffff ) {
-                log_error ("rewind file failed on handle %p: ec=%d\n",
-                           fp, (int)GetLastError () );
-                fp = INVALID_FP;
-            }
-          #else
-            if ( lseek (fp, 0, SEEK_SET) == (off_t)-1 ) {
-                log_error("can't rewind fd %d: %s\n", fp, strerror(errno) );
-                fp = INVALID_FP;
-            }
-          #endif
-            return fp;
-        }
-    }
-    if( DBG_IOBUF )
-        log_debug ("fd_cache_open (%s) not cached\n", fname);
-    return direct_open (fname, mode);
-}
-
-
-#endif /*FILE_FILTER_USES_STDIO*/
-
 
 /****************
  * Read data from a file into buf which has an allocated length of *LEN.
@@ -325,32 +80,28 @@ fd_cache_open (const char *fname, const char *mode)
  * IOBUFCTRL_FLUSH: called by iobuf_flush() to write out the collected stuff.
  *		    *RET_LAN is the number of bytes in BUF.
  *
- * IOBUFCTRL_CANCEL: send to all filters on behalf of iobuf_cancel.  The
- *		    filter may take appropriate action on this message.
  */
 static int
 file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 {
     file_filter_ctx_t *a = opaque;
-    FILEP_OR_FD f = a->fp;
+    FILE *fp = a->fp;
     size_t size = *ret_len;
     size_t nbytes = 0;
     int rc = 0;
 
-#ifdef FILE_FILTER_USES_STDIO
     if( control == IOBUFCTRL_UNDERFLOW ) {
 	assert( size ); /* need a buffer */
-	if ( feof(f)) {	/* On terminals you could easiely read as many EOFs as you call 	*/
+	if ( feof(fp)) {	/* On terminals you could easiely read as many EOFs as you call 	*/
 	    rc = -1;		/* fread() or fgetc() repeatly. Every call will block until you press	*/
 	    *ret_len = 0;	/* CTRL-D. So we catch this case before we call fread() again.		*/
 	}
 	else {
-	    clearerr( f );
-	    nbytes = fread( buf, 1, size, f );
-	    if( feof(f) && !nbytes ) {
+	    clearerr( fp );
+	    nbytes = fread( buf, 1, size, fp );
+	    if( feof(fp) && !nbytes )
 		rc = -1; /* okay: we can return EOF now. */
-            }
-	    else if( ferror(f) && errno != EPIPE  ) {
+	    else if( ferror(fp) && errno != EPIPE  ) {
 		log_error("%s: read error: %s\n",
 			  a->fname, strerror(errno));
 		rc = G10ERR_READ_FILE;
@@ -360,9 +111,9 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
     }
     else if( control == IOBUFCTRL_FLUSH ) {
 	if( size ) {
-	    clearerr( f );
-	    nbytes = fwrite( buf, 1, size, f );
-	    if( ferror(f) ) {
+	    clearerr( fp );
+	    nbytes = fwrite( buf, 1, size, fp );
+	    if( ferror(fp) ) {
 		log_error("%s: write error: %s\n", a->fname, strerror(errno));
 		rc = G10ERR_WRITE_FILE;
 	    }
@@ -370,220 +121,23 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 	*ret_len = nbytes;
     }
     else if( control == IOBUFCTRL_INIT ) {
-        a->keep_open = a->no_cache = 0;
     }
     else if( control == IOBUFCTRL_DESC ) {
 	*(char**)buf = "file_filter";
     }
     else if( control == IOBUFCTRL_FREE ) {
-	if( f != stdin && f != stdout ) {
+	if( fp != stdin && fp != stdout ) {
 	    if( DBG_IOBUF )
-		log_debug("%s: close fd %d\n", a->fname, fileno(f) );
-            if (!a->keep_open)
-                fclose(f);
+		log_debug("%s: close fd %d\n", a->fname, fileno(fp) );
+	    fclose(fp);
 	}
-	f = NULL;
+	fp = NULL;
 	m_free(a); /* we can free our context now */
     }
-#else /* !stdio implementation */
 
-    if( control == IOBUFCTRL_UNDERFLOW ) {
-	assert( size ); /* need a buffer */
-	if ( a->eof_seen) {
-	    rc = -1;		
-	    *ret_len = 0;	
-	}
-	else {
-          #ifdef HAVE_DOSISH_SYSTEM
-            unsigned long nread;
-
-            nbytes = 0;
-            if ( !ReadFile ( f, buf, size, &nread, NULL ) ) {
-                int ec = (int)GetLastError ();
-                if ( ec != ERROR_BROKEN_PIPE ) {
-                    log_error("%s: read error: ec=%d\n", a->fname, ec);
-                    rc = G10ERR_READ_FILE;
-                }
-            }
-            else if ( !nread ) {
-                a->eof_seen = 1;
-                rc = -1;
-            }
-            else {
-                nbytes = nread;
-            }
-
-          #else
-
-            int n;
-
-            nbytes = 0;
-            do {
-                n = read ( f, buf, size );
-            } while (n == -1 && errno == EINTR );
-            if ( n == -1 ) { /* error */
-                if (errno != EPIPE) {
-                    log_error("%s: read error: %s\n",
-                              a->fname, strerror(errno));
-                    rc = G10ERR_READ_FILE;
-                }
-            }
-            else if ( !n ) { /* eof */
-                a->eof_seen = 1;
-                rc = -1;
-            }
-            else {
-                nbytes = n;
-            }
-          #endif
-	    *ret_len = nbytes;
-	}
-    }
-    else if( control == IOBUFCTRL_FLUSH ) {
-	if( size ) {
-          #ifdef HAVE_DOSISH_SYSTEM
-            byte *p = buf;
-            unsigned long n;
-
-            nbytes = size;
-            do {
-                if ( size && !WriteFile ( f,  p, nbytes, &n, NULL) ) {
-                    int ec = (int)GetLastError ();
-                    log_error("%s: write error: ec=%d\n", a->fname, ec);
-                    rc = G10ERR_WRITE_FILE;
-                    break;
-                }
-                p += n;
-                nbytes -= n;
-            } while ( nbytes );
-            nbytes = p - buf;
-          #else
-            byte *p = buf;
-            int n;
-
-            nbytes = size;
-            do {
-                do {
-                    n = write ( f, p, nbytes );
-                } while ( n == -1 && errno == EINTR );
-                if ( n > 0 ) {
-                    p += n;
-                    nbytes -= n;
-                }
-            } while ( n != -1 && nbytes );
-	    if( n == -1 ) {
-		log_error("%s: write error: %s\n", a->fname, strerror(errno));
-		rc = G10ERR_WRITE_FILE;
-	    }
-            nbytes = p - buf;
-          #endif
-	}
-	*ret_len = nbytes;
-    }
-    else if ( control == IOBUFCTRL_INIT ) {
-        a->eof_seen = 0;
-        a->keep_open = 0;
-        a->no_cache = 0;
-    }
-    else if ( control == IOBUFCTRL_DESC ) {
-	*(char**)buf = "file_filter(fd)";
-    }
-    else if ( control == IOBUFCTRL_FREE ) {
-      #ifdef HAVE_DOSISH_SYSTEM
-        if ( f != FILEP_OR_FD_FOR_STDIN && f != FILEP_OR_FD_FOR_STDOUT ) {
-	    if( DBG_IOBUF )
-		log_debug("%s: close handle %p\n", a->fname, f );
-            if (!a->keep_open)
-                fd_cache_close (a->no_cache?NULL:a->fname, f);
-        }
-      #else
-	if ( (int)f != 0 && (int)f != 1 ) {
-	    if( DBG_IOBUF )
-		log_debug("%s: close fd %d\n", a->fname, f );
-            if (!a->keep_open)
-                fd_cache_close (a->no_cache?NULL:a->fname, f);
-	}
-	f = INVALID_FP;
-      #endif
-	m_free (a); /* we can free our context now */
-    }
-#endif /* !stdio implementation */
     return rc;
 }
 
-#ifdef __MINGW32__
-/* Becuase sockets are an special object under Lose32 we have to
- * use a special filter */
-static int
-sock_filter (void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
-{
-    sock_filter_ctx_t *a = opaque;
-    size_t size = *ret_len;
-    size_t nbytes = 0;
-    int rc = 0;
-
-    if( control == IOBUFCTRL_UNDERFLOW ) {
-	assert( size ); /* need a buffer */
-	if ( a->eof_seen) {
-	    rc = -1;		
-	    *ret_len = 0;	
-	}
-	else {
-            int nread;
-
-            nread = recv ( a->sock, buf, size, 0 );
-            if ( nread == SOCKET_ERROR ) {
-                int ec = (int)WSAGetLastError ();
-                log_error("socket read error: ec=%d\n", ec);
-                rc = G10ERR_READ_FILE;
-            }
-            else if ( !nread ) {
-                a->eof_seen = 1;
-                rc = -1;
-            }
-            else {
-                nbytes = nread;
-            }
-	    *ret_len = nbytes;
-	}
-    }
-    else if( control == IOBUFCTRL_FLUSH ) {
-	if( size ) {
-            byte *p = buf;
-            int n;
-
-            nbytes = size;
-            do {
-                n = send (a->sock, p, nbytes, 0);
-                if ( n == SOCKET_ERROR ) {
-                    int ec = (int)WSAGetLastError ();
-                    log_error("socket write error: ec=%d\n", ec);
-                    rc = G10ERR_WRITE_FILE;
-                    break;
-                }
-                p += n;
-                nbytes -= n;
-            } while ( nbytes );
-            nbytes = p - buf;
-	}
-	*ret_len = nbytes;
-    }
-    else if ( control == IOBUFCTRL_INIT ) {
-        a->eof_seen = 0;
-        a->keep_open = 0;
-        a->no_cache = 0;
-    }
-    else if ( control == IOBUFCTRL_DESC ) {
-	*(char**)buf = "sock_filter";
-    }
-    else if ( control == IOBUFCTRL_FREE ) {
-        if (!a->keep_open)
-            closesocket (a->sock);
-	m_free (a); /* we can free our context now */
-    }
-    return rc;
-}
-#endif /*__MINGW32__*/
 
 /****************
  * This is used to implement the block write mode.
@@ -906,11 +460,12 @@ iobuf_alloc(int use, size_t bufsize)
     return a;
 }
 
+
 int
-iobuf_close ( IOBUF a )
+iobuf_close( IOBUF a )
 {
     IOBUF a2;
-    size_t dummy_len=0;
+    size_t dummy_len;
     int rc=0;
 
     if( a && a->directfp ) {
@@ -932,10 +487,7 @@ iobuf_close ( IOBUF a )
 					 a->chain, NULL, &dummy_len)) )
 	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
 	m_free(a->real_fname);
-        if (a->d.buf) {
-            memset (a->d.buf, 0, a->d.size); /* erase the buffer */
-            m_free(a->d.buf);
-        }
+	m_free(a->d.buf);
 	m_free(a);
     }
     return rc;
@@ -945,41 +497,13 @@ int
 iobuf_cancel( IOBUF a )
 {
     const char *s;
-    IOBUF a2;
-    int rc;
-  #if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
-    char *remove_name = NULL;
-  #endif
 
     if( a && a->use == 2 ) {
 	s = iobuf_get_real_fname(a);
-	if( s && *s ) {
-	  #if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
-	    remove_name = m_strdup ( s );
-	  #else
-	    remove(s);
-	  #endif
-	}
-    }
-
-    /* send a cancel message to all filters */
-    for( a2 = a; a2 ; a2 = a2->chain ) {
-	size_t dummy;
-	if( a2->filter )
-	    a2->filter( a2->filter_ov, IOBUFCTRL_CANCEL, a2->chain,
-							 NULL, &dummy );
-    }
-
-    rc = iobuf_close(a);
-  #if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
-    if ( remove_name ) {
-	/* Argg, MSDOS does not allow to remove open files.  So
-	 * we have to do it here */
-	remove ( remove_name );
-	m_free ( remove_name );
-    }
-  #endif
-    return rc;
+	if( s && *s )
+	    remove(s);	/* remove the file. Fixme: this will fail for MSDOZE*/
+    }			/* because the file is still open */
+    return iobuf_close(a);
 }
 
 
@@ -1010,32 +534,6 @@ iobuf_temp_with_content( const char *buffer, size_t length )
     return a;
 }
 
-void
-iobuf_enable_special_filenames ( int yes )
-{
-    special_names_enabled = yes;
-}
-
-/*
- * see whether the filename has the for "-&nnnn", where n is a
- * non-zero number.
- * Returns this number or -1 if it is not the case.
- */
-static int
-check_special_filename ( const char *fname )
-{
-    if ( special_names_enabled
-         && fname && *fname == '-' && fname[1] == '&' ) {
-        int i;
-
-        fname += 2;
-        for (i=0; isdigit (fname[i]); i++ )
-            ;
-        if ( !fname[i] ) 
-            return atoi (fname);
-    }
-    return -1;
-}
 
 /****************
  * Create a head iobuf for reading from a file
@@ -1045,23 +543,17 @@ IOBUF
 iobuf_open( const char *fname )
 {
     IOBUF a;
-    FILEP_OR_FD fp;
+    FILE *fp;
     file_filter_ctx_t *fcx;
     size_t len;
     int print_only = 0;
-    int fd;
 
     if( !fname || (*fname=='-' && !fname[1])  ) {
-	fp = FILEP_OR_FD_FOR_STDIN;
-      #ifdef USE_SETMODE
-	setmode ( my_fileno(fp) , O_BINARY );
-      #endif
+	fp = stdin; /* fixme: set binary mode for msdoze */
 	fname = "[stdin]";
 	print_only = 1;
     }
-    else if ( (fd = check_special_filename ( fname )) != -1 )
-        return iobuf_fdopen ( translate_file_handle (fd,0), "rb" );
-    else if( (fp = my_fopen_ro(fname, "rb")) == INVALID_FP )
+    else if( !(fp = fopen(fname, "rb")) )
 	return NULL;
     a = iobuf_alloc(1, 8192 );
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
@@ -1076,7 +568,7 @@ iobuf_open( const char *fname )
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
 	log_debug("iobuf-%d.%d: open `%s' fd=%d\n",
-		   a->no, a->subno, fname, (int)my_fileno(fcx->fp) );
+		   a->no, a->subno, fname, fileno(fcx->fp) );
 
     return a;
 }
@@ -1089,16 +581,12 @@ IOBUF
 iobuf_fdopen( int fd, const char *mode )
 {
     IOBUF a;
-    FILEP_OR_FD fp;
+    FILE *fp;
     file_filter_ctx_t *fcx;
     size_t len;
 
-#ifdef FILE_FILTER_USES_STDIO
     if( !(fp = fdopen(fd, mode)) )
 	return NULL;
-#else
-    fp = (FILEP_OR_FD)fd;
-#endif
     a = iobuf_alloc( strchr( mode, 'w')? 2:1, 8192 );
     fcx = m_alloc( sizeof *fcx + 20 );
     fcx->fp = fp;
@@ -1110,34 +598,7 @@ iobuf_fdopen( int fd, const char *mode )
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
 	log_debug("iobuf-%d.%d: fdopen `%s'\n", a->no, a->subno, fcx->fname );
-    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */
-    return a;
-}
 
-
-IOBUF
-iobuf_sockopen ( int fd, const char *mode )
-{
-    IOBUF a;
-#ifdef __MINGW32__
-    sock_filter_ctx_t *scx;
-    size_t len;
-
-    a = iobuf_alloc( strchr( mode, 'w')? 2:1, 8192 );
-    scx = m_alloc( sizeof *scx + 25 );
-    scx->sock = fd;
-    scx->print_only_name = 1;
-    sprintf(scx->fname, "[sock %d]", fd );
-    a->filter = sock_filter;
-    a->filter_ov = scx;
-    sock_filter( scx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
-    sock_filter( scx, IOBUFCTRL_INIT, NULL, NULL, &len );
-    if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: sockopen `%s'\n", a->no, a->subno, scx->fname);
-    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */ 
-#else
-    a = iobuf_fdopen (fd, mode);
-#endif
     return a;
 }
 
@@ -1148,23 +609,17 @@ IOBUF
 iobuf_create( const char *fname )
 {
     IOBUF a;
-    FILEP_OR_FD fp;
+    FILE *fp;
     file_filter_ctx_t *fcx;
     size_t len;
     int print_only = 0;
-    int fd;
 
     if( !fname || (*fname=='-' && !fname[1]) ) {
-	fp = FILEP_OR_FD_FOR_STDOUT;
-      #ifdef USE_SETMODE
-	setmode ( my_fileno(fp) , O_BINARY );
-      #endif
+	fp = stdout;
 	fname = "[stdout]";
 	print_only = 1;
     }
-    else if ( (fd = check_special_filename ( fname )) != -1 )
-        return iobuf_fdopen ( translate_file_handle (fd, 1), "wb" );
-    else if( (fp = my_fopen(fname, "wb")) == INVALID_FP )
+    else if( !(fp = fopen(fname, "wb")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
@@ -1186,9 +641,7 @@ iobuf_create( const char *fname )
 /****************
  * append to an iobuf; if the file does not exist, create it.
  * cannot be used for stdout.
- * Note: This is not used.
  */
-#if 0 /* not used */
 IOBUF
 iobuf_append( const char *fname )
 {
@@ -1199,7 +652,7 @@ iobuf_append( const char *fname )
 
     if( !fname )
 	return NULL;
-    else if( !(fp = my_fopen(fname, "ab")) )
+    else if( !(fp = fopen(fname, "ab")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
@@ -1215,19 +668,18 @@ iobuf_append( const char *fname )
 
     return a;
 }
-#endif
 
 IOBUF
 iobuf_openrw( const char *fname )
 {
     IOBUF a;
-    FILEP_OR_FD fp;
+    FILE *fp;
     file_filter_ctx_t *fcx;
     size_t len;
 
     if( !fname )
 	return NULL;
-    else if( (fp = my_fopen(fname, "r+b")) == INVALID_FP )
+    else if( !(fp = fopen(fname, "r+b")) )
 	return NULL;
     a = iobuf_alloc(2, 8192 );
     fcx = m_alloc( sizeof *fcx + strlen(fname) );
@@ -1245,61 +697,38 @@ iobuf_openrw( const char *fname )
 }
 
 
-int
-iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
-{
-    if ( cmd == 1 ) {  /* keep system filepointer/descriptor open */
-        if( DBG_IOBUF )
-            log_debug("iobuf-%d.%d: ioctl `%s' keep=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
-        for( ; a; a = a->chain )
-            if( !a->chain && a->filter == file_filter ) {
-                file_filter_ctx_t *b = a->filter_ov;
-                b->keep_open = intval;
-                return 0;
-            }
-          #ifdef __MINGW32__
-            else if( !a->chain && a->filter == sock_filter ) {
-                sock_filter_ctx_t *b = a->filter_ov;
-                b->keep_open = intval;
-                return 0;
-            }
-          #endif
-    }
-    else if ( cmd == 2 ) {  /* invalidate cache */
-        if( DBG_IOBUF )
-            log_debug("iobuf-*.*: ioctl `%s' invalidate\n",
-                      ptrval? (char*)ptrval:"?");
-        if ( !a && !intval && ptrval ) {
-          #ifndef FILE_FILTER_USES_STDIO
-            fd_cache_invalidate (ptrval);
-          #endif
-            return 0;
-        }
-    }
-    else if ( cmd == 3 ) {  /* disallow/allow caching */
-        if( DBG_IOBUF )
-            log_debug("iobuf-%d.%d: ioctl `%s' no_cache=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
-        for( ; a; a = a->chain )
-            if( !a->chain && a->filter == file_filter ) {
-                file_filter_ctx_t *b = a->filter_ov;
-                b->no_cache = intval;
-                return 0;
-            }
-          #ifdef __MINGW32__
-            else if( !a->chain && a->filter == sock_filter ) {
-                sock_filter_ctx_t *b = a->filter_ov;
-                b->no_cache = intval;
-                return 0;
-            }
-          #endif
-    }
 
-    return -1;
+/****************
+ * You can overwrite the normal iobuf behaviour by using this function.
+ * If used the iobuf is a simple wrapper around stdio.
+ * NULL if an error occures and sets errno
+ */
+IOBUF
+iobuf_fopen( const char *fname, const char *mode )
+{
+    IOBUF a;
+    FILE *fp;
+    int print_only = 0;
+
+    if( !fname || (*fname=='-' && !fname[1])  ) {
+	fp = stdin; /* fixme: set binary mode for msdoze */
+	fname = "[stdin]";
+	print_only = 1;
+    }
+    else if( !(fp = fopen(fname, mode) ) )
+	return NULL;
+    a = iobuf_alloc(1, 8192 );
+    a->directfp = fp;
+    a->real_fname = m_strdup( fname );
+
+    if( DBG_IOBUF )
+	log_debug("iobuf_fopen -> %p\n", a->directfp );
+
+    return a;
 }
 
 
+
 /****************
  * Register an i/o filter.
  */
@@ -1526,7 +955,7 @@ underflow(IOBUF a)
 
 	}
 	if( a->use == 1 && rc == -1 ) { /* EOF: we can remove the filter */
-	    size_t dummy_len=0;
+	    size_t dummy_len;
 
 	    /* and tell the filter to free itself */
 	    if( (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE, a->chain,
@@ -1583,6 +1012,7 @@ iobuf_flush(IOBUF a)
     if( a->directfp )
 	return 0;
 
+    /*log_debug("iobuf-%d.%d: flush\n", a->no, a->subno );*/
     if( a->use == 3 ) { /* increase the temp buffer */
 	char *newbuf;
 	size_t newsize = a->d.size + 8192;
@@ -1820,7 +1250,7 @@ iobuf_flush_temp( IOBUF temp )
  * Setting the limit to 0 disables this feature.
  */
 void
-iobuf_set_limit( IOBUF a, off_t nlimit )
+iobuf_set_limit( IOBUF a, unsigned long nlimit )
 {
     if( nlimit )
 	a->nofast |= 1;
@@ -1836,7 +1266,7 @@ iobuf_set_limit( IOBUF a, off_t nlimit )
 /****************
  * Return the length of an open file
  */
-off_t
+u32
 iobuf_get_filelength( IOBUF a )
 {
     struct stat st;
@@ -1844,8 +1274,8 @@ iobuf_get_filelength( IOBUF a )
     if( a->directfp )  {
 	FILE *fp = a->directfp;
 
-       if( !fstat(fileno(fp), &st) )
-           return st.st_size;
+	if( !fstat(fileno(fp), &st) )
+	    return st.st_size;
 	log_error("fstat() failed: %s\n", strerror(errno) );
 	return 0;
     }
@@ -1854,20 +1284,11 @@ iobuf_get_filelength( IOBUF a )
     for( ; a; a = a->chain )
 	if( !a->chain && a->filter == file_filter ) {
 	    file_filter_ctx_t *b = a->filter_ov;
-	    FILEP_OR_FD fp = b->fp;
+	    FILE *fp = b->fp;
 
-          #if defined(HAVE_DOSISH_SYSTEM) && !defined(FILE_FILTER_USES_STDIO)
-            ulong size;
-
-            if ( (size=GetFileSize (fp, NULL)) != 0xffffffff ) 
-                return size;
-            log_error ("GetFileSize for handle %p failed: ec=%d\n",
-                       fp, (int)GetLastError () );
-          #else
-            if( !fstat(my_fileno(fp), &st) )
+	    if( !fstat(fileno(fp), &st) )
 		return st.st_size;
 	    log_error("fstat() failed: %s\n", strerror(errno) );
-          #endif
 	    break;
 	}
 
@@ -1877,55 +1298,27 @@ iobuf_get_filelength( IOBUF a )
 /****************
  * Tell the file position, where the next read will take place
  */
-off_t
+ulong
 iobuf_tell( IOBUF a )
 {
     return a->ntotal + a->nbytes;
 }
 
 
-#if !defined(HAVE_FSEEKO) && !defined(fseeko)
-
-#ifdef HAVE_LIMITS_H
-# include <limits.h>
-#endif
-#ifndef LONG_MAX
-# define LONG_MAX ((long) ((unsigned long) -1 >> 1))
-#endif
-#ifndef LONG_MIN
-# define LONG_MIN (-1 - LONG_MAX)
-#endif
-
-/****************
- * A substitute for fseeko, for hosts that don't have it.
- */
-static int
-fseeko( FILE *stream, off_t newpos, int whence )
-{
-    while( newpos != (long) newpos ) {
-       long pos = newpos < 0 ? LONG_MIN : LONG_MAX;
-       if( fseek( stream, pos, whence ) != 0 )
-           return -1;
-       newpos -= pos;
-       whence = SEEK_CUR;
-    }
-    return fseek( stream, (long)newpos, whence );
-}
-#endif
 
 /****************
  * This is a very limited implementation. It simply discards all internal
  * buffering and removes all filters but the first one.
  */
 int
-iobuf_seek( IOBUF a, off_t newpos )
+iobuf_seek( IOBUF a, ulong newpos )
 {
     file_filter_ctx_t *b = NULL;
 
     if( a->directfp ) {
 	FILE *fp = a->directfp;
-        if( fseeko( fp, newpos, SEEK_SET ) ) {
-            log_error("can't seek: %s\n", strerror(errno) );
+	if( fseek( fp, newpos, SEEK_SET ) ) {
+	    log_error("can't seek to %lu: %s\n", newpos, strerror(errno) );
 	    return -1;
 	}
 	clearerr(fp);
@@ -1939,25 +1332,10 @@ iobuf_seek( IOBUF a, off_t newpos )
 	}
 	if( !a )
 	    return -1;
-#ifdef FILE_FILTER_USES_STDIO
-       if( fseeko( b->fp, newpos, SEEK_SET ) ) {
-           log_error("can't fseek: %s\n", strerror(errno) );
-           return -1;
-       }
-#else
-    #ifdef HAVE_DOSISH_SYSTEM
-       if (SetFilePointer (b->fp, newpos, NULL, FILE_BEGIN) == 0xffffffff ) {
-           log_error ("SetFilePointer failed on handle %p: ec=%d\n",
-                      b->fp, (int)GetLastError () );
-           return -1;
-       }
-    #else
-       if ( lseek (b->fp, newpos, SEEK_SET) == (off_t)-1 ) {
-           log_error("can't lseek: %s\n", strerror(errno) );
-           return -1;
-       }
-    #endif
-#endif
+	if( fseek( b->fp, newpos, SEEK_SET ) ) {
+	    log_error("can't seek to %lu: %s\n", newpos, strerror(errno) );
+	    return -1;
+	}
     }
     a->d.len = 0;   /* discard buffer */
     a->d.start = 0;
@@ -2136,58 +1514,3 @@ iobuf_read_line( IOBUF a, byte **addr_of_buffer,
     return nbytes;
 }
 
-/* This is the non iobuf specific function */
-int
-iobuf_translate_file_handle ( int fd, int for_write )
-{
-  #ifdef __MINGW32__
-    {
-        int x;
-            
-        if  ( fd <= 2 )
-            return fd; /* do not do this for error, stdin, stdout, stderr */
-
-        x = _open_osfhandle ( fd, for_write? 1:0 );
-        if (x==-1 )
-            log_error ("failed to translate osfhandle %p\n", (void*)fd );
-        else {
-            /*log_info ("_open_osfhandle %p yields %d%s\n",
-              (void*)fd, x, for_write? " for writing":"" );*/
-            fd = x;
-        }
-    }
-  #endif
-    return fd;
-}
-
-static int
-translate_file_handle ( int fd, int for_write )
-{
-  #ifdef __MINGW32__
-   #ifdef FILE_FILTER_USES_STDIO  
-    fd = iobuf_translate_file_handle (fd, for_write);
-   #else
-    {
-        int x;
-
-        if  ( fd == 0 ) 
-            x = (int)GetStdHandle (STD_INPUT_HANDLE);
-        else if (fd == 1)    
-            x = (int)GetStdHandle (STD_OUTPUT_HANDLE);
-        else if (fd == 2)    
-            x = (int)GetStdHandle (STD_ERROR_HANDLE);
-        else
-            x = fd;
-
-        if (x == -1)
-            log_debug ("GetStdHandle(%d) failed: ec=%d\n",
-                       fd, (int)GetLastError () );
-
-        fd = x;
-    }
-   #endif
-  #endif
-    return fd;
-}
-
-
