@@ -1,5 +1,5 @@
 /* simple-gettext.c  - a simplified version of gettext.
- * Copyright (C) 1995, 1996, 1997, 1999 Free Software Foundation, Inc.
+ * Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -19,7 +19,7 @@
  */
 
 /* This is a simplified version of gettext written by Ulrich Drepper.
- * It is used for the Win32 version of GnuPG beucase all the overhead
+ * It is used for the Win32 version of GnuPG becuase all the overhead
  * of gettext is not needed and we have to do some special Win32 stuff.
  * I decided that this is far easier than to tweak gettext for the special
  * cases (I tried it but it is a lot of code).	wk 15.09.99
@@ -27,8 +27,8 @@
 
 #include <config.h>
 #ifdef USE_SIMPLE_GETTEXT
-#if !defined (__MINGW32__) && !defined (__CYGWIN32__)
-  #error This file can only be used with MingW32 or Cygwin32
+#ifndef __MINGW32__
+  #error This file can only be used with MinGW32
 #endif
 
 #include <stdio.h>
@@ -217,6 +217,13 @@ load_domain( const char *filename )
 	free( domain );
 	return NULL;
     }
+    /* Currently we have only a Latin-1 to IBM850 translation, so
+     * we simply mark everything mapped if we don't have this codepage. */
+    {
+	const char *s = get_native_charset();
+	if( !s || strcmp(s, "ibm850")
+	    memset( domain->mapped, 1, domain->nstrings );
+    }
 
     return domain;
 }
@@ -244,19 +251,16 @@ set_gettext_file( const char *filename )
 	    /* absolute path - use it as is */
 	    domain = load_domain( filename );
 	}
-	else { /* relative path - append ".mo" and get dir from the environment */
+	else { /* relative path - append ".mo" and get DIR from env */
 	    char *buf = NULL;
-	    char *dir;
+	    const char *s;
 
-	    dir = read_w32_registry_string( NULL,
-					    "Control Panel\\Mingw32\\NLS",
-					    "MODir" );
-	    if( dir && (buf=malloc(strlen(dir)+strlen(filename)+1+3+1)) ) {
-		strcpy(stpcpy(stpcpy(stpcpy( buf, dir),"/"), filename),".mo");
+	    s = getenv("MINGW32_NLS_DIR");
+	    if( s && (buf=malloc(strlen(s)+strlen(filename)+1+3+1)) ) {
+		strcpy(stpcpy(stpcpy(stpcpy( buf, s),"/"), filename),".mo");
 		domain = load_domain( buf );
 		free(buf);
 	    }
-	    free(dir);
 	}
 	if( !domain )
 	    return -1;
@@ -282,7 +286,7 @@ get_string( struct loaded_domain *domain, u32 idx )
 	byte *pp;
 
 	domain->mapped[idx] = 1;
-	/* we assume Latin1 -> CP 850 for now */
+	/* currently we only support Latin-1 to CP 850 */
 	for( pp=p; *pp; pp++ ) {
 	    if( (*pp & 0x80) ) {
 		switch( *pp ) {
